/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * Segment2D.h - definicao das classes Segment2D e GraphicSegment2D.
 *
 *               link flag -lm
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    07.01.92  13:40 	Welson R. Jacometti  	Arquivo criado
 *
 */

#ifndef _SEGMENT2D_DEFINED
#define _SEGMENT2D_DEFINED

#include <iostream.h>
#include <stdio.h>
#include "global.h"
#include "World.h"
#include "Obj.h"
#include "Point2D.h"

// forward declaration
class GraphicSegment2D;
class Segment2D;

typedef Segment2D* Segment2D_ptr;
typedef GraphicSegment2D* GraphicSegment2D_ptr;

class Segment2D : public Obj {
  public:
    // construtores
    Segment2D(char *s = "Segment2D", ObjType t = Generic_t, bool v = false) : Obj(s, t, v) { setP1(0, 0); setP2(0, 0); }
    Segment2D(double x1, double y1, double x2, double y2) : Obj("Segment2D") { set(x1, y1, x2, y2); }
    Segment2D(Point2D a, Point2D b) : Obj("Segment2D") { set(a, b); }
    Segment2D(Segment2D &s) : Obj(s.what_is(), s.stype, s.volati) { set(s.P1(), s.P2()); }
    
    //destrutor
    ~Segment2D() {}				
    
    // construtor virtual (simulado) que estabelece o relacionamento
    //
    //                 Segment2D --> GraphicSegment2D
    
    GraphicObj* new_GraphicObj();
    
    // funcao para auto-reproducao de um objeto - necessario para possibilitar 
    // que a interface manipule objetos externos ao geolab (alem disso torna
    // mais homogeneo o tratamento de objetos conhecidos e desconhecidos)

    Obj* reproduct()  {
    	return new Segment2D(*this);
    }
        
    // query & set functions
    Point2D P1()  { return p1; }
    Point2D P2()  { return p2; }

    virtual void setP1(double x, double y) { p1.setXY(x, y); }
    virtual void setP2(double x, double y) { p2.setXY(x, y); }

    virtual void set(double x1, double y1, double x2, double y2) {
    	p1.setXY(x1, y1);
    	p2.setXY(x2, y2);
    }
    
    virtual void set(Point2D a, Point2D b) {
    	p1 = a; p2 = b;
    }
    
    bool is_ray()  {
    	return p1.at_infinity() || p2.at_infinity();
    }
    
    bool is_line()  {
    	return p1.at_infinity() && p2.at_infinity();
    }

    bool equal(Segment2D &s)         { return (p1 == s.p1 && p2 == s.p2) || p1 == s.p2 && p2 == s.p1; }
    bool operator == (Segment2D &s)  { return equal(s); }
    
    // geometric functions

    bool intersect(Segment2D s);
    bool intersect(Point2D a, Point2D b);
    Point2D intersection(Segment2D s);
    Point2D intersection(Point2D a, Point2D b);
    bool point_in (Point2D& p);
    
    // inherited methods redefinition
    void load(FILE *f) { Obj::load(f); p1.load(f); p2.load(f); }
    void save(FILE *f) { Obj::save(f); p1.save(f); p2.save(f); }

    ObjType type() { return Segment2D_t; }
    
    void translate(double to_x, double to_y) {
    	p1.translate(to_x, to_y);
    	p2.translate(to_x, to_y);
    }

    void rotate(double orig_x, double orig_y, double angle) {
	p1.rotate(orig_x, orig_y, angle);
	p2.rotate(orig_x, orig_y, angle);
    }
    
    void print()  { cout << "Segment2D ... P1 (" << P1().X() << "," << P1().Y() << ")" << " to P2 (" << P2().X() << "," << P2().Y() << ")"<< endl; }

    void unfill_handles(World *w, bool show_handle = true) ;
    void draw_handles(World *w, GC gc, bool highlight = false, bool show_handle = true) ;
    void draw(World *w, GC gc) ;

    bool select_test(double x, double y)  {
    	return p1.select_test(x, y) || p2.select_test(x, y);
    }
    
    bool select_test(double x1, double y1, double x2, double y2)  {
    	return p1.select_test(x1, y1, x2, y2) && p2.select_test(x1, y1, x2, y2);
    }

    double distance_to_point(double x, double y) ;

    void translate_handle_at(double x, double y, double to_x, double to_y);

    // Metodos necessarios para "Input Tools"
    virtual bool get_segments_implemented() { return true; }
    virtual list(Obj_ptr) get_segments()  {
        Segment2D_ptr s_ptr = new Segment2D(*this); assert(s_ptr);
	list(Obj_ptr) Objs;
	Objs.push (s_ptr);
	return Objs;
    }

    // Methods for information retrieval.
    virtual bool get_points_implemented() { return true; }
    virtual list(Obj_ptr) get_points() ;
    
  protected:
    Point2D p1, p2; 
};

/*
 *
 * GraphicSegment2D.h
 *
 *
 */

class GraphicSegment2D : public GraphicObj {
  public:
    // construtores
    GraphicSegment2D() : GraphicObj(new Segment2D()) {}
    GraphicSegment2D(Segment2D *s) : GraphicObj(s) {}
    
    //destrutor
    ~GraphicSegment2D() {}				
    
    // inherited methods redefinition
    void int_start(World *, double initial_X, double initial_Y, Int_Mode);
    void int_change(World *, double new_X, double new_Y, Int_Mode);
    void int_end(World *, double final_X, double final_Y, Int_Mode);
    
    void resize_start(World *, double X, double Y, Int_Mode);
    void resize_change(World *, double X, double Y, Int_Mode);
    void resize_end(World *, double X, double Y, Int_Mode);

    bool where_draw_string (double &x, double &y);
};

#endif ~_SEGMENT2D_DEFINED;





