/*
 * Misc.C - algoritmos geometricos genericos e de baixo nivel para auxiliar
 *          no processo de animacao de algoritmos geometricos.
 * 
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    24.04.92  09:30 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "Misc.h"

// includes para animacao

#include <GEOLAB/Circle.h>
#include <GEOLAB/Ray2D.h>

// variaveis para animacao

static GraphicCircle *anim_c;
static GraphicRay2D *anim_r;

#define anim_circle	((Circle *)anim_c->obj)
#define anim_ray	((Ray2D *)anim_r->obj)

void misc_animation_init(World *w) {
  static bool FIRST = true;
  
  if (FIRST) {
    anim_c = new GraphicCircle();
    anim_r = new GraphicRay2D();
    FIRST = false;
  }
  anim_c->set_all(w, FOREGROUND, BACKGROUND, 0, 0, false);
  anim_r->set_all(w, FOREGROUND, BACKGROUND, 0, 0, false);
}
 
/*
 **************************************************************************
 * As funcoes abaixo sao copias fieis daquelas presentes na classe Point2D.
 * Foram reescritas aqui para serem animadas.
 **************************************************************************
 */

list_item it_get_min_Y( list(Point2D_ptr) &Points, World *w, int animation_depth, int reference_depth) {
    list_item it, min = Points.first();

    ANIMATION_BEGIN(w, animation_depth, reference_depth)
      misc_animation_init(w);
      anim_circle->set(Points[min]->X(), Points[min]->Y(), 10);
      anim_c->draw(w, XorMode);
    ANIMATION_END(w, animation_depth, reference_depth)
    
    forall_list_items(it, Points) {

      ANIMATION_BEGIN(w, animation_depth, reference_depth)
        anim_circle->set(Points[it]->X(), Points[it]->Y(), 6);
        anim_c->draw(w, XorMode);
      ANIMATION_END(w, animation_depth, reference_depth)
      
      if (lexico_Ycomp(*Points[it], *Points[min]) == Less) {

        ANIMATION_BEGIN(w, animation_depth, reference_depth)
          frame_stop_mark(w);
          anim_circle->set(Points[min]->X(), Points[min]->Y(), 10);
          anim_c->draw(w, XorMode);
        ANIMATION_END(w, animation_depth, reference_depth)
      	
        min=it;

        ANIMATION_BEGIN(w, animation_depth, reference_depth)
          anim_circle->set(Points[min]->X(), Points[min]->Y(), 10);
          anim_c->draw(w, XorMode);
        ANIMATION_END(w, animation_depth, reference_depth)

      }

      ANIMATION_BEGIN(w, animation_depth, reference_depth)
        frame_stop_mark(w);
        anim_circle->set(Points[it]->X(), Points[it]->Y(), 6);
        anim_c->draw(w, XorMode);
      ANIMATION_END(w, animation_depth, reference_depth)
      
    }

    ANIMATION_BEGIN(w, animation_depth, reference_depth)
      frame_stop_mark(w);
      anim_circle->set(Points[min]->X(), Points[min]->Y(), 10);
      anim_c->draw(w, XorMode);
    ANIMATION_END(w, animation_depth, reference_depth)
    
    return min;
}

list_item it_lower_angle_ccw( list(Point2D_ptr) &Points,  Point2D &ref, World *w, int animation_depth, int reference_depth) {
    list_item i, inext;
					// busca linear
    if (Points.empty())
      i = 0;
    else {
      i = Points.first();
      
      ANIMATION_BEGIN(w, animation_depth, reference_depth)
        misc_animation_init(w);
        anim_ray->set(ref, *Points[i]);
        anim_r->draw(w, XorMode);
        frame_stop_mark(w);
      ANIMATION_END(w, animation_depth, reference_depth)
      
      if (Points.size() > 1) {
        inext = Points.succ(i);
        SideType s;
        do {

          ANIMATION_BEGIN(w, animation_depth, reference_depth)
            anim_circle->set(Points[inext]->X(), Points[inext]->Y(), 6);
            anim_ray->set(ref, *Points[inext]);
            anim_c->draw(w, XorMode);
            anim_r->draw(w, XorMode);
            frame_stop_mark(w);
          ANIMATION_END(w, animation_depth, reference_depth)
      
          s = Points[inext]->which_side(ref, *Points[i]);
    	  if (s == Right || (s == Colinear && ref.e_distance_sqr(*Points[inext]) > ref.e_distance_sqr(*Points[i]))) {
      
            ANIMATION_BEGIN(w, animation_depth, reference_depth)
              anim_ray->set(ref, *Points[i]);
              anim_r->draw(w, XorMode);
            ANIMATION_END(w, animation_depth, reference_depth)

      	    i = inext;

            ANIMATION_BEGIN(w, animation_depth, reference_depth)
              anim_ray->set(ref, *Points[i]);
              anim_r->draw(w, XorMode);
            ANIMATION_END(w, animation_depth, reference_depth)

      	  }
      	  
          ANIMATION_BEGIN(w, animation_depth, reference_depth)
            anim_circle->set(Points[inext]->X(), Points[inext]->Y(), 6);
            anim_ray->set(ref, *Points[inext]);
            anim_c->draw(w, XorMode);
            anim_r->draw(w, XorMode);
            frame_stop_mark(w);
          ANIMATION_END(w, animation_depth, reference_depth)
      	  
          inext = Points.succ(inext);
        } while(inext);
      }

      ANIMATION_BEGIN(w, animation_depth, reference_depth)
        anim_ray->set(ref, *Points[i]);
        anim_r->draw(w, XorMode);
      ANIMATION_END(w, animation_depth, reference_depth)

    }
    
    return i;
}

/*
 **************************************************************************
 * A implementacao do algoritmo para calculo do Winding Number abaixo
 * foi extraida da tese de doutorado de Peter Schorn - XYZ Geobench
 ************************************************************************** 
 */

int winding_number( list(Point2D_ptr) &Points,  Point2D &ref, World *w, int animation_depth, int reference_depth) {
    int wn = 0;
    Point2D_ptr u = Points.head();
    list_item it = Points.first();
  
    ANIMATION_BEGIN(w, animation_depth, reference_depth)
      misc_animation_init(w);
      anim_circle->set(ref.X(), ref.Y(), 10);
      anim_c->draw(w, XorMode);
    ANIMATION_END(w, animation_depth, reference_depth)
      
    do {
      it = Points.cyclic_succ(it);
      Point2D_ptr v = Points[it];

      switch (ref.which_side(*u, *v)) {
        case Left:
          if (u->X() <= ref.X() && ref.X() < v->X())
            wn--;
          break;
        case Right:
          if (v->X() <= ref.X() && ref.X() < u->X())
            wn++;
          break;
      }

      ANIMATION_BEGIN(w, animation_depth, reference_depth)
        char s[20];
        Frame f = (Frame)xv_get(w->get_canvas(), XV_OWNER);
        (void)sprintf(s, "%d", wn);
        xv_set(f, FRAME_RIGHT_FOOTER, s, NULL);
        anim_circle->set(u->X(), u->Y(), 6);
        anim_c->draw(w, XorMode);
        anim_circle->set(v->X(), v->Y(), 6);
        anim_c->draw(w, XorMode);
        anim_ray->set(ref, *u);
        anim_r->draw(w, XorMode);
        frame_stop_mark(w);
        anim_circle->set(u->X(), u->Y(), 6);
        anim_c->draw(w, XorMode);
        anim_circle->set(v->X(), v->Y(), 6);
        anim_c->draw(w, XorMode);
        anim_r->draw(w, XorMode);
      ANIMATION_END(w, animation_depth, reference_depth)

      u = v;
    } while(it != Points.first());
  
    ANIMATION_BEGIN(w, animation_depth, reference_depth)
      anim_circle->set(ref.X(), ref.Y(), 10);
      anim_c->draw(w, XorMode);
    ANIMATION_END(w, animation_depth, reference_depth)
      
    return wn;
}






