/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * Arrow2D.C - Arrow2D & GraphicArrow2D methods.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    07.01.92  13:40 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "Obj.h"
#include "Arrow2D.h"

#define ARROW_WIDTH	4
#define ARROW_HEIGHT	10
#define ARROW_TAIL	6

GraphicObj* Arrow2D::new_GraphicObj() {
    return new GraphicArrow2D(this);
}

void Arrow2D::unfill_handles(World *w, bool show_handle)  {
    if (show_handle)
      p1.unfill_handles(w);
}

void Arrow2D::draw_handles(World *w, GC gc, bool highlight, bool show_handle)  {
    if (show_handle)
      p1.draw_handles(w, gc, highlight);
}

void Arrow2D::draw(World *w, GC gc)  {
    double tam, width, height, relwidth, relheight, reltail;
    Point2D a1, a2, a3, a4, tmp;

    if (!(p2.at_infinity())) {
      tam = p1.e_distance(p2);
      width = p2.X() - p1.X();
      height = p2.Y() - p1.Y();

      if (tam == 0)
        tam = width = height = 1;
        
      relwidth = ARROW_WIDTH / tam;
      relheight = ARROW_HEIGHT / tam;
      reltail = ARROW_TAIL / tam;

      a1.setXY(((1 - relheight) * width) + p1.X(), ((1 - relheight) * height) + p1.Y());
      a2.setXY(((1 - relheight - reltail) * width) - (relwidth * height) + p1.X(), 
               ((1 - relheight - reltail) * height) + (relwidth * width) + p1.Y());
      a3.setXY(p2.X(), p2.Y());
      a4.setXY(((1 - relheight - reltail) * width) + (relwidth * height) + p1.X(), 
               ((1 - relheight - reltail) * height) - (relwidth * width) + p1.Y());

      w->DrawLine(gc, a1.X(), a1.Y(), a2.X(), a2.Y());
      w->DrawLine(gc, a2.X(), a2.Y(), a3.X(), a3.Y());
      w->DrawLine(gc, a3.X(), a3.Y(), a4.X(), a4.Y());
      w->DrawLine(gc, a4.X(), a4.Y(), a1.X(), a1.Y());

      tmp = p2; p2 = a1;    
      Segment2D::draw(w, gc);
      p2 = tmp;
    }
    else
      Segment2D::draw(w, gc);
}



