/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * Circle.C - Circle & GraphicCircle methods.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    07.01.92  13:40 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include <sys/time.h>
#include "Obj.h"
#include "Circle.h"
#include "color.h"

GraphicObj* Circle::new_GraphicObj() {
    return new GraphicCircle(this);
}

// Metodos necessarios para "Input Tools"
    
bool Circle::point_inside(double x, double y)  {
    double dc = c.distance_to_point(x, y);
    return dc <= r;
}

void Circle::bounding_box(double& x1, double& y1, double& x2, double& y2)  {
    set_handles();
    x1=h1.X();
    y1=h1.Y();
    x2=h4.X();
    y2=h4.Y();
}
     

list(Obj_ptr) Circle::get_segments()  {
    list(Obj_ptr) S;
    Segment2D_ptr s_ptr;
    set_handles();
    s_ptr = new Segment2D(h1,h3);S.push (s_ptr);assert(s_ptr);
    s_ptr = new Segment2D(h1,h4);S.push (s_ptr);assert(s_ptr);
    s_ptr = new Segment2D(h2,h3);S.push (s_ptr);assert(s_ptr);
    s_ptr = new Segment2D(h2,h4);S.push (s_ptr);assert(s_ptr);
    return S;   
}
    
void Circle::load(FILE *f) {
    Obj::load(f);
    c.load(f);
    fscanf(f, "%lf\n", &r);
}

void Circle::save(FILE *f) {
    Obj::save(f);
    c.save(f);
    fprintf(f, "%lf\n", r);
}

void Circle::unfill_handles(World *w, bool show_handle)  {
    if (show_handle) {
      set_handles();
      h1.unfill_handles(w);
      h2.unfill_handles(w);
      h3.unfill_handles(w);
      h4.unfill_handles(w);
    }
}

void Circle::draw_handles(World *w, GC gc, bool highlight, bool show_handle)  {
    if (show_handle) {
      set_handles();
      h1.draw_handles(w, gc, highlight);
      h2.draw_handles(w, gc, highlight);
      h3.draw_handles(w, gc, highlight);
      h4.draw_handles(w, gc, highlight);

      // pequenina GAMBI para evitar distorcoes no desenho em 
      // funcao do sistema de coordenadas
    
      int cx = w->xcoord(c.X());
      int cy = w->ycoord(c.Y());
    
      w->DrawLine(gc, w->virtual_xcoord(cx - 2), w->virtual_ycoord(cy), 
                      w->virtual_xcoord(cx + 2), w->virtual_ycoord(cy));
      w->DrawLine(gc, w->virtual_xcoord(cx), w->virtual_ycoord(cy - 2), 
                      w->virtual_xcoord(cx), w->virtual_ycoord(cy + 2));
    }
}

void Circle::draw(World *w, GC gc)  {
    w->DrawArc(gc, c.X(), c.Y(), r);
}

/*
 *
 *
 */

// cast macro

#define circle	((Circle *)obj)

static GraphicSquare gs;

void GraphicCircle::int_start(World *w, double initial_X, double initial_Y, Int_Mode mode) {
    gs.int_start(w, initial_X, initial_Y, mode);
    if (mode == LeftButton) {
      gs.set_all(w, get_pixel_from_value(BLACK), get_pixel_from_value(WHITE), 0, 0, false);
      gs.draw(w, XorMode);
    }
    circle->set_inscr(*(Square *)gs.obj);
}

void GraphicCircle::int_change(World *w, double new_X, double new_Y, Int_Mode mode) {
    if (mode == LeftButton)
      gs.draw(w, XorMode);
    gs.int_change(w, new_X, new_Y, mode);
    if (mode == LeftButton)
      gs.draw(w, XorMode);
    circle->set_inscr(*(Square *)gs.obj);
}

void GraphicCircle::int_end(World *w, double final_X, double final_Y, Int_Mode mode) {
    if (mode == LeftButton) 
      gs.draw(w, XorMode);
    gs.int_change(w, final_X, final_Y, mode);
    circle->set_inscr(*(Square *)gs.obj);
}

void GraphicCircle::resize_start(World *w, double X, double Y, Int_Mode mode) {
    ((Square *)gs.obj)->set(circle->center().X() - circle->radius(), circle->center().Y() - circle->radius(),
    			    circle->center().X() + circle->radius(), circle->center().Y() + circle->radius());
    gs.resize_start(w, X, Y, mode);
    if (mode == LeftButton) {
      gs.set_all(w, get_pixel_from_value(BLACK), get_pixel_from_value(WHITE), 0, 0, false);
      gs.draw(w, XorMode);
    }
}

void GraphicCircle::resize_change(World *w, double X, double Y, Int_Mode mode) {
    int_change(w, X, Y, mode);
}

void GraphicCircle::resize_end(World *w, double X, double Y, Int_Mode mode) {
    int_end(w, X, Y, mode);
}

bool GraphicCircle::where_draw_string (double &x, double &y) {
    Point2D p (((Circle *)obj)->center());
    x = p.X();
    y = p.Y();
    return true;
}







