/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * Coords3D.h - contem a definicao das classes SpaceCoords que representa
 *              coordenadas espaciais de um ponto atraves de coordenadas
 *              homogeneas, e da classe TransfMatrix, que define a matriz
 *              de transformacoes sobre as coordenadas (translacao, rotacao,
 *              escala).
 *  
 *              link flag (-lm)
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    20.01.92  15:00   Alexandre Teles         Operador <<
 *    31.10.91  13:45	Welson R. Jacometti	Arquivo Criado
 *
 */

#ifndef _COORDS3D_DEFINED
#define _COORDS3D_DEFINED

#include <math.h>
#include <iostream.h>
#include "global.h"
#include "tools.h"

/*
 * Constantes exportaveis
 *
 */

#define Space_XCoord   0
#define Space_YCoord   1
#define Space_ZCoord   2
#define Space_WCoord   3

class SpaceCoords {
  friend ostream&
  operator <<( ostream &os, SpaceCoords &sc );

public:
  SpaceCoords() {}

  SpaceCoords( SpaceCoords&);
  SpaceCoords& operator = ( SpaceCoords&);

  double X()  {
      if (coords[Space_WCoord])
        return coords[Space_XCoord] / coords[Space_WCoord];
      else
        return coords[Space_XCoord];
  }

  double Y()  {
      if (coords[Space_WCoord])
        return coords[Space_YCoord] / coords[Space_WCoord];
      else
        return coords[Space_YCoord];
  }
  
  double Z()  {
      if (coords[Space_WCoord])
        return coords[Space_ZCoord] / coords[Space_WCoord];
      else
        return coords[Space_ZCoord];
  }

  double W()  {
      return coords[Space_WCoord];
  }


  void setXYZ(double x, double y, double z) {
      coords[Space_XCoord] = x;
      coords[Space_YCoord] = y;
      coords[Space_ZCoord] = z;
      coords[Space_WCoord] = 1;
  }


  void set_at_infinity(double x, double y, double z) {
      coords[Space_XCoord] = x;
      coords[Space_YCoord] = y;
      coords[Space_YCoord] = z;
      coords[Space_WCoord] = 0;
  }

  void move_to_infinity()
    {
       coords[Space_WCoord] = 0;
    }

  bool at_infinity()  { return coords[Space_WCoord] == 0; }
  
  double& operator [] (int index) { return coords[index]; }
  double operator * ( SpaceCoords&) ;
  SpaceCoords operator + ( SpaceCoords&) ;
  SpaceCoords operator - ( SpaceCoords&) ;

  double normalize();
  int drop();
  void print() { cout << *this; }
        
private:
  double coords[4];
};

class TransfMatrix {
  friend ostream&
  operator <<( ostream &os, TransfMatrix &tm );

public:
  TransfMatrix() {}

  TransfMatrix(TransfMatrix&);

  double* operator [] (int index) { return matrix[index]; }
  TransfMatrix& operator = ( TransfMatrix&);
  TransfMatrix operator * ( TransfMatrix&);
  void ident();
  void trans(double tx, double ty, double tz);
  void rotat(double rx, double ry, double rz);
  void print() { cout << *this; }
        
private:
  void maketrans(double tx, double ty, double tz);
  void makerotat(double rx, double ry, double rz);
  void makexrotat(double rx);
  void makeyrotat(double ry);
  void makezrotat(double rz);
  double matrix[4][4];
};

// tools

bool null( SpaceCoords &p, double tol);
bool equal( SpaceCoords &p1,  SpaceCoords &p2, double tol);
SpaceCoords cross( SpaceCoords&,  SpaceCoords&);
SpaceCoords multmatrix( SpaceCoords&,  TransfMatrix&);

#endif ~_COORDS3D_DEFINED;






