/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 * GapsCovers.h - implementacao dos algoritmos para calculo do menor circulo
 *                envolvente e do maior circulo vazio de um conjunto de pontos.
 * 
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    14.05.92  09:30 	Welson R. Jacometti  	Arquivo criado
 *
 */

#ifndef _GAPSCOVERS_DEFINED
#define _GAPSCOVERS_DEFINED
 
#include "External.h"		// obrigatorio a todo modulo externo
#include "Algorithm.h"
#include "Point2D.h"
#include "Segment2D.h"
#include "Circle.h"

#ifndef _Point2D_ptr_list
#define _Point2D_ptr_list
declare(list,Point2D_ptr)
#endif ~_Point2D_ptr_list;

Circle_ptr smallest_enclosing_circle_shamos(list(Point2D_ptr)&, World *w = 0, int animation_depth = 0, int reference_depth = 0);
Circle_ptr smallest_enclosing_circle_toussaint(list(Point2D_ptr)&, World *w = 0, int animation_depth = 0, int reference_depth = 0);
Circle_ptr largest_empty_circle(list(Point2D_ptr)&, World *w = 0, int animation_depth = 0, int reference_depth = 0);

// Modelagem dos algoritmos

class GapsSetOfPoints : public Algorithm {
    bool animation() { return true; }
    bool is_able_to_handle(const SelectedObjects &, const GraphicObjects &);
};

class SmallestEnclosingCircleShamos : public GapsSetOfPoints {
    ResultObjects binder(const SelectedObjects &, const GraphicObjects &, World *w = 0, int animation_depth = 0, int reference_depth = 0);
};
    
class SmallestEnclosingCircleToussaint : public GapsSetOfPoints {
    ResultObjects binder(const SelectedObjects &, const GraphicObjects &, World *w = 0, int animation_depth = 0, int reference_depth = 0);
};
    
class LargestEmptyCircle : public GapsSetOfPoints {
    ResultObjects binder(const SelectedObjects &, const GraphicObjects &, World *w = 0, int animation_depth = 0, int reference_depth = 0);
};
    
Algorithm* smallest_enclosing_circle_shamos_link();
Algorithm* smallest_enclosing_circle_toussaint_link();
Algorithm* largest_empty_circle_link();

#endif ~_GAPSCOVERS_DEFINED;







