/*
 * This code has been modified for compatibility with other
 * source codes of GeoLab.
 */

/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+                                                                            +
+  LEDA  2.0.1                                                               +
+                                                                            +
+                                                                            +
+  Copyright (c) 1991  by  Max-Planck-Institut fuer Informatik               +
+  Im Stadtwald, 6600 Saarbruecken, FRG                                      +
+  All rights reserved.                                                      +
+                                                                            +
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

/*-----------------------------------------------------------------------------

    File "graph.c" enthaelt die Definitionen  der Member-Funktionen
    der Klasse graph und andere wichtige Routinen 
    (arrays, stacks, error handling, memory management, ... )

    Stefan Naeher
    FB10 Informatik 
    Universitaet des Saarlandes 
    6600 Saarbruecken

    September 1988
    
------------------------------------------------------------------------------*/


#include "Graph.h"


// --------------------------------------------------------------------------
// graph member functions:
// --------------------------------------------------------------------------

list(edge) graph::insert_reverse_edges()
{ list(edge) L = E;
  list(edge) result;
  edge e;
  forall(e,L)
  { result.append(new_edge(e->t,e->s,e->contents));
    copy_edge_entry(e->contents);
   }
  return result;
}


void graph::sort_edges(COMPARE(graph,edge) f)
{ E.sort(f);
  node v;
  forall(v,V) v->adj_edges.sort(f);
}


list(node) graph::adj_nodes(node v) const
{ list(node) result;
  edge e;
  forall(e,v->adj_edges) result.append(target(e));
  return result;
}

void graph::make_undirected()     
{ edge e;
  node v;
  forall(v,V) v->adj_edges.init_iterator();
  forall(e,E)
    if (e->adj_loc2==0)  
        e->adj_loc2 = e->t->adj_edges.append(e);
 }

void graph::make_directed()     
{ edge e;
  node v;
  forall(v,V) v->adj_edges.init_iterator();
  forall(e,E) 
   if (e->adj_loc2)  
    {  e->t->adj_edges.del(e->adj_loc2);
       e->adj_loc2 = 0;
     }
 }


/*   // entry for subgraphs

ent&  graph::entry(node v)     
{ graph* g = this;
  while (g->parent)   // v is subgraph node
  { v = node(v->contents);
    g = g->parent; 
   }
  return v->contents; 
 }

ent&  graph::entry(edge e)     
{ graph* g = this;
  while (g->parent)   // e is subgraph edge 
  { e = edge(e->contents);
    g = g->parent; 
   }
  return e->contents; 
 }

*/

// writing and reading graphs

void put_int(filebuf& fb, int n) 
{ register char* A = (char*)&n;
           char* E = A+4;

  while (A<E) fb.sputc(*(A++)); 
}

int get_int(istream& from) 
{ int n;
  register char* A = (char*)&n;
           char* E = A+4;
  while (A<E) from.get(*(A++));
  return n;
}



void graph::write(string s) const
{ filebuf fbuf;

  if (fbuf.open(s.cstring(),output)==0) 
   error_handler(1,"write: cannot open file");
    
  ostream out(&fbuf);  // ~ostream: flushes file buffer

  int* A = new int[max_node_index+2];

  // nodes get numbers from 1 to |V| (trouble with 0)

  int count = 1;

  out << "LEDA.GRAPH\n"; 
  out << node_type() << "\n";
  out << edge_type() << "\n";
  
  //put_int(fbuf,V.length());

  out << V.length() << "\n";

  node v;
  forall_nodes(v,*this)
  { //put_int(fbuf,int(v->contents));  // node entry (integer)
    write_node_entry(out,v->contents);
    out << "\n";
    A[v->i_name] = count++;
   }

  //put_int(fbuf,E.length());
  out << E.length() << "\n";
  edge e;
  forall_edges(e,*this)
  { //put_int(fbuf,A[e->s->i_name]);
    //put_int(fbuf,A[e->t->i_name]);
    //put_int(fbuf,int(e->contents)); // edge entry (integer)
    out << A[e->s->i_name] << " " << A[e->t->i_name] << " ";
    write_edge_entry(out,e->contents);
    out << "\n";
   }
delete A;
}

int graph::read(string s)
{ filebuf fbuf;
  int result = 0;

  if (fbuf.open(s.cstring(),input)==0)  return 1;
   //error_handler(2,"read: cannot open file");

  istream in(&fbuf);

  clear();

  edge e;
  int n,i,v,w;

  string d_type,n_type,e_type;

  in >> d_type;


  if (d_type != "LEDA.GRAPH") return 3;

  in >> n_type >> e_type;

  in >> n;         // number of nodes
  read_line(in);

  node* A = new node[n+1];

  if (n_type != node_type())
  { if (node_type() != "void") result = 2;
      //error_handler(0,"graph: incompatible node types in read");
    for (i=1; i<=n; i++) 
    { A[i] = new_node();
      read_line(in);
     }
   }
  else
    for (i=1; i<=n; i++) 
    { A[i] = new_node();
      read_node_entry(in,A[i]->contents);
     }
  
  in >> n;       // number of edges

  if (e_type != edge_type())
  { if (edge_type() != "void") result = 2;
      //error_handler(0,"graph: incompatible edge types in read");
    while (n--) { in >> v >> w;
                  e = new_edge(A[v],A[w]); 
                  read_line(in);
                 }
   }
  else
   while (n--) { in >> v >> w;
                 e = new_edge(A[v],A[w]); 
                 read_edge_entry(in,e->contents);
                }

  delete A;
  return result;
}


/*
void   print_node(node v) const
{ if (super() != 0)
     super()->print_node(node(graph::inf(v)));
  else 
     { cout << "[" << index(v) <<"] (" ;
       write_node_entry(v,cout);
       cout << ")";
      }
}

void   print_edge(edge e) const
{ if (super() != 0)
     super()->print_edge(edge(graph::inf(e)));
  else 
     { cout << form("[%d]--(",index(source(e)));
       write_edge_entry(e,cout);
       cout << form(")-->[%d]",index(target(e)));
      }
}
*/

void graph::print(string s) const
{ node v; 
  edge e;
  cout << s << "\n";
  forall_nodes(v,*this)
  { print_node(v);
    cout << ": ";
    forall(e,v->adj_edges) 
      { print_edge(e); 
        cout << " "; 
       }
    cout << "\n"; 
   }
  // cout << "\n space: " << space() << " bytes \n";
  cout << "\n";
}

void graph::reset()  const   // reset all iterators
{ node v;
  forall(v,V) v->adj_edges.init_iterator();
  V.init_iterator();
  E.init_iterator();
}

node graph::new_node(ent i)
{ 
  node v = new i_node(i); 
  v->g = this;
  v->i_name = ++max_node_index;
  v->loc = V.append(v);
  return v;
}

void graph::del_node(node v)
{ if (v->g != this) error_handler(4,"del_node: v is not in G");

  // delete adjacent edges
  while (!v->adj_edges.empty()) del_edge(v->adj_edges.head());

  list_item loc;
  edge e;
  if (v->indegree) 
  { error_handler(0,
    "del_node: indeg >0 , I delete incoming edges (time: O(|E|))");

    loc = E.first();
    while (loc) { e = E[loc];
                  loc = E.succ(loc);
                  if (e->t == v) del_edge(e); } 
   } 

  V.del(v->loc);

  if (parent==0)   // no subgraph
    clear_node_entry(v->contents);
 
  delete v;
}

edge graph::new_edge(node v, node w, ent i)
{ // add edge (v,w) to G

  if ((v->g!=this) || (w->g!=this)) 
  error_handler(6, "G.new_edge(v,w): v or w not in G");


  edge e = new i_edge(v,w,i);

  e->i_name = ++max_edge_index;
  e->loc = E.append(e);
  e->adj_loc=v->adj_edges.append(e);
  e->adj_loc2 = 0;
  w->indegree++;
  return e ; 
}

edge graph::new_edge(edge e1, node w, ent i, int dir=0)  
{ //add edge (source(e1),w) e after/before e1 

  node v  = e1->s;

  if ((v->g!=this) || (w->g!=this)) 
     error_handler(9,"G.new_edge(e,w): e or w not in G");


  edge e = new i_edge(v,w,i);

  e->i_name = ++max_edge_index;
  e->loc = E.append(e);
  e->adj_loc=v->adj_edges.insert(e,e1->adj_loc,dir);
  e->adj_loc2 = 0;
  w->indegree++;
  return e ; 
}

void graph::del_edge(edge e)
{ node v = e->s;
  node w = e->t;

  if (v->g != this) error_handler(10,"del_edge: e is not in G");
     
  E.del(e->loc);

  v->adj_edges.del(e->adj_loc);

  if (e->adj_loc2)                  //undirected edge
  w->adj_edges.del(e->adj_loc2);

  w->indegree--;

  e->i_name = -1;

  if (parent == 0)  // no subgraph
     clear_edge_entry(e->contents);

  delete e; 
}

edge graph::rev_edge(edge e)
{ if (e->adj_loc2) return e;  //undirected edge
  node v = e->s;
  node w = e->t;
  if (v->g != this) error_handler(11,"rev_edge: e is not in G");
  v->adj_edges.del(e->adj_loc);
  e->adj_loc = w->adj_edges.append(e);
  w->indegree--;
  v->indegree++;
  e->s = w;
  e->t = v;
  return(e); 
}

graph& graph::rev()              
{ edge e;
  forall(e,E) rev_edge(e);
  return *this; 
}

void graph::del_all_nodes() { clear(); }

void graph::del_all_edges()
{ node v;
  forall(v,V)
  { v->adj_edges.clear();
    v->indegree=0;
   }
  while (!E.empty()) delete E.pop();
  max_edge_index = -1;
}


graph& graph::operator=(const graph& G)
{ node v,w;
  edge e,e1;
  node* node_vec = new node[G.max_node_index+1];

  edge* edge_vec = new edge[G.max_edge_index+1];

  clear();


  forall_nodes(v,G) 
  { copy_node_entry(v->contents);
    node_vec[v->i_name] = new_node(v->contents);
   }
                             
  forall_edges(e,G) 
   { v = node_vec[e->s->i_name];
     w = node_vec[e->t->i_name];
     copy_edge_entry(e->contents);
     edge_vec[e->i_name] = e1 = new i_edge(v,w,e->contents);
     e1->loc = E.append(e1);
     e1->i_name = ++max_edge_index;
     w->indegree++;
    }

  forall_nodes(v,G) 
  forall_adj_edges(e,v)
  { e1 = edge_vec[e->i_name];
    if (v == e->s && e1->adj_loc == nil) 
      e1->adj_loc  = node_vec[v->i_name]->adj_edges.append(e1);
    else 
      e1->adj_loc2 = node_vec[v->i_name]->adj_edges.append(e1);
   }

  forall_nodes(v,*this) init_adj_iterator(v); 

  parent = G.parent;
  undirected = G.undirected;

  delete node_vec;
  delete edge_vec;

  return *this;
}


void graph::copy_all_entries()
{ node v;
  edge e;
  forall(v,V) copy_node_entry(v->contents);
  forall(e,E) copy_edge_entry(e->contents);
}


graph::graph(const graph& G)  
{ node v,w;
  edge e,e1;
  node* node_vec = new node[G.max_node_index+1];
  edge* edge_vec = new edge[G.max_edge_index+1];


  max_node_index = max_edge_index = -1;

  forall_nodes(v,G) node_vec[v->i_name] = new_node(v->contents);
                             
  forall_edges(e,G) 
   { v = node_vec[e->s->i_name];
     w = node_vec[e->t->i_name];
     copy_edge_entry(e->contents);
     edge_vec[e->i_name] = e1 = new i_edge(v,w,e->contents);
     e1->loc = E.append(e1);
     e1->i_name = ++max_edge_index;
     w->indegree++;
    }

  forall_nodes(v,G) 
  forall_adj_edges(e,v)
  { e1 = edge_vec[e->i_name];
    if (v == e->s && e1->adj_loc == nil) 
      e1->adj_loc  = node_vec[v->i_name]->adj_edges.append(e1);
    else 
      e1->adj_loc2 = node_vec[v->i_name]->adj_edges.append(e1);
   }


  forall_nodes(v,*this) init_adj_iterator(v); 

  parent = G.parent;
  undirected = G.undirected;

  delete node_vec;
  delete edge_vec;

}


// subgraph constructors  (do not work for undirected graphs)

graph::graph(graph& G, list(node)& nl, list(edge)& el)
{ // construct subgraph (nl,el) of graph G

  parent = &G;
  node v,w;
  edge e;

  node* N = new node[G.max_node_index+1];

  forall(v,nl)
   { if (graph_of(v) != parent) 
      error_handler(1,"graph: illegal node in subgraph constructor");
     N[v->i_name] = new_node(ent(v));
    }

  forall(e,el)
   { v = source(e);
     w = target(e);
     if ( graph_of(e)!= parent || N[v->i_name]==0 || N[w->i_name]==0 ) 
      error_handler(1,"graph: illegal edge in subgraph constructor");
     new_edge(N[v->i_name],N[w->i_name],ent(e));
    }

  undirected = G.undirected;

  delete N;

 }

graph::graph(graph& G, list(edge)& el)
{ /* construct subgraph of graph G with edge set el  */

  node  v,w;
  edge  e;
  node* N = new node[G.max_node_index+1];

  forall(v,G.V) N[v->i_name] = 0;

  parent = &G;

  forall(e,el)
   { v = source(e);
     w = target(e);
     if (N[v->i_name] == 0) N[v->i_name] = new_node(ent(v));
     if (N[w->i_name] == 0) N[w->i_name] = new_node(ent(w));
     if ( graph_of(e) != parent )
      error_handler(1,"graph: illegal edge in subgraph constructor");
     new_edge(N[v->i_name],N[w->i_name],ent(e));
    }

  undirected = G.undirected;

  delete N;

 }


// destructor

void graph::clear()
{ node v;
  edge e;

  if (parent==0)  // no subgraph
  {
   forall(e,E) { clear_edge_entry(e->contents);
                 delete e;
                }
   forall(v,V) { clear_node_entry(v->contents);
                 v->adj_edges.clear();
                 delete v;
                }
  }
  else           // subgraph
  { 
    forall(e,E) delete e;
    forall(v,V) { v->adj_edges.clear();
                  delete v;
                 }
   }

  V.clear();
  E.clear();

  max_node_index = max_edge_index = -1;
}

int  graph::space() const
{ int vs = sizeof(dlink) + sizeof(i_node);
  int es = 2*sizeof(dlink) + sizeof(i_edge);
  return V.length()*vs + E.length()*es + 48;  
}

//------------------------------------------------------------------------------
// undirected graphs 
//------------------------------------------------------------------------------

edge ugraph::new_edge(node v, node w, ent i) 
{ 

/*
  if (v==w) error_handler(99,ugraph::new_edge: self-loop");
*/

  if ((v->g!=this) || (w->g!=this)) 
  error_handler(1, "ugraph::new_edge(v,w): v or w  not in G ");

  edge e = graph::new_edge(v,w,i);
  e->adj_loc2 = w->adj_edges.append(e);
  return e;
 }

edge ugraph::new_edge(node v,node w,edge e1,edge e2,ent i,int d1,int d2) 
{ 

/*
  if (v==w) error_handler(99,ugraph::new_edge: self-loop");
*/

  if ((v->g!=this) || (w->g!=this)) 
  error_handler(1, "ugraph::new_edge(v,w): v or w  not in G ");

  edge e = new i_edge(v,w,i);

  e->i_name = ++max_edge_index;
  w->indegree++;
  e->loc = E.append(e);

  if (v==e1->s)
      e->adj_loc=v->adj_edges.insert(e,e1->adj_loc,d1);
  else 
      if (v==e1->t) 
          e->adj_loc=v->adj_edges.insert(e,e1->adj_loc2,d1);
      else 
          error_handler(1,"ugraph::new_edge: e1 not adjacent to v.");


  if (w==e2->s) 
      e->adj_loc2 = w->adj_edges.insert(e,e2->adj_loc,d2);
  else 
      if (w==e2->t) 
          e->adj_loc2 = w->adj_edges.insert(e,e2->adj_loc2,d2);
      else 
          error_handler(1,"ugraph::new_edge: e2 not adjacent to w.");

  return e;
 }



list(node) ugraph::adj_nodes(node v) const
{ list(node) result;
  edge e;
  forall(e,v->adj_edges) result.append(opposite(v,e));
  return result;
}

// ugraph subgraph constructors:

ugraph::ugraph(ugraph& G, list(node)& nl, list(edge)& el)
{ // construct subugraph (nl,el) of ugraph G

  parent = &G;
  node v,w;
  edge e;

  node* N = new node[G.max_node_index+1];

  forall(v,nl)
   { if (graph_of(v) != parent) 
      error_handler(1,"ugraph: illegal node in subgraph constructor");
     N[v->i_name] = new_node(ent(v));
    }

  forall(e,el)
   { v = source(e);
     w = target(e);
     if ( graph_of(e)!= parent || N[v->i_name]==0 || N[w->i_name]==0 ) 
      error_handler(1,"ugraph: illegal edge in subgraph constructor");
     new_edge(N[v->i_name],N[w->i_name],ent(e));
    }

  delete N;

 }

ugraph::ugraph(ugraph& G, list(edge)& el)
{ // construct subgraph of ugraph G with edgelist el 

  node  v,w;
  edge  e;
  node* N = new node[G.max_node_index+1];

  forall(v,G.V) N[v->i_name] = 0;

  parent = &G;

  forall(e,el)
   { v = source(e);
     w = target(e);
     if (N[v->i_name] == 0) N[v->i_name] = new_node(ent(v));
     if (N[w->i_name] == 0) N[w->i_name] = new_node(ent(w));
     if ( graph_of(e) != parent )
      error_handler(1,"ugraph: illegal edge in subgraph constructor");
     new_edge(N[v->i_name],N[w->i_name],ent(e));
    }

  delete N;

 }

node ugraph::opposite(node v, edge e)  const
{ if (v==e->s) return e->t;
  if (v==e->t) return e->s;
  return 0;
 }

edge ugraph::adj_succ(edge e,node v)  const
{ list_item loc;

  if (e->s == e->t)  
       error_handler(9999,"ugraph::adj_succ: self-loop");

  if (v == e->s) 
    loc = e->s->adj_edges.succ(e->adj_loc);
  else if (v == e->t) 
         loc = e->s->adj_edges.succ(e->adj_loc2);
       else error_handler(9999,"adj_succ: v is no endpoint of e");
  return (loc) ? e->s->adj_edges[loc] : 0;
}

edge ugraph::adj_pred(edge e,node v)  const
{ list_item loc;

  if (e->s == e->t)  
       error_handler(9999,"ugraph::adj_pred: self-loop");

  if (v == e->s) 
     loc = e->s->adj_edges.pred(e->adj_loc);
  else if (v == e->t) 
         loc = e->s->adj_edges.pred(e->adj_loc2);
       else error_handler(9999,"ugraph::adj_pred: v is no endpoint of e");
  return (loc) ? e->s->adj_edges[loc] : 0;
 }

edge ugraph::cyclic_adj_succ(edge e,node v)  const
{ list_item loc;

  if (e->s == e->t)  
       error_handler(9999,"ugraph::cyclic_adj_succ: self-loop");

  if (v == e->s)
    loc =  v->adj_edges.cyclic_succ(e->adj_loc);
  else if (v == e->t)
         loc =  v->adj_edges.cyclic_succ(e->adj_loc2);
       else error_handler(9999,"ugraph::cyclic_adj_succ: v is no endpoint of e");
  return v->adj_edges[loc];
 }

edge ugraph::cyclic_adj_pred(edge e,node v)  const
{ list_item loc;

  if (e->s == e->t)  
       error_handler(9999,"ugraph::cyclic_adj_pred: self-loop");

  if (v == e->s)
    loc =  v->adj_edges.cyclic_pred(e->adj_loc);
  else if (v == e->t)
         loc =  v->adj_edges.cyclic_pred(e->adj_loc2);
       else error_handler(9999,"ugraph::cyclic_adj_pred: v is no endpoint of e");
  return v->adj_edges[loc];
}

//------------------------------------------------------------------------------
// node and edge arrays       
//------------------------------------------------------------------------------

#define graph_arrayimplement(itype)\
\
ent& graph_array(itype)::entry(itype x)\
{ if (g != graph_of(x) || index(x) > max_i)\
         error_handler(102,"(node/edge)_array[x]: not defined for x");\
 return arr[index(x)];\
}\
\
ent  graph_array(itype)::inf(itype x) const\
{ if (g != graph_of(x) || index(x) > max_i)\
         error_handler(102,"(node/edge)_array[x]: not defined for x");\
 return arr[index(x)];\
}\
\
void graph_array(itype)::init(const graph& G, ent i)\
{ if (g!=0) clear();\
  g = (graph*)&G;\
  max_i=name3(G.max_,itype,_i)(); \
  arr=new ent[max_i+1];\
  if (arr==0)  error_handler(104,"(node/edge)_array.init(G): out of memory");\
  if (max_i<0) error_handler(0,"(node/edge)_array.init(G): empty graph");\
  for (int j=0;j<=max_i;j++) { copy_entry(i);\
                               arr[j] = i;\
                              }\
  clear_entry(i);\
}\
\
void graph_array(itype)::init(const graph_array(itype)& A)\
{ if (g!=0) clear();\
  g = A.g;\
  max_i=A.max_i;\
  arr=new ent[max_i+1];\
  if (arr==0)  error_handler(1,"(node/edge)_array.init: out of memory");\
  for (int j=0;j<=max_i;j++) { arr[j] = A.arr[j];\
                               copy_entry(arr[j]);\
                              }\
}\
\
void graph_array(itype)::clear()\
{ int j;\
  for (j=0;j<=max_i;j++) clear_entry(arr[j]);\
  if (arr) delete arr;\
  arr=0; g=0; max_i = -1;\
}\


implement(graph_array,node)
implement(graph_array,edge)

//------------------------------------------------------------------------------
// node matrices
//------------------------------------------------------------------------------

graph_array(node)& Node_Matrix::operator[](node v)
{ return *(M[v]); }

ent& Node_Matrix::entry(node v, node w)     { return M[v]->entry(w); }
ent  Node_Matrix::inf(node v, node w) const { return M[v]->inf(w); }

void Node_Matrix::init(const graph& G, ent x) 
{ int i,j;
  g = (graph*)&G;
  M.init(G,0);
  for(i=0;i<M.size();i++) 
    { M.elem(i) = new graph_array(node);
      M.elem(i) -> init(G,0);
      for(j=0;j<M.elem(i)->size();j++) 
        { copy_entry(x);
          M.elem(i)->elem(j) = x;
         }
     }
  clear_entry(x); 
}

void Node_Matrix::init(Node_Matrix& A) 
{ int i,j;
  g = A.g;
  M.init(*g,0);
  for(i=0;i<M.size();i++) 
    { M.elem(i) = new graph_array(node);
      M.elem(i) -> init(*g,0);
      for(j=0;j<M.elem(i)->size();j++) 
        { ent x = A.M.elem(i)->elem(j);
          copy_entry(x);
          M.elem(i)->elem(j) = x;
         }
     }
}

void Node_Matrix::clear()
{ int i,j;
  for(i=0;i<M.size();i++) 
   { for(j=0;j<M.elem(i)->size();j++) 
      { ent x = M.elem(i)->elem(j);
        clear_entry(x); 
       }
     delete M.elem(i);
    }
  M.clear();
}

//------------------------------------------------------------------------------
// node partitions
//------------------------------------------------------------------------------

node_partition::node_partition(const graph& G) 
{ node v;
  I.init(G,nil);
  forall_nodes(v,G) I.entry(v) = ent(P.make_block(ent(v)));
}








