/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 * Intersections.C - mapeamento dos algoritmos para problemas de intersecao
 *                   implementados em intersections.C (.h)
 * 
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    24.04.92  09:30 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "Intersections.h"
#include "basic.h"
#include "List.h"
#include "Point2D.h"
#include "Segment2D.h"
#include "Ray2D.h"
#include "Line2D.h"
#include "Integer.h"
#include "intersections.h"

#ifndef _Point2D_ptr_list
#define _Point2D_ptr_list
declare(list,Point2D_ptr)
#endif ~_Point2D_ptr_list

#ifndef _Segment2D_ptr_list
#define _Segment2D_ptr_list
declare(list,Segment2D_ptr)
#endif ~_Segment2D_ptr_list

#ifndef _Ray2D_ptr_list
#define _Ray2D_ptr_list
declare(list,Ray2D_ptr)
#endif ~_Ray2D_ptr_list

#ifndef _Line2D_ptr_list
#define _Line2D_ptr_list
declare(list,Line2D_ptr)
#endif ~_Line2D_ptr_list

static list(Segment2D_ptr) extract_segments(const SelectedObjects &SO, const GraphicObjects &GO) {
    list(Segment2D_ptr) res;
    list_item it;
    forall(it, SO) {			 	// is_able_to_handle deve ser
      res.push((Segment2D *)GO[it]->obj);	// chamado antes para certificar
    }						// que sao todos segmentos
    return res;
}

static list(Ray2D_ptr) extract_rays(const SelectedObjects &SO, const GraphicObjects &GO) {
    list(Ray2D_ptr) res;
    list_item it;
    forall(it, SO) {			 	// is_able_to_handle deve ser
      res.push((Ray2D *)GO[it]->obj);		// chamado antes para certificar
    }						// que sao todos raios
    return res;
}

static list(Line2D_ptr) extract_lines(const SelectedObjects &SO, const GraphicObjects &GO) {
    list(Line2D_ptr) res;
    list_item it;
    forall(it, SO) {			 	// is_able_to_handle deve ser
      res.push((Line2D *)GO[it]->obj);		// chamado antes para certificar
    }						// que sao todos retas
    return res;
}

bool InterSegments::is_able_to_handle(const SelectedObjects &SO, const GraphicObjects &GO) {
    list_item it;
    
    forall(it, SO) {
      if (GO[it]->type() != Segment2D_t) {
      	SO.init_iterator();
      	return false;
      }
    }
    
    return true;
}

bool InterRays::is_able_to_handle(const SelectedObjects &SO, const GraphicObjects &GO) {
    list_item it;
    
    forall(it, SO) {
      if (GO[it]->type() != Ray2D_t) {
      	SO.init_iterator();
      	return false;
      }
    }
    
    return true;
}

bool InterLines::is_able_to_handle(const SelectedObjects &SO, const GraphicObjects &GO) {
    list_item it;
    
    forall(it, SO) {
      if (GO[it]->type() != Line2D_t) {
      	SO.init_iterator();
      	return false;
      }
    }
    
    return true;
}

ResultObjects AllIntersectionsCountingBF::binder(const SelectedObjects &SO, const GraphicObjects &GO, World *w, int animation_depth, int reference_depth) {
    ResultObjects result;
    Integer_ptr val;
    
    if (!SO.empty()) {
      val = new Integer(all_intersections_counting_bf(extract_segments(SO, GO), w, animation_depth, reference_depth));
      val->obj_volatile(true);
      result.push(val);
    }

    return result;
}
  
ResultObjects AllIntersectionsReportBF::binder(const SelectedObjects &SO, const GraphicObjects &GO, World *w, int animation_depth, int reference_depth) {
    ResultObjects result;
    list(Point2D_ptr) all_inter;
    Point2D_ptr p;

    if (!SO.empty()) {
      all_inter = all_intersections_report_bf(extract_segments(SO, GO), w, animation_depth, reference_depth);
      forall(p, all_inter) {
        result.push(p);
      }
    }

    return result;
}

ResultObjects AllIntersectionsRaysCountingBF::binder(const SelectedObjects &SO, const GraphicObjects &GO, World *w, int animation_depth, int reference_depth) {
    ResultObjects result;
    Integer_ptr val;
    
    if (!SO.empty()) {
      val = new Integer(all_intersections_rays_counting_bf(extract_rays(SO, GO), w, animation_depth, reference_depth));
      val->obj_volatile(true);
      result.push(val);
    }

    return result;
}
  
ResultObjects AllIntersectionsRaysReportBF::binder(const SelectedObjects &SO, const GraphicObjects &GO, World *w, int animation_depth, int reference_depth) {
    ResultObjects result;
    list(Point2D_ptr) all_inter;
    Point2D_ptr p;

    if (!SO.empty()) {
      all_inter = all_intersections_rays_report_bf(extract_rays(SO, GO), w, animation_depth, reference_depth);
      forall(p, all_inter) {
        result.push(p);
      }
    }

    return result;
}

ResultObjects AllIntersectionsLinesCountingBF::binder(const SelectedObjects &SO, const GraphicObjects &GO, World *w, int animation_depth, int reference_depth) {
    ResultObjects result;
    Integer_ptr val;
    
    if (!SO.empty()) {
      val = new Integer(all_intersections_lines_counting_bf(extract_lines(SO, GO), w, animation_depth, reference_depth));
      val->obj_volatile(true);
      result.push(val);
    }

    return result;
}
  
ResultObjects AllIntersectionsLinesReportBF::binder(const SelectedObjects &SO, const GraphicObjects &GO, World *w, int animation_depth, int reference_depth) {
    ResultObjects result;
    list(Point2D_ptr) all_inter;
    Point2D_ptr p;

    if (!SO.empty()) {
      all_inter = all_intersections_lines_report_bf(extract_lines(SO, GO), w, animation_depth, reference_depth);
      forall(p, all_inter) {
        result.push(p);
      }
    }

    return result;
}

Algorithm* all_intersections_counting_bf_link() {
    return new AllIntersectionsCountingBF();
}

Algorithm* all_intersections_report_bf_link() {
    return new AllIntersectionsReportBF();
}

Algorithm* all_intersections_rays_counting_bf_link() {
    return new AllIntersectionsRaysCountingBF();
}

Algorithm* all_intersections_rays_report_bf_link() {
    return new AllIntersectionsRaysReportBF();
}

Algorithm* all_intersections_lines_counting_bf_link() {
    return new AllIntersectionsLinesCountingBF();
}

Algorithm* all_intersections_lines_report_bf_link() {
    return new AllIntersectionsLinesReportBF();
}






