/*
    Copyright (C) 1992-1994
                    Cesar N. Gon

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 * PSPM.h - Point Set Pattern Matching Algorithms
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    16.09.92  21:11 	Cesar N. Gon 	  	Arquivo criado
 *
 */

#ifndef _PSPM_DEFINED
#define _PSPM_DEFINED

#include "External.h"		// obrigatorio a todo modulo externo
#include "Algorithm.h"
#include "Point2D.h"

#ifndef _Point2D_ptr_list
#define _Point2D_ptr_list
declare(list,Point2D_ptr)
#endif ~_Point2D_ptr_list;

// Algoritmos
// point set pattern matching in one dimension
list(Point2D_ptr) PSPM_in_one_dimension(const list(Point2D_ptr)& S, const list(Point2D_ptr)& P, World *w, int animation_depth, int reference_depth);

// point set pattern matching in two dimension
list(Point2D_ptr) PSPM_in_two_dimension(const list(Point2D_ptr)* Ss, const list(Point2D_ptr)& P, World *w, int animation_depth, int reference_depth);


// Modelagem dos algoritmos acima em termos do ambiente GeoLab 

class PSPM1 : public ExternalFunctionMode {
  public:
    
    bool is_able_to_activate(const SelectedObjects &, const GraphicObjects &, World *);
    
    ResultObjects binder(const SelectedObjects &, const GraphicObjects &, World *, int, int);

    bool capture_events() {return false;}

    void installer(World *);
    void uninstaller(World *);

    void activator(World *);
    void deactivator(World *);
    
    icon_bits *icon();

  private:
    list(Point2D_ptr) *S, *P;

};

Algorithm* PSPM1_link();


class PSPM2 : public ExternalFunctionMode {
  public:
    
    bool is_able_to_activate(const SelectedObjects &, const GraphicObjects &, World *);
    
    ResultObjects binder(const SelectedObjects &, const GraphicObjects &, World *, int, int);

    bool capture_events() {return false;}

    void installer(World *);
    void uninstaller(World *);

    void activator(World *);
    void deactivator(World *);
    
    icon_bits *icon();

  private:
    unsigned lenS;
    list(Point2D_ptr) *Ss, *S, *P;
    void preprocessing();

};

Algorithm* PSPM2_link();


#endif ~_PSPM_DEFINED;




