/*
    Copyright (C) 1992-1994
                    Laerte F. Morgado

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 *	Pslg.C - Implementa a classe PSLG
 *
 *        Data      Hora     Nome             Modificado/Motivo
 *       27/01/92   17:05   Laerte Morgado     Arquivo criado
 *
 *
 */



#include "half_edge.h"
#include "handle.h"
#include "Coords2D.h"
#include "Point2D.h"
#include "Line2D.h"
#include "Segment2D.h"
#include "Pslg_tools.h"
#include "Obj.h"
#include "line_settings.h"
#include "External.h"
#include "Pslg.h"
#include <iostream.h>

Pslg::~Pslg()
{;}

GraphicObj* Pslg::new_GraphicObj()
{
    return new GraphicPslg(this);
}


void Pslg::translate(double to_x, double to_y)
{
    Vertex_ptr v;

    v = getSolid(0)->sverts;
    while (v != 0)
      {
	  v->vcoord[Space_XCoord] = v->X() + to_x;
	  v->vcoord[Space_YCoord] = v->Y() + to_y;
	  v = v->nextv;
      }
}

void Pslg::unfill_handles(World* w,bool show_handle) 
{
    Vertex* v;
    Point2D p;
    if ((v = getFirsts()->sverts) == 0) return;

    if (show_handle)
      while(v)
	{
	    p.setXY(v->vcoord[Space_XCoord],v->vcoord[Space_YCoord]);
	    p.unfill_handles(w);
	    p.unfill_handles(w);
	    v = v->nextv;
	};
}

void Pslg::draw_handles(World* w,GC gc,bool highlight,bool show_handle) 
{
    Vertex* v;
    Point2D p;
    if ((v = getFirsts()->sverts) == 0) return;

    if (show_handle)
      while(v)
	{
	    p.setXY(v->vcoord[Space_XCoord],v->vcoord[Space_YCoord]);
	    p.draw_handles(w,gc,highlight);
	    p.draw_handles(w,gc,highlight);
	    v = v->nextv;
	};
}

void Pslg::draw(World* w, GC gc) 
{
    Edge  *edge;
    Vertex *v1, *v2;
    edge  = (getFirsts())->sedges;
    while (edge)
      {
	 v1 = edge->he1->vtx;
	 v2 = edge->he1->nxt->vtx;
	 w->DrawLine(gc, v1->vcoord[Space_XCoord],v1->vcoord[Space_YCoord],v2->vcoord[Space_XCoord],v2->vcoord[Space_YCoord]);
	 edge = edge->nexte;
      }
}


bool Pslg::select_test(double x, double y) 
{
    Vertex* dummy;
    return (vertex_gravity(getFirsts(),x,y,dummy));
}


bool Pslg::select_test(double x1, double y1, double x2, double y2) 
{
    Vertex* v;
    Point2D p;
    v = getFirsts()->sverts;


    while(v)
      {
	  p.setXY(v->vcoord[Space_XCoord],v->vcoord[Space_YCoord]);
	  if (!p.select_test(x1,y1,x2,y2)) return false;
	  v = v->nextv;
      };
    return true;
}



double Pslg::distance_to_point(double x, double y) 
{
    Point2D    p;
    Segment2D  s;
    Vertex_ptr v;
    Edge_ptr   e;
    double aux,distance = MAXDOUBLE; 

    v = getSolid(0)->sverts;
    while (v != 0)
      {
	  p.setXY(v->X() , v->Y());
	  if ((aux = p.distance_to_point(x,y)) < distance)
	    distance = aux;
	  v = v->nextv;
      }

    e = getSolid(0)->sedges;
    while (e != 0)
      {
	  s.setP1(e->he1->X() , e->he1->Y());
	  s.setP2(e->he2->X() , e->he2->Y());
	  if ((aux = s.distance_to_point(x,y)) < distance)
	    distance = aux;
	  e = e->nexte;
      }
    
    return distance;
}


void Pslg::load(FILE* f)
{
    EulerSequence l;
    EulerOperatorRepresentation_ptr E;
    long counter;

    fscanf(f,"%ld\n",&counter);

    while (counter)
      {
	  E = new EulerOperatorRepresentation;
	  fscanf(f,"%ld %ld %ld %ld %ld %ld %lf %lf %lf %ld\n",&(E->f1),&(E->f2),&(E->v1),&(E->v2),&(E->v3),&(E->v4),&(E->x),&(E->y),&(E->z),&(E->name));
	  l.it.append(E);
	  counter--;
      }
    build(0,l);
    l.kill();    
    
}


void Pslg::save(FILE* f)
{
    EulerSequence_ptr l;
    EulerOperatorRepresentation_ptr E;
    l = invert(0);  // Destroy the solid (0).
    build(0,*l);
    fprintf(f, "%ld\n",l->it.length());  // Header...
    
    forall(E,l->it)
      {
	  fprintf(f, "%ld %ld %ld %ld %ld %ld %lf %lf %lf %ld\n",E->f1,E->f2,E->v1,E->v2,E->v3,E->v4,E->x,E->y,E->z,(long)E->name);
      }
    l->kill();
        
}
    

void Pslg::replicate(Pslg& pslg)
{
    EulerSequence_ptr l;
    l = invert(0);  // Destroy the solid (0).
    build(0,*l);    // Rebuild Solid (0)
    pslg.build(0,*l); // Build New Solid based on Euler List *l
    l->kill();        //  Kill Euler List l
}

void Pslg::replicate2(Pslg& pslg)
{
    duplicate(pslg);
}


Face_ptr Pslg::geo_mef(Id solid_id,Id face,Id v1,Id v2,Id new_face)
{

    HalfEdge *he1,*he2;

    get_face_polygon(getVertex(solid_id,v1),getVertex(solid_id,v2),he1,he2,getSolid(solid_id)->findFace(face));

    lmef(he1,he2,new_face);

    return he1->wloop->lface;
}


void Pslg::geo_smev( Id sno, Id f1, Id v1, Id v4, float x, float y, float z )
{
    HalfEdge *he,*aux;
    aux = he = getSolid(sno)->findFace(f1)->findhe1(v1);
    do
      {
	  if (left_side(x,y,aux->prv->vtx->vcoord[Space_XCoord],aux->prv->vtx->vcoord[Space_YCoord], aux->vtx->vcoord[Space_XCoord],aux->vtx->vcoord[Space_YCoord],aux->nxt->vtx->vcoord[Space_XCoord],aux->nxt->vtx->vcoord[Space_YCoord]))
	    {
		he = aux;
		break;
	    };
	  aux = mate(aux)->nxt;
      }
    while (aux != he);

    lmev(he,he,v4,x,y,z);
    
}
    

void Pslg::kill_rays(Vertex_ptr v,list(address)& t,Face_ptr& face)
{
    HalfEdge_ptr base = 0, he = 0;

    base = he = v->vedge;

    do
      {
	  t.push((address)he->wloop->lface->info);
	  he = mate(he)->nxt;
      }
    while (he != base);

    face = ::kill_rays(v);   // Calls the global function : Extended Euler operator.

}





/*
 *
 *
 *  GraphicPslg.h
 *
 *
 */












