/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * QuadEdge.C - implementacao dos metodos das classes QEdge, QuadEdge e 
 *              GraphicQuadEdge.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    18.02.92  12:10 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "QuadEdge.h"
#include "Pslg_tools.h"

// static variables to avoid parameters 
// passing in recursive procedures

bool rec_flag;
World *rec_world;
GC rec_gc;
bool rec_highlight, rec_show_handle;
unsigned long rec_vertexno;
double rec_x1, rec_y1, rec_x2, rec_y2, rec_val;
bool rec_result;
list(Point2D_ptr) *rec_Point2D_list;
list(Segment2D_ptr) *rec_Segment2D_list;


static void set_org_info(QEdge_ptr e,void* inf);
static void rec_to_pslg(QEdge_ptr e,Pslg_ptr pslg);
     
void QEdge::make_edge( Point2D &porg,  Point2D &pdest) {
    Rot = new QEdge(); 			assert(Rot);
    Rot->Rot = new QEdge();		assert(Rot->Rot);
    Rot->Rot->Rot = new QEdge();	assert(Rot->Rot->Rot);
    
    // fecha ciclo de arestas
    
    Rot->Rot->Rot->Rot = this;
    
    // para a aresta primal, Onext nao forma loop
    
    Onext = this;
    Rot->Rot->Onext = Rot->Rot;		// sym()->Onext = sym()
    
    // a aresta dual e' um loop (topologicamente em torno da primal)
    
    Rot->Onext = Rot->Rot->Rot;		// Rot->Onext = Rot->sym()
    Rot->Rot->Rot->Onext = Rot;		// Rot->sym()->Onext = Rot
    
    // aloca espaco para as informacoes geometricas (Org e Dest)
    
    Org = new Point2D(porg);	assert(Org);
    Dest = new Point2D(pdest);	assert(Dest);
    
    Rot->Org = new Point2D();	assert(Rot->Org);
    Rot->Dest = new Point2D();	assert(Rot->Dest);
    
    // ajusta referencias cruzadas entre uma aresta e sua simetrica (inclusive dual)
    
    Rot->Rot->Org = Dest;
    Rot->Rot->Dest = Org;
    
    Rot->Rot->Rot->Org = Rot->Dest;
    Rot->Rot->Rot->Dest = Rot->Org;
    
    // aloca espaco para Flag e OFlag
    
    Flag = new bool;	assert(Flag); 	*Flag = false;
    Rot->Flag = Rot->Rot->Flag = Rot->Rot->Rot->Flag = Flag;
    
    OFlag = new bool; 	assert(OFlag);	*OFlag = false;
    Rot->OFlag = OFlag;
    
    Rot->Rot->OFlag = new bool; assert(Rot->Rot->OFlag); *(Rot->Rot->OFlag) = false;
    Rot->Rot->Rot->OFlag = Rot->Rot->OFlag;
	
    // No caso de um "delete" externo, entao o destrutor invoca 'delete_edge'
    // para liberar a memoria alocada dinamicamente. Porem, como esta e' uma
    // estrutura recursiva, e' preciso controlar a acao do destrutor para
    // que a operacao nao entre em um ciclo infinito. Este controle
    // e' feito verificando-se o valor do flag __delete;

    __delete = Rot->__delete = Rot->Rot->__delete = Rot->Rot->Rot->__delete = new bool;
    *__delete = true;    
}
 
void QEdge::delete_edge() {
    splice(this, oprev());
    splice(sym(), lnext());
    
    // desabilita destrutor
    *__delete = false;
    
    // desaloca toda memoria alocada, na ordem inversa de alocacao
    delete OFlag;
    delete Rot->Rot->OFlag;
    delete Flag;
    delete Org;
    delete Dest;
    delete Rot->Org;
    delete Rot->Dest;
    delete Rot->Rot->Rot;
    delete Rot->Rot;
    delete Rot;   
    delete __delete;
}

void QEdge::print() {
    cout << "QEdge components printed below:" << endl;
    cout << "Edges cicle: " << (unsigned long)this << " -> " << (unsigned long) Rot << " -> " <<
                               (unsigned long)Rot->Rot << " -> " << (unsigned long)Rot->Rot->Rot << " -> " <<
                               (unsigned long)Rot->Rot->Rot->Rot << endl;
    cout << "Link fields:" << endl;
    cout << " Onext = " << (unsigned long)onext() << endl;
    cout << " Oprev = " << (unsigned long)oprev() << endl;
    cout << " sym()->Onext = " << (unsigned long)sym()->onext() << endl;
    cout << " sym()->Oprev = " << (unsigned long)sym()->oprev() << endl;
    cout << " rot()->Onext = " << (unsigned long)rot()->onext() << endl;
    cout << " rot()->Oprev = " << (unsigned long)rot()->oprev() << endl;
    cout << " rot()->sym()->Onext = " << (unsigned long)rot()->sym()->onext() << endl;
    cout << " rot()->sym()->Oprev = " << (unsigned long)rot()->sym()->oprev() << endl;
    cout << "Control flags:" << endl;
    cout << " Flag = " << *Flag << endl;
    cout << " OFlag = " << *OFlag << "   Rot->Rot->OFlag = " << *(Rot->Rot->OFlag) << endl;
    cout << "Geometry: (primal)" << endl;
    Org->print(); Dest->print();
    cout << "Geometry: (dual)" << endl;
    Rot->Org->print(); Rot->Dest->print();
}
    
// auxiliary friend functions for QEdge

void splice(QEdge_ptr a, QEdge_ptr b) {
    QEdge_ptr aux, alpha, beta;
    
    alpha = a->Onext->Rot;
    beta  = b->Onext->Rot;
    
    aux      = a->Onext;
    a->Onext = b->Onext;
    b->Onext = aux;
    
    aux          = alpha->Onext;
    alpha->Onext = beta->Onext;
    beta->Onext  = aux;
}

QEdge_ptr connect_edges(QEdge_ptr a, QEdge_ptr b) {
    QEdge *e = new QEdge(*(a->dest()), *(b->org()));
    splice(e, a->lnext());
    splice(e->sym(), b);
    return e;
}

void delete_incident_edges(QEdge_ptr e) {
    QEdge_ptr eaux = e;
    do {
      e = eaux;
      eaux = e->onext();
      delete e;
    } while (e != eaux);
}

/*
 *
 */
 
GraphicObj* QuadEdge::new_GraphicObj() {
    return new GraphicQuadEdge(this);
}

QuadEdge_forall_bf_BEGIN(recursive_copy_vertices,e)
    if (!already) {
      rec_Point2D_list->append((Point2D_ptr)eaux->org()->reproduct());      
      already = true;
    }
QuadEdge_forall_bf_END(recursive_copy_vertices,e)   

list(Point2D_ptr)* QuadEdge::vertices() {
    rec_Point2D_list = new list(Point2D_ptr);
    if (edge_ptr) {
      rec_flag = *(edge_ptr->oflag());
      recursive_copy_vertices(edge_ptr);
    }
    return rec_Point2D_list;
}

QuadEdge_forall_df_BEGIN(recursive_copy_edges,e)
    rec_Segment2D_list->append(new Segment2D(*(e->org()), *(e->dest())));
QuadEdge_forall_df_END(recursive_copy_edges,e)   

list(Segment2D_ptr)* QuadEdge::edges() {
    rec_Segment2D_list = new list(Segment2D_ptr);
    if (edge_ptr) {
      rec_flag = *(edge_ptr->flag());
      recursive_copy_edges(edge_ptr);
    }
    return rec_Segment2D_list;
}

QuadEdge_forall_bf_BEGIN(recursive_handles_at,e)
    if (rec_result) {
      if (!eaux->org()->at_infinity() && eaux->org()->select_test(rec_x1, rec_y1)) {
        rec_Point2D_list->append(eaux->org());
        already = true;
      }
      if (eaux->onext() == e && already) 
        rec_result = false;
    }
QuadEdge_forall_bf_END(recursive_handles_at,e)   

list(Point2D_ptr)* QuadEdge::handles_at(double x, double y)  {
    rec_Point2D_list = new list(Point2D_ptr);
    if (edge_ptr) {
      rec_flag = *(edge_ptr->oflag());
      rec_x1 = x; rec_y1 = y;
      rec_result = true;
      recursive_handles_at(edge_ptr);
    }
    return rec_Point2D_list;
}

QuadEdge_forall_df_BEGIN(recursive_free,e)
    delete e;
QuadEdge_forall_df_END(recursive_free,e)

void QuadEdge::clear() {
    if (!edge_ptr)
      return;
    rec_flag = *(edge_ptr->flag());
    recursive_free(edge_ptr);
    edge_ptr = 0;
}

QuadEdge_forall_bf_BEGIN(recursive_translate,e)
    eaux->org()->translate(rec_x1, rec_y1);
QuadEdge_forall_bf_END(recursive_translate,e)

void QuadEdge::translate(double to_x, double to_y) {
    if (!edge_ptr)
      return;
    rec_flag = *(edge_ptr->oflag());
    rec_x1 = to_x; rec_y1 = to_y;
    recursive_translate(edge_ptr);
}

QuadEdge_forall_bf_BEGIN(recursive_print,e)
    if (!already) {
      cout << "Vertex #" << ++rec_vertexno << endl;
      eaux->org()->print();
      cout << "Adjacent edges:" << endl;
      already = true;
    }
    eaux->dest()->print();
QuadEdge_forall_bf_END(recursive_print,e)
  
void QuadEdge::print()  {
    if (!edge_ptr)
      return;
    rec_flag = *(edge_ptr->oflag());
    rec_vertexno = 0;
    cout << "QuadEdge listed below: " << endl;
    recursive_print(edge_ptr);
}
	
QuadEdge_forall_bf_BEGIN(recursive_unfill_handles,e)
    if (!already) {
      eaux->org()->unfill_handles(rec_world);
      already = true;
    }
QuadEdge_forall_bf_END(recursive_unfill_handles,e)

QuadEdge_forall_bf_BEGIN(recursive_draw_handles,e)
    if (!already) {
      eaux->org()->draw_handles(rec_world, rec_gc, rec_highlight);
      already = true;
    }
QuadEdge_forall_bf_END(recursive_draw_handles,e)

QuadEdge_forall_df_BEGIN(recursive_draw_edges,e)
    Segment2D(*(e->org()), *(e->dest())).draw(rec_world, rec_gc);
QuadEdge_forall_df_END(recursive_draw_edges,e)   

void QuadEdge::unfill_handles(World *w, bool show_handle)  {
    if (!edge_ptr)
      return;
    if (show_handle) {
      rec_flag = *(edge_ptr->oflag());
      rec_world = w; 
      recursive_unfill_handles(edge_ptr);
    }
}

void QuadEdge::draw_handles(World *w, GC gc, bool highlight, bool show_handle)  {
    if (!edge_ptr)
      return;
    if (show_handle) {
      rec_flag = *(edge_ptr->oflag());
      rec_highlight = highlight;
      rec_world = w;
      rec_gc = gc;
      recursive_draw_handles(edge_ptr);
    }
}

void QuadEdge::draw(World *w, GC gc)  {
    if (!edge_ptr)
      return;
    rec_flag = *(edge_ptr->flag());
    rec_world = w;
    rec_gc = gc;
    recursive_draw_edges(edge_ptr);
}

QuadEdge_forall_bf_BEGIN(recursive_select_or,e)
    if (!already) {
      if (!eaux->org()->at_infinity())
        rec_result = rec_result || eaux->org()->select_test(rec_x1, rec_y1);
      already = true;
    }
QuadEdge_forall_bf_END(recursive_select_or,e)   

bool QuadEdge::select_test(double x, double y)  {
    if (!edge_ptr)
      return false;
    rec_flag = *(edge_ptr->oflag());
    rec_x1 = x; rec_y1 = y;
    rec_result = false;
    recursive_select_or(edge_ptr);
    return rec_result;
}

QuadEdge_forall_bf_BEGIN(recursive_select_and,e)
    if (!already) {
      if (!eaux->org()->at_infinity())
        rec_result = rec_result && eaux->org()->select_test(rec_x1, rec_y1, rec_x2, rec_y2);
      already = true;
    }
QuadEdge_forall_bf_END(recursive_select_and,e)   

bool QuadEdge::select_test(double x1, double y1, double x2, double y2)  {
    if (!edge_ptr)
      return false;
    rec_flag = *(edge_ptr->oflag());
    rec_x1 = x1; rec_y1 = y1;
    rec_x2 = x2; rec_y2 = y2;
    rec_result = true;
    recursive_select_and(edge_ptr);
    return rec_result;
}

QuadEdge_forall_df_BEGIN(recursive_distance_to_point,e)
    double dist = Segment2D(*(e->org()), *(e->dest())).distance_to_point(rec_x1, rec_y1);
    if (dist < rec_val)
      rec_val = dist;
QuadEdge_forall_df_END(recursive_distance_to_point,e)   

double QuadEdge::distance_to_point(double x, double y)  {
    rec_val = TOO_FAR;
    if (edge_ptr) {
      rec_flag = *(edge_ptr->flag());
      rec_x1 = x; rec_y1 = y;
      recursive_distance_to_point(edge_ptr);
    }
    return rec_val;
}

/*
 *
 */
 
// cast macro

#define quad_edge	((QuadEdge *)obj)

static list(Point2D_ptr)* handles_to_move;

void GraphicQuadEdge::resize_start(World *, double X, double Y, Int_Mode) {
    handles_to_move = quad_edge->handles_at(X, Y);
}

void GraphicQuadEdge::resize_change(World *, double X, double Y, Int_Mode) {
    Point2D_ptr p;
    forall(p, *handles_to_move) {
      p->setXY(X, Y);
    }
}

void GraphicQuadEdge::resize_end(World *, double X, double Y, Int_Mode) {
    Point2D_ptr p;
    forall(p, *handles_to_move) {
      p->setXY(X, Y);
    }
    handles_to_move->clear();
    delete handles_to_move;
    quad_edge->subtype(Generic_t);
}




// Coversores

/*
 *   set_org_info:
 *
 *   Faz os campos INFO de todas as arestas que circundam 
 * e->org() iguais a "*inf".
 *
 */
  
static void set_org_info(QEdge_ptr e,void* inf)
{
    QEdge *eaux = e;
    do
      {
	  eaux->info(inf);
	  eaux = eaux->onext();
      } while (eaux!=e);
}



/*
 *   make_QuadEdge_info_NULL:
 *
 *   Faz os campos INFO de todas as arestas do primal 
 * de "e" iguais a 0.
 *
 */

QuadEdge_forall_bf_BEGIN(make_QuadEdge_info_NULL,e)
     //cerr<<"eh"<<endl;
     set_org_info(e,(void*)0);
     //cerr<<"eheh"<<endl;
     set_org_info(e->sym(),(void*)0);
QuadEdge_forall_bf_END(make_QuadEdge_info_NULL,e)

QuadEdge_forall_bf_BEGIN(debug_it,e)
     e->print();
QuadEdge_forall_bf_END(debug_it,e)

     
/*
 *   QEdge::to_pslg
 *
 *   Converte um PSLG (Planar Straight Line Graph) representado
 * sob uma QuadEdge para uma representacao sob HalfEdge.
 *
 */

static bool rec_to_pslg_bool=true;
static Id max_vertex = 0,max_face = 0;
     
static void rec_to_pslg(QEdge_ptr e,Pslg_ptr pslg)
{
    Solid *solid = 0;
    Face  *face = 0;
    Vertex *v1 = 0,*v2 = 0;

    Point2D_ptr  B,C;

    //cerr<<"b"<<endl;
    if (e == 0); //cerr<<";;;;;;"<<endl;
    if (*(e->oflag()) == rec_flag);//  cerr<<"......."<<endl;
    //cerr<<"<--------    Current QuadEdge(e) ---------->"<<endl;
    //cerr<<"("<<e->org()->X()<<","<<e->org()->Y()<<")"<<endl;
    //cerr<<"("<<e->dest()->X()<<","<<e->dest()->Y()<<")"<<endl;
    //cerr<<"->: "<<*(e->oflag())<<endl;
    //cerr<<"<--------    Current QuadEdge(e) ---------->"<<endl;
    if (e && ((*(e->oflag())) == rec_flag))
      {			
	  QEdge *eaux = e;
	  //cerr<<"c"<<endl;
	  do
	    {
		//cerr<<"<--------    Current QuadEdge(eaux) ---------->"<<endl;
		//cerr<<"("<<eaux->org()->X()<<","<<eaux->org()->Y()<<")"<<endl;
		//cerr<<"("<<eaux->dest()->X()<<","<<eaux->dest()->Y()<<")"<<endl;
		//cerr<<"->: "<<*(eaux->oflag())<<endl;
		//cerr<<"<--------  Current QuadEdge(eaux) ---------->"<<endl;
		if ((*(eaux->oflag())) == rec_flag)
		  {
		      B = eaux->org();
		      C = eaux->dest();
		      //cerr<<"d"<<endl;
		      if (rec_to_pslg_bool)
			{
			    //cerr<<"e"<<endl;
			    //cerr<<"e1"<<endl;
			    pslg->mvfs(solid,0,max_face=0,max_vertex=0,B->X(),B->Y());
			    //cerr<<"e2"<<endl;
			    set_org_info(eaux,(void*)(v1 = pslg->getVertex(0,max_vertex)));
			    //cerr<<"e3"<<endl;
			    if (v1 == 0);// cerr<<"hhhhhhhhhhh"<<endl;
			    //cerr<<v1->X()<<"   "<<v1->Y()<<endl;
			    face = solid->sfaces; 
			    //cerr<<"e4"<<endl;
			    if (face == 0);// cerr<<"[][]"<<endl;
			    //cerr<<face->faceno<<endl;
			    //cerr<<v1->vno<<endl;
			    pslg->geo_smev(0,face->faceno,v1->vno,++max_vertex,C->X(),C->Y());
			    //cerr<<"e5"<<endl;
			    set_org_info(eaux->sym(),(void*)pslg->getVertex(0,max_vertex));
			    //cerr<<"e6"<<endl;
			    rec_to_pslg_bool = false;
			    
			}
		      else
			{
			    
			    v1 = (Vertex*)eaux->info();
			    v2 = (Vertex*)eaux->sym()->info();
			    //cerr<<"f"<<endl;
			    
			    if (v2 == 0)
			      {
				  
				  //cerr<<"g"<<endl;
				  find_face(face,v1,(B->X()+C->X())/2,(B->Y()+C->Y())/2);
				  pslg->geo_smev(0,face->faceno,v1->vno,++max_vertex,C->X(),C->Y());
				  set_org_info(eaux->sym(),(void*)pslg->getVertex(0,max_vertex));
			      }
			    else if (v1 == 0)
			      {
				  //cerr<<"gi"<<endl;
				  find_face(face,v2,(B->X()+C->X())/2,(B->Y()+C->Y())/2);
				  pslg->geo_smev(0,face->faceno,v2->vno,++max_vertex,C->X(),C->Y());
				  set_org_info(eaux,(void*)pslg->getVertex(0,max_vertex));
			      }
			    else
			      {
				  //cerr<<"pqp"<<endl;
				  find_face(face,v1,(B->X()+C->X())/2,(B->Y()+C->Y())/2);
				  //cerr<<"+"<<endl;
				  pslg->getFirsts()->getmaxnames(max_face,max_vertex);
				  pslg->geo_mef(0,face->faceno,v1->vno,v2->vno,++max_face);
				  //cerr<<"++++"<<endl;
				  
			      }
			}
		      //cerr<<"i"<<endl;
		  }
		*(eaux->oflag()) = !rec_flag;
		*(eaux->sym()->oflag()) = !rec_flag; 	
		eaux = eaux->onext();
	    } while (eaux != e);
	    
      
	  do
	    {
		//cerr<<"j"<<endl;
		rec_to_pslg(eaux->lnext(),pslg);
		eaux = eaux->onext();					
	    } while (eaux != e);					
      }
    //cerr<<"h"<<endl;
}


     
Pslg_ptr QEdge::to_pslg()
{
    Pslg_ptr pslg = new Pslg();assert(pslg);
    rec_flag = *(this->oflag());
    rec_to_pslg_bool = true;
    max_vertex = max_face = 0;
    make_QuadEdge_info_NULL(this);
    rec_flag = *(this->oflag());
    //cerr<<rec_flag<<endl;
    // cerr<<"***********************************************"<<endl;
    // debug_it(this);
    // cerr<<"***********************************************"<<endl;
    rec_flag = *(this->oflag());
    //cerr<<rec_flag<<endl;
    rec_to_pslg(this,pslg);
    return pslg;
}





/*
 *   QEdge::from_pslg
 *
 *   Converte um PSLG representado sob uma HalfEdge para
 * uma representacao sob QuadEdge.
 *
 */


QEdge_ptr QEdge::from_pslg( Pslg& pslg)
{
    Vertex* v = pslg.getFirsts()->sverts;
    QEdge_ptr e,ref_qe;
    Point2D   p1,p2;
    HalfEdge *he,*ref_he;
    Edge *edge = pslg.getFirsts()->sedges;

    //cerr<<"2"<<endl;
    while (edge!=0)
      {
	  edge->info = (void*)0;
	  edge = edge->nexte;
      }
    //cerr<<"3"<<endl;
    
    while (v!=0)
      {
	  //cerr<<"4"<<endl;
	  he = v->vedge;
	  do
	    {
		//cerr<<"5"<<endl;
		if (he->edg->info == 0)
		  {
		      //cerr<<"6"<<endl;
		      p1.setXY(he->X(),he->Y());  
		      p2.setXY(he->nxt->X(),he->nxt->Y());
		      e = new QEdge();assert(e);
		      e->make_edge(p1,p2);
		      e->info((void*)he);
		      he->edg->info = (void*)e;
		  }
		he = mate(he)->nxt;
            } while  (he!=v->vedge);

	  //cerr<<"7"<<endl;
	  ref_he = v->vedge;
	  ref_qe = (QEdge_ptr)ref_he->edg->info;
	  if ((HalfEdge*)ref_qe->info() != ref_he)
	    ref_qe = ref_qe->sym();
	  he = mate(ref_he)->nxt;

	  //cerr<<"8"<<endl;
	  while  (he!=ref_he)
	    {
		//cerr<<"9"<<endl;
	       e = (QEdge_ptr)he->edg->info;
	       if ((HalfEdge*)e->info() == he)
		 splice(ref_qe,e);
	       else
		 splice(ref_qe,e->sym());
		he = mate(he)->nxt;
            }
	  v = v->nextv;
	  //cerr<<"10"<<endl;
      }
    //cerr<<"11"<<endl;
    //cerr<<"aqui "<<*(((QEdge_ptr)pslg.getFirsts()->sedges->info)->oflag())<<endl;
    return (QEdge_ptr)pslg.getFirsts()->sedges->info;
}
       


 
