/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * QuadEdge.h - definicao das classes QuadEdge e GraphicQuadEdge.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    18.02.92  12:10 	Welson R. Jacometti  	Arquivo criado
 *
 */

#ifndef _QUADEDGE_DEFINED
#define _QUADEDGE_DEFINED

#include <iostream.h>
#include <assert.h>
#include "global.h"
#include "World.h"
#include "Obj.h"
#include "Point2D.h"
#include "Segment2D.h"
#include "Pslg.h"
#include "ExternalObj_id.h"
#include "basic.h"
#include "List.h"

#ifndef _Segment2D_ptr_list
#define _Segment2D_ptr_list
declare(list,Segment2D_ptr)
#endif ~_Segment2D_ptr_list;

// forward declarations
class GraphicQuadEdge;
class QEdge;

typedef QEdge* QEdge_ptr;

class QEdge {
  public:
    // friendness
    friend class QuadEdge;
    friend void splice(QEdge_ptr, QEdge_ptr);
    friend QEdge_ptr connect_edges(QEdge_ptr, QEdge_ptr);
    friend void delete_incident_edges(QEdge_ptr);
    
    // construtores
    QEdge() { init(); }
    QEdge( Point2D &porg,  Point2D &pdest) { make_edge(porg, pdest); }
    
    // destrutor
    ~QEdge() { if (*__delete) delete_edge(); }
    
    // funcoes de manipulacao
    QEdge_ptr rot() { return Rot; }
    QEdge_ptr sym() { return Rot->Rot; }
    QEdge_ptr onext() { return Onext; }
    QEdge_ptr oprev() { return Rot->Onext->Rot; }
    QEdge_ptr lnext() { return Rot->Rot->Rot->Onext->Rot; }  // sym()->oprev()
    QEdge_ptr rprev() { return Rot->Rot->Onext; }	     // sym()->onext()
    
    Point2D_ptr org() { return Org; }
    Point2D_ptr dest() { return Dest; }
    
    bool* flag() { return Flag; }
    bool* oflag() { return OFlag; }

    void* info() { return INFO; }
    void* info(void *i) { return INFO = i; }
    
    
    // funcao pra debug
    void print();


    // Conversores
    Pslg_ptr to_pslg();
    QEdge_ptr from_pslg( Pslg& pslg);
                
  private:
    void make_edge( Point2D &porg,  Point2D &pdest);
    void delete_edge();
    

    void init() {
    	Flag = 0;
    	Org = Dest = 0;
    	Onext = Rot = 0;
    }

    bool *Flag;			// flag para percorrer a estrutura (arestas)
    bool *OFlag;		// flag para percorrer a estrutura (vertices)
    Point2D *Org, *Dest;	// origem e destino de uma aresta
    QEdge_ptr Onext;		// proxima aresta no sentido anti-horario
    QEdge_ptr Rot;		// aresta dual (rotacionada)
    bool *__delete;		// flag PRIVADO para controlar a acao do destrutor
    void *INFO;			// INFO especifica para aplicacoes
};

/*************************************************************************
 *  
 *                  	    TRAVERSAL MACROS
 *
 *************************************************************************/
 
EXPORT bool rec_flag;
EXPORT World *rec_world;
EXPORT GC rec_gc;
EXPORT bool rec_highlight, rec_show_handle;
EXPORT unsigned long rec_vertexno;
EXPORT double rec_x1, rec_y1, rec_x2, rec_y2;
EXPORT bool rec_result;
EXPORT list(Point2D_ptr) *rec_Point2D_list;
EXPORT list(Segment2D_ptr) *rec_Segment2D_list;

#define QuadEdge_forall_df_BEGIN(function_name,e)		\
void function_name(QEdge *e) {					\
    if (e && (*(e->flag()) == rec_flag)) {			\
      *(e->flag()) = !rec_flag;					\
      function_name(e->onext());				\
      function_name(e->rprev());
      /* HERE GOES THE ACTION */
      
#define QuadEdge_forall_df_END(function_name,e) } }

#define QuadEdge_forall_bf_BEGIN(function_name,e)		\
void function_name(QEdge *e) {					\
    if (e && (*(e->oflag()) == rec_flag)) {			\
      QEdge *eaux = e;						\
      QEdge *control_edge = 0;					\
      bool already = false; 					\
      bool yet = true;						\
      do { /* HERE GOES THE ACTION */

#define QuadEdge_forall_bf_END(function_name,e)			\
        *(eaux->oflag()) = !rec_flag;				\
        eaux = eaux->onext();					\
      } while (eaux != e);					\
								\
      do {							\
        function_name(eaux->sym());				\
        eaux = eaux->onext();					\
      } while (eaux != e);					\
    }								\
} /* END */

/*
 * Nota:
 *       As variaveis starting_edge, already e yet introduzidas na macro
 *       QuadEdge_forallxxxxx_bf (BEGIN/END) sao variaveis auxiliares que
 *       devem ser usadas de acordo com a necessidade em questao. 
 *
 *************************************************************************/
 
class QuadEdge : public Obj {
  public:
    // construtores
    QuadEdge(char *s = "QuadEdge", ObjType t = Generic_t, bool v = false) : Obj(s, t, v) { edge_ptr = 0; }
    QuadEdge(QEdge_ptr e) : Obj("QuadEdge") { edge_ptr = e; }
    QuadEdge(QEdge_ptr e, QEdge_ptr le, QEdge_ptr re) : Obj("QuadEdge") { edge_ptr = e; Le = le; Re = re; }
//    QuadEdge( QuadEdge &q) : Obj(q.what_is(), q.stype, q.volati) {}
    
    // destrutor
    ~QuadEdge() { clear(); }
    
    // construtor virtual (simulado) que estabelece o relacionamento
    //
    //                 QuadEdge --> GraphicQuadEdge
    
    virtual GraphicObj* new_GraphicObj();

    // funcao para auto-reproducao de um objeto -> necessario quando
    // queremos, por exemplo, copiar um objeto na interface e este
    // objeto e' externo ao GeoLab.

//    virtual Obj* reproduct()  {
//        return new QuadEdge(*this);
//    }
    
    // information getting methods
    
    virtual QEdge_ptr root() { return edge_ptr; }
    virtual QEdge_ptr le() { return Le; }
    virtual QEdge_ptr re() { return Re; }
    
    virtual void set(QEdge_ptr e, QEdge_ptr le, QEdge_ptr re) {
    	edge_ptr = e; Le = le; Re = re;		// this can be dangerous
    }
    
    virtual list(Point2D_ptr)* vertices();
    virtual list(Segment2D_ptr)* edges();
    virtual list(Point2D_ptr)* handles_at(double x, double y) ;
        
    // general methods
    bool empty() { return !edge_ptr; }
    void clear(); 
    
    // inherited methods redefinition
    ObjType type()  { return QuadEdge_t; }

    void translate(double to_x, double to_y);
    
    void print() ;
    void unfill_handles(World *w, bool show_handle = true) ;
    void draw_handles(World *w, GC gc, bool highlight = false, bool show_handle = true) ;
    void draw(World *, GC) ;

    bool select_test(double x, double y) ;
    bool select_test(double x1, double y1, double x2, double y2) ;

    double distance_to_point(double x, double y) ;


    // Conversores
    bool to_pslg_implemented() { return true;}
    Pslg_ptr to_pslg() {
      Pslg_ptr pslg;
      //cerr<<"zzzzzz "<<*(root()->oflag())<<endl;
      pslg = root()->to_pslg();
      //cerr<<"--------- Result  ----------"<<endl;
      //pslg->getFirsts()->solidls(4);
      //cerr<<"--------- Result  ----------"<<endl;
      return  pslg;
      }

    bool from_pslg_implemented() {return true;}
    QuadEdge*  from_pslg( Pslg& pslg)
      {
        QEdge_ptr tmp;
        tmp = root()->from_pslg(pslg);
        //cerr<<"yyyyyyyy "<<*(tmp->oflag())<<endl;
        return new QuadEdge(tmp);
      }
                
  protected:
    QEdge_ptr edge_ptr;
    QEdge_ptr Le, Re;
};

/*
 *
 * GraphicQuadEdge
 *
 *
 */

class GraphicQuadEdge : public GraphicObj {
  public:
    // construtores
    GraphicQuadEdge() : GraphicObj(new QuadEdge()) {}
    GraphicQuadEdge(QuadEdge *q) : GraphicObj(q) {}
    
    //destrutor
    ~GraphicQuadEdge() {}
    
    // inherited methods redefinition

    void resize_start(World *, double X, double Y, Int_Mode);
    void resize_change(World *, double X, double Y, Int_Mode);
    void resize_end(World *, double X, double Y, Int_Mode);
};

// auxiliary functions

inline bool left_of(const Point2D_ptr &p, QEdge_ptr e) {
    return p->left_turn(*(e->org()), *(e->dest()));
}

inline bool right_of(const Point2D_ptr &p, QEdge_ptr e) {
    return p->right_turn(*(e->org()), *(e->dest()));
}

inline bool valid(QEdge_ptr e, QEdge_ptr basel) {
    return (basel->org()->left_turn(*(e->dest()), *(basel->dest())));
}

typedef QuadEdge* QuadEdge_ptr;
typedef GraphicQuadEdge* GraphicQuadEdge_ptr;

#endif ~_QUADEDGE_DEFINED;










