/*
    Copyright (C) 1992-1994
                    Cesar N. Gon

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * XString.C - XString & GraphicXString methods.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    26.05.92  17:44 	Cesar N. Gon     	Arquivo criado
 *
 */

#include "XString.h"
#include "color.h"
#include <xview/font.h>
#include <xview/notice.h>


char *FontFamily[] = {FONT_FAMILY_DEFAULT, FONT_FAMILY_DEFAULT_FIXEDWIDTH, FONT_FAMILY_LUCIDA, FONT_FAMILY_LUCIDA_FIXEDWIDTH, FONT_FAMILY_ROMAN, FONT_FAMILY_SERIF, FONT_FAMILY_COUR, FONT_FAMILY_OLGLYPH, FONT_FAMILY_OLCURSOR};

char *FontStyle[] = {FONT_STYLE_DEFAULT, FONT_STYLE_NORMAL, FONT_STYLE_BOLD, FONT_STYLE_ITALIC, FONT_STYLE_BOLD_ITALIC};

static bool is_P1;
static bool resizing = false;
static int visual_width;
static double r_x1, r_y1, r_x2, r_y2;

GraphicObj* XString::new_GraphicObj() {
    return new GraphicXString(this);
}


void XString::load(FILE *f)
{
    char *st;
    int lenst;

    Obj::load(f);
    
    p.load(f);
    
    fscanf(f, "%d ", &lenst);
    st = new char[lenst+1];
    fgets (st, lenst+1, f);
    String aux(st);
    xstr = aux;
    delete st;    
}

void XString::save(FILE *f)
{
    Obj::save(f);
      
    p.save(f);

    fprintf(f, "%d ", xstr.length());
    fprintf(f, "%s ", xstr.value());    
}


void XString::draw(World *w, GC gc)  {
    if (!resizing)
      w->DrawString(gc, p.X(), p.Y(),xstr.value(), xstr.length());
    else 
      w->DrawRectangle(gc, r_x1, r_y1, r_x2, r_y2);
}


void GraphicXString::load(FILE *f, World *w)
{
    GraphicObj::load(f,w);
    fscanf(f, "%d %d %d\n", &FontFamily_val, &FontStyle_val, &FontSize_val);
    if (!set_font(w, FontFamily_val, FontStyle_val, FontSize_val)) {
	notice_font_not_found(w);
	set_font_default(w);
    }			
    set_dimensions();
}

void GraphicXString::save(FILE *f) {
    GraphicObj::save(f);
    fprintf(f, "%d %d %d\n", FontFamily_val, FontStyle_val, FontSize_val);
}

void GraphicXString::resize_start(World *w, double X, double Y, Int_Mode) {
    resizing = true;
    visual_width = w->xcoord(p2.X())-(w->xcoord(p1.X()));
    if (p1.select_test(X, Y))
      is_P1 = true;
    else
      is_P1 = false;
    r_x1 = p1.X();
    r_y1 = p1.Y()-descent;
    r_x2 = p2.X();
    r_y2 = p2.Y()+ascent;
}


void GraphicXString::resize_change(World *w, double X, double Y, Int_Mode)
{
    if ( (is_P1 && X < p2.X()) || (!is_P1 && p1.X() < X) ) {
	if (is_P1)
	  setP1(X, p1.Y());
	else 
	  setP2(X, p2.Y());
	double factor = (double)(w->xcoord(p2.X())-(w->xcoord(p1.X())))/visual_width;
	r_x1 = p1.X();
	r_y1 = p1.Y()-descent*factor;
	r_x2 = p2.X();
	r_y2 = p2.Y()+ascent*factor;
    }
    else
      is_P1 = !is_P1;
}

void GraphicXString::resize_end(World *w, double X, double Y, Int_Mode m) {
    resize_change(w, X, Y, m);
    double factor = (double)(w->xcoord(p2.X())-(w->xcoord(p1.X())))/visual_width;
    int FontResize = (int) (FontSize_val * factor);
    draw(w,XorMode);
    if (!set_font (w, FontFamily_val, FontStyle_val, FontResize)) {
	cerr << "Coudn't create a font for this size.\n";
    }
    else
      xstring->setP(p1.X()+HANDLE_DX, obj_P().Y());
    draw(w,XorMode);
    resizing = false;
}


void GraphicXString::set_font_default(World *w)
{
    Xv_Font font;
    font = (Xv_Font) xv_get (w->get_canvas(), XV_FONT);
    FontFamily_val = 0;
    FontStyle_val = 0;
    FontSize_val = (int)xv_get (font, FONT_SIZE);
    fontinfo = (XFontStruct *)xv_get (font, FONT_INFO);
    // inicializa gc_world, gch, gco
    gc_init(w);
    w->SetFont (gco, fontinfo->fid);
}

bool GraphicXString::set_font(World* w, int FFamily, int FStyle, int FSize)
{
    Xv_Font font;

    Frame frame = (Frame)xv_get(w->get_canvas(), XV_OWNER);
    /* little clock, yes */
    xv_set(frame, FRAME_BUSY, TRUE, NULL);
    font = (Xv_font) xv_find (w->get_canvas (),
			      FONT,
			      FONT_FAMILY, FontFamily[FFamily],
			      FONT_STYLE, FontStyle[FStyle],
			      FONT_SIZE, FSize,
			      NULL);
    if (font != 0) {
	FontSize_val = (int)xv_get (font, FONT_SIZE);
	FontStyle_val = FStyle;
	FontFamily_val = FFamily;
	fontinfo = (XFontStruct *)xv_get (font, FONT_INFO);
	// inicializa gc_world, gch, gco
	gc_init(w);
	w->SetFont (gco, fontinfo->fid);
	/* little clock, no more */
	xv_set(frame, FRAME_BUSY, FALSE, NULL);
	return true;
    }

    xv_set(frame, FRAME_BUSY, FALSE, NULL);
    return false;
}


void GraphicXString::draw_cursor(World *w, bool inwhite)
{
    unsigned long sav_fg, sav_bg, wh;
    set_dimensions();
    short des = fontinfo->max_bounds.descent;
    short asc = fontinfo->max_bounds.ascent;
    short rbe = fontinfo->max_bounds.rbearing;
    short lbe = fontinfo->min_bounds.lbearing;
    double x1 = vx_add (w, obj_P().X(), overall_return.width+1);
    double y1 = vy_add (w, obj_P().Y(), des);
    double x2 = vx_add (w, obj_P().X(), overall_return.width+rbe-lbe+1);
    double y2 = vy_add (w, obj_P().Y(), -asc);
    if (saved_dm != CopyMode)
      set_draw_mode(w, CopyMode);
    saved_dm = CopyMode;
    if (inwhite) {
	get_color (sav_fg, sav_bg);
	wh = get_pixel_from_value(WHITE);
	set_color (w, wh, wh);
	w->DrawRectangle (gco, x1, y1, x2, y2);
	set_color (w, sav_fg, sav_bg);
    }
    else
      w->DrawRectangle (gco, x1, y1, x2, y2);
}


void GraphicXString::unfill_handles(World *w) {
    if (show_handle) {
	p1.unfill_handles(w);
	p2.unfill_handles(w);
    }
}

void GraphicXString::draw_handles(World *w) {    
    if (show_handle) {
	if (!resizing)
	  adjust_handles();
	p1.draw_handles(w, gch, is_selected);
	p2.draw_handles(w, gch, is_selected);
    }
}


void GraphicXString::adjust_handles()
{
    /* gc_world j'a foi setado em set_font() */
    set_dimensions();
    /* O c'alculo de p1 e p2 n~ao pode ser alterado somente aqui,
       pois ele influencia fun,c~oes com as de resize, etc. */
    p1.setXY (vx_add(gc_world, obj_P().X(), -HANDLE_DX), vy_add(gc_world, obj_P().Y(), 0));
    p2.setXY (vx_add(gc_world, obj_P().X(), overall_return.width+HANDLE_DX), p1.Y());
}


/* Esta fun,c~ao recalcula v'arias vari'aveis da classe */
void GraphicXString::set_dimensions() {
	XTextExtents (fontinfo, ((*(XString *)obj).XStr().value()),((*(XString *)obj).XStr().length()), &direction, &ascent, &descent, &overall_return);
}
    

/* Fun,c~oes Auxiliares */

void notice_font_not_found(World *w)
{
    Frame frame = (Frame)xv_get(w->get_canvas(), XV_OWNER);
    (void)notice_prompt(frame, NULL, 
			NOTICE_MESSAGE_STRINGS,
			"Coud not find the specified font.",
			"Using default font instead.",
			NULL,
			NOTICE_BUTTON_YES,
			"Continue",
			NULL);
}




