/*
    Copyright (C) 1992-1994
                    Cesar N. Gon

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * XString.h - definicao das classes XString e GraphicXString.
 *
 *             link flag -lm
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    26.05.92  19:29 	Cesar N. Gon  	        Arquivo criado
 *
 */

#ifndef _XSTRING_DEFINED
#define _XSTRING_DEFINED

#include <iostream.h>
#include "global.h"
#include "tools.h"
#include "World.h"
#include "Obj.h"
#include "Point2D.h"
#include "Segment2D.h"
#include "ExternalObj_id.h"

// forward declaration
class GraphicXString;

// cast macro
#define xstring	((XString *)obj)

#define HANDLE_DX 5


extern char *FontFamily[];
extern char *FontStyle[];

inline double vx_add(World *w, double x, int width) {
    return w->virtual_xcoord(w->xcoord(x)+width);
}

inline double vy_add(World *w, double y, int width) {
    return w->virtual_ycoord(w->ycoord(y)+width);
}
    

class XString : public Obj {
  public:
    // construtores
    XString(char *s = "XString", ObjType t = Generic_t, bool v = false) : Obj (s, t, v) { setP(0,0); set_xstring("");}
    XString( Point2D &p, char *s = "") : Obj("XString") {setP(p); set_xstring(s);}
    XString( XString &xs) : Obj(xs.what_is(), xs.stype, xs.volati) {setP(xs.P()); set_xstring(xs.xstr);}
    
    //destrutor
    ~XString() {}				
    
    // construtor virtual (simulado) que estabelece o relacionamento
    //
    //                  XString --> GraphicXString
    
    GraphicObj* new_GraphicObj();
    
    // funcao para auto-reproducao de um objeto - necessario para possibilitar 
    // que a interface manipule objetos externos ao geolab (alem disso torna
    // mais homogeneo o tratamento de objetos conhecidos e desconhecidos)

    Obj* reproduct()  {
    	return new XString(*this);
    }

    // query & set functions
    Point2D P()  { return p; }
    String XStr()  {return xstr;}
    
    virtual void setP(double x, double y) {p.setXY(x, y); }
    virtual void setP( Point2D &a) {p = a; }

    virtual void set_xstring (char *st) {
	String aux (st);
	xstr = st;
    }

    virtual void set_xstring (String& str) {
	xstr = str;
    }
        
    // inherited methods redefinition

    void load(FILE *f);
    void save(FILE *f);

    ObjType type()  { return XString_t; }    

    void print()  { cout << "XString ..." << xstr.value() << "in P (" << P().X() << "," << P().Y() << ")" << endl;}
    
    void draw(World *w, GC gc) ;
          
  protected:
    String xstr;
    Point2D p;
};

/*
 *
 * GraphicXString.h
 *
 *
 */

class GraphicXString : public GraphicObj {
  public:
    // construtores
    GraphicXString() : GraphicObj(new XString()) {}
    GraphicXString(XString *a) : GraphicObj(a) {}
    
    //destrutor
    ~GraphicXString() {}

    GraphicObj* reproduct() {
	GraphicXString *go;
        go = (GraphicXString *)GraphicObj::reproduct();
	go->fontinfo = fontinfo;
	go->FontFamily_val = FontFamily_val;
	go->FontStyle_val = FontStyle_val;
	go->FontSize_val = FontSize_val;	
	go->set_dimensions();
	return go;
    }

    void load(FILE *f, World *w);
    void save(FILE *f);

   
    void resize_start(World *, double X, double Y, Int_Mode);
    void resize_change(World *, double X, double Y, Int_Mode);
    void resize_end(World *, double X, double Y, Int_Mode);

    virtual bool select_test(double x, double y) {
    	return p1.select_test(x, y) || p2.select_test(x, y);
    }
    
    virtual bool select_test(double x1, double y1, double x2, double y2) {
    	return p1.select_test(x1, y1, x2, y2) && p2.select_test(x1, y1, x2, y2);
    }

    virtual double distance_to_point(double x, double y) {
	Segment2D seg (p1,p2);
	return seg.distance_to_point(x,y);	
    }

    
    virtual void translate(double to_x, double to_y) {
    	p1.translate(to_x, to_y);
    	p2.translate(to_x, to_y);
	Point2D p(((XString *)obj)->P());
        p.translate (to_x, to_y);
        ((XString *)obj)->setP(p);
    }
    
    virtual void draw_handles(World *w);

    virtual void unfill_handles(World *w);


    // query & set functions
    Point2D obj_P() {return xstring->P();}
    void set_obj_P( Point2D &q) {xstring->setP(q);}
    
    Point2D P1()  { return p1; }
    Point2D P2()  { return p2; }

    virtual void setP1(double x, double y) { p1.setXY(x, y); }
    virtual void setP2(double x, double y) { p2.setXY(x, y); }

    virtual void set(double x1, double y1, double x2, double y2) {
    	p1.setXY(x1, y1);
    	p2.setXY(x2, y2);
    }
    
    void set( Point2D &a,  Point2D &b) {
    	p1 = a; p2 = b;
    }

    // set functions for XString
    void setP(double x, double y) {
	(*(XString *)obj).setP(x,y);
	// adjust_handles();
    }
    
    void setP( Point2D &a) {
	(*(XString *)obj).setP(a);
	// adjust_handles();
    }

    void set_xstring(char *st) {
	(*(XString *)obj).set_xstring(st);
	// adjust_handles();
    }

    void set_xstring(String& str) {
	(*(XString *)obj).set_xstring(str);
	// adjust_handles();
    }


    bool set_font(World* w, int FFamily, int FStyle, int FSize);

    void set_font_default(World *w);

    void set_dimensions();

    void draw_cursor(World *w, bool inwhite);

    int heigth() {return ascent+descent;}

    int width() {return fontinfo->max_bounds.width;}
    
  protected:
    void adjust_handles();
    Point2D p1, p2;
    XFontStruct *fontinfo;
    XCharStruct overall_return;
    int direction, ascent, descent;
    int FontFamily_val;
    int FontStyle_val;
    int FontSize_val;
};

typedef XString* XString_ptr;
typedef GraphicXString* GraphicXString_ptr;

#endif ~_XSTRING_DEFINED;


/* Prototypes de fun,c~oes auxiliares */

void notice_font_not_found(World *w);


