/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * animation.C - constantes para manipulacao dos valores de profundidade de
 *               animacao.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    08.10.91  21:25 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "animation.h"
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/openmenu.h>
#include "edit_mode.h"
#include "geolab_ui.h"
#include "algorithms.h"
#include "goodies_file.h"

int REFERENCE_DEPTH = 0;
bool PAUSE = false;
int STEP = 0;
int FRAME_DELAY = 0;

// variaveis internas
bool FIRST_FRAME_SAVE = true;
GC GC_FRAME_SAVE = 0;

static bool popup_color, popup_zoom, popup_line, popup_compose,
            popup_random_points, popup_random_cover, popup_random_polygon, 
            popup_color_bw, popup_algorithms_settings, popup_algorithms_statistics, 
            popup_algorithms_hackers_dream, popup_edit_string, popup_goodies;
            
extern geolab_popup_color_settings_objects		geolab_popup_color_settings;
extern geolab_popup_zoom_settings_objects		geolab_popup_zoom_settings;
extern geolab_popup_line_settings_objects		geolab_popup_line_settings;
extern geolab_popup_random_points_settings_objects	geolab_popup_random_points_settings;
extern geolab_popup_random_cover_settings_objects	geolab_popup_random_cover_settings;
extern geolab_popup_random_polygon_settings_objects	geolab_popup_random_polygon_settings;
extern geolab_popup_animation_settings_objects		geolab_popup_animation_settings;
extern geolab_popup_color_settings_bw_objects		geolab_popup_color_settings_bw;
extern geolab_popup_algorithms_settings_objects 	geolab_popup_algorithms_settings;
extern geolab_popup_algorithms_statistics_objects	geolab_popup_algorithms_statistics;
extern geolab_popup_algorithms_hackers_dream_objects	geolab_popup_algorithms_hackers_dream;
extern geolab_popup_edit_string_objects			geolab_popup_edit_string;
extern geolab_popup_compose_objects			geolab_popup_compose;
extern geolab_geolab_base_window_objects 		geolab_geolab_base_window;

void frame_save(World *w) {
    if (FIRST_FRAME_SAVE) {
      XGCValues v;
      v.foreground = (unsigned long)xv_get(w->get_canvas(), WIN_BACKGROUND_COLOR);
      GC_FRAME_SAVE = w->CreateGC(GCForeground, v);
      FIRST_FRAME_SAVE = false;
    }

    w->pscroll_begin(GC_FRAME_SAVE);    
}

void frame_restore(World *w) {
    if (GC_FRAME_SAVE) {
      w->pscroll_same_size(GC_FRAME_SAVE, 0, 0);
      w->pscroll_end(0, 0);
    }
}

void frame_stop_mark(World *w) {
    if (animation_on()) {
	notify_dispatch();
	w->Flush();
      
	if (STEP > 0)
	  STEP--;
      else {
      	if (PAUSE) {
          while(PAUSE && !STEP) {
            usleep(100000);
            notify_dispatch();
            w->Flush();
          }
          if (STEP)
            STEP--;
        }
        else
          if (FRAME_DELAY == 0)
            for(int i = 0; i < 10; i++) {
              notify_dispatch();
              w->Flush();
            }
          else
            if (FRAME_DELAY < 50)
              usleep(FRAME_DELAY * 5000);
            else
              usleep(FRAME_DELAY * 10000);	  
      }
      
      notify_dispatch();
      w->Flush();
    }
}

void frame_delay_mark(World *w) {
    if (animation_on()) {
      notify_dispatch();
      w->Flush();
      
      if (FRAME_DELAY == 0)
        for(int i = 0; i < 10; i++) {
          notify_dispatch();
          w->Flush();
        }
      else
        if (FRAME_DELAY < 50)
          usleep(FRAME_DELAY * 5000);
        else
          usleep(FRAME_DELAY * 10000);	  
      
      notify_dispatch();
      w->Flush();
    }
}

// interface related functions

bool animation_events(World *, Xv_Window, Event *event) {
    // Esta funcao consome todos os eventos, de forma a tornar seguro
    // as operacoes efetuadas durante uma animacao. Para contornar um
    // bug do xview, que e' o de nao acionar a funcao FRAME_DONE_PROC
    // para command frames (no caso, a janela de settings para animacao)
    // e portanto nao garantir que a janela nao sera' retirada da tela
    // (o que pode emperrar o ambiente, ja' que se a tecla PAUSE estiver
    // pressionada e a janela se for, nao ha' como continuar a animacao)
    // foi colocado aqui um tratamento especial para o click com o botao
    // da esquerda, que coloca na tela a janela de settings para animacao.
    
    if (event_action(event) == ACTION_SELECT && event_is_down(event))
      (void)xv_set(geolab_popup_animation_settings.popup_animation_settings, XV_SHOW, TRUE, 0);
    return TRUE;
}

void animation_disable_interface(World *w) {
    //
    // retira janelas que nao devem estar presentes durante a animacao e
    // guarda o status (XV_SHOW) para depois restaurar coerentemente
    //
    if (popup_color = (bool)xv_get(geolab_popup_color_settings.popup_color_settings, XV_SHOW))
      (void)xv_set(geolab_popup_color_settings.popup_color_settings, FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, 0);
    if (popup_zoom = (bool)xv_get(geolab_popup_zoom_settings.popup_zoom_settings, XV_SHOW))
      (void)xv_set(geolab_popup_zoom_settings.popup_zoom_settings, FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, 0);
    if (popup_line = (bool)xv_get(geolab_popup_line_settings.popup_line_settings, XV_SHOW))
      (void)xv_set(geolab_popup_line_settings.popup_line_settings, FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, 0);
    if (popup_random_points = (bool)xv_get(geolab_popup_random_points_settings.popup_random_points_settings, XV_SHOW))
      (void)xv_set(geolab_popup_random_points_settings.popup_random_points_settings, FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, 0);
    if (popup_random_cover = (bool)xv_get(geolab_popup_random_cover_settings.popup_random_cover_settings, XV_SHOW))
      (void)xv_set(geolab_popup_random_cover_settings.popup_random_cover_settings, FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, 0);
    if (popup_random_polygon = (bool)xv_get(geolab_popup_random_polygon_settings.popup_random_polygon_settings, XV_SHOW))
      (void)xv_set(geolab_popup_random_polygon_settings.popup_random_polygon_settings, FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, 0);
    if (popup_color_bw = (bool)xv_get(geolab_popup_color_settings_bw.popup_color_settings_bw, XV_SHOW))
      (void)xv_set(geolab_popup_color_settings_bw.popup_color_settings_bw, FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, 0);
    if (popup_algorithms_settings = (bool)xv_get(geolab_popup_algorithms_settings.popup_algorithms_settings, XV_SHOW))
      (void)xv_set(geolab_popup_algorithms_settings.popup_algorithms_settings, FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, 0);
    if (popup_algorithms_statistics = (bool)xv_get(geolab_popup_algorithms_statistics.popup_algorithms_statistics, XV_SHOW))
      (void)xv_set(geolab_popup_algorithms_statistics.popup_algorithms_statistics, FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, 0);
    if (popup_algorithms_hackers_dream = (bool)xv_get(geolab_popup_algorithms_hackers_dream.popup_algorithms_hackers_dream, XV_SHOW))
      (void)xv_set(geolab_popup_algorithms_hackers_dream.popup_algorithms_hackers_dream, FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, 0);
    if (popup_edit_string = (bool)xv_get(geolab_popup_edit_string.popup_edit_string, XV_SHOW))
      (void)xv_set(geolab_popup_edit_string.popup_edit_string, FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, 0);
    if (popup_compose = (bool)xv_get(geolab_popup_compose.popup_compose, XV_SHOW))
      (void)xv_set(geolab_popup_compose.popup_compose, FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, 0);
    if (popup_goodies = goodies_file_window_on_screen())
      goodies_file_window_show_false();
      
    //
    // desabilita botoes e panel_choice (draw/function) mode
    //
    
    (void)xv_set(geolab_geolab_base_window.button_file, PANEL_INACTIVE, TRUE, 0);
    (void)xv_set(geolab_geolab_base_window.button_view, PANEL_INACTIVE, TRUE, 0);
    (void)xv_set(geolab_geolab_base_window.button_edit, PANEL_INACTIVE, TRUE, 0);
    (void)xv_set(geolab_geolab_base_window.button_properties, PANEL_INACTIVE, TRUE, 0);
    (void)xv_set(geolab_geolab_base_window.button_input_tools, PANEL_INACTIVE, TRUE, 0);
    (void)xv_set(geolab_geolab_base_window.button_algorithms, PANEL_INACTIVE, TRUE, 0);
    (void)xv_set(geolab_geolab_base_window.button_animation, PANEL_INACTIVE, TRUE, 0);
    (void)xv_set(geolab_geolab_base_window.button_compose, PANEL_INACTIVE, TRUE, 0);
    (void)xv_set(geolab_geolab_base_window.setting_mode, PANEL_INACTIVE, TRUE, 0);

    //
    // desabilita menu de algoritmos
    //

    disable_all_algorithms();
    
    //
    // finalmente, desabilita o tratamento de eventos (canvas) adicionando
    // uma funcao tratadora que consome todos os eventos que recebe
    //
    
    w->add_event_proc(animation_events);    
}

void animation_restore_interface(World *w) {
    //
    // restaura janelas retiradas (ou colocadas)
    //
    if (popup_color) 
      (void)xv_set(geolab_popup_color_settings.popup_color_settings, FRAME_CMD_PUSHPIN_IN, TRUE, XV_SHOW, TRUE, 0);
    if (popup_zoom) 
      (void)xv_set(geolab_popup_zoom_settings.popup_zoom_settings, FRAME_CMD_PUSHPIN_IN, TRUE, XV_SHOW, TRUE, 0);
    if (popup_line) 
      (void)xv_set(geolab_popup_line_settings.popup_line_settings, FRAME_CMD_PUSHPIN_IN, TRUE, XV_SHOW, TRUE, 0);
    if (popup_random_points) 
      (void)xv_set(geolab_popup_random_points_settings.popup_random_points_settings, FRAME_CMD_PUSHPIN_IN, TRUE, XV_SHOW, TRUE, 0);
    if (popup_random_cover)
      (void)xv_set(geolab_popup_random_cover_settings.popup_random_cover_settings, FRAME_CMD_PUSHPIN_IN, TRUE, XV_SHOW, TRUE, 0);
    if (popup_random_polygon)
      (void)xv_set(geolab_popup_random_polygon_settings.popup_random_polygon_settings, FRAME_CMD_PUSHPIN_IN, TRUE, XV_SHOW, TRUE, 0);
    if (popup_color_bw)
      (void)xv_set(geolab_popup_color_settings_bw.popup_color_settings_bw, FRAME_CMD_PUSHPIN_IN, TRUE, XV_SHOW, TRUE, 0);
    if (popup_algorithms_settings)
      (void)xv_set(geolab_popup_algorithms_settings.popup_algorithms_settings, FRAME_CMD_PUSHPIN_IN, TRUE, XV_SHOW, TRUE, 0);
    if (popup_algorithms_statistics)
      (void)xv_set(geolab_popup_algorithms_statistics.popup_algorithms_statistics, FRAME_CMD_PUSHPIN_IN, TRUE, XV_SHOW, TRUE, 0);
    if (popup_algorithms_hackers_dream)
      (void)xv_set(geolab_popup_algorithms_hackers_dream.popup_algorithms_hackers_dream, FRAME_CMD_PUSHPIN_IN, TRUE, XV_SHOW, TRUE, 0);
    if (popup_edit_string)
      (void)xv_set(geolab_popup_edit_string.popup_edit_string, FRAME_CMD_PUSHPIN_IN, TRUE, XV_SHOW, TRUE, 0);
    if (popup_compose)
      (void)xv_set(geolab_popup_compose.popup_compose, FRAME_CMD_PUSHPIN_IN, TRUE, XV_SHOW, TRUE, 0);
    if (popup_goodies)
      goodies_file_window_show_true();
      
    //
    // restaura botoes do painel principal
    //
    
    (void)xv_set(geolab_geolab_base_window.button_file, PANEL_INACTIVE, FALSE, 0);
    (void)xv_set(geolab_geolab_base_window.button_view, PANEL_INACTIVE, FALSE, 0);
    (void)xv_set(geolab_geolab_base_window.button_edit, PANEL_INACTIVE, FALSE, 0);
    (void)xv_set(geolab_geolab_base_window.button_properties, PANEL_INACTIVE, FALSE, 0);
    (void)xv_set(geolab_geolab_base_window.button_input_tools, PANEL_INACTIVE, FALSE, 0);
    (void)xv_set(geolab_geolab_base_window.button_algorithms, PANEL_INACTIVE, FALSE, 0);
    (void)xv_set(geolab_geolab_base_window.button_animation, PANEL_INACTIVE, FALSE, 0);
    (void)xv_set(geolab_geolab_base_window.button_compose, PANEL_INACTIVE, FALSE, 0);
    (void)xv_set(geolab_geolab_base_window.setting_mode, PANEL_INACTIVE, FALSE, 0);

    //
    // restaura menu de algoritmos
    //

    select_applyable_algorithms();
    select_applyable_mode(w);
    
    //
    // retira o tratador de eventos de animacao (consumidor de eventos)
    //
    
    w->del_event_proc();
}
