/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * color.C - modulo de alocacao de cores do GeoLab. Conexao entre a interface
 *           e a aplicacao feita manualmente.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    12.12.91  08:45 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "color.h"

unsigned long FOREGROUND;
unsigned long BACKGROUND;
bool SET_FOREGROUND;
int SCREEN_DEPTH;

static char *color_names[] = {"Black",
  			      "Gray",
			      "Cyan",
			      "Blue",
			      "Sienna",
			      "Red",
			      "Magenta",
			      "Yellow",
			      "Orange",
			      "Medium Sea Green",
			      "Dark Green",
			      "Green",
			      "Goldenrod",
			      "Gold",
			      "Khaki",
			      "White"};
			     
static unsigned long pixel_table[NUM_COLORS];

void init_colors(Xv_opaque win) {
    Display *dpy = (Display *)xv_get(win, XV_DISPLAY);
    Colormap cmap;
    XVisualInfo vinfo;
    XColor exact_def;
    int i = NUM_COLORS + 1;

    pixel_table[WHITE] = WhitePixel(dpy, 0);
    pixel_table[BLACK] = BlackPixel(dpy, 0);
     
    if ((SCREEN_DEPTH = DefaultDepth(dpy, 0)) > 1) {
      cmap = DefaultColormap(dpy, 0);
      while (!XMatchVisualInfo(dpy, 0, SCREEN_DEPTH, i--, &vinfo));
    
      if (i < StaticColor) {
        SCREEN_DEPTH = 1;
        return;
      }
    
      for (i=0; i < NUM_COLORS; i++) {
        if (!XParseColor(dpy, cmap, color_names[i], &exact_def)) {
          cerr << "color.C: init_colors: " << color_names[i] << " not found" << endl;
          continue;
        }
        if (!XAllocColor(dpy, cmap, &exact_def)) {
      	  cerr << "color.C: init_colors: " << color_names[i] << " could not be allocated" << endl;
      	  continue;
        }
        pixel_table[i] = exact_def.pixel;
      }
    }

    //
    // Default values
    //    

    SET_FOREGROUND = true;

    // Agora, default para FOREGROUND e BACKGROUND s~ao setados
    // em geolab_stubs.C (dependem de Default_Obj_Attributes).
    // Gon, 24/08/92
}

unsigned long get_pixel_from_value(int value) {
    if (!pixel_table) {
      cerr << "color.C: get_pixel_from_value: pixel table null" << endl;
      exit(0);
    }

    return pixel_table[value];
}


int get_value_from_pixel(unsigned long pixel) {
    if (!pixel_table) {
      cerr << "color.C: get_value_from_pixel: pixel table null" << endl;
      exit(0);
    }

    register int i;
    for (i=0;i<NUM_COLORS;i++)
      if (pixel == pixel_table[i]) return i;

    // pixel not found...
    return BLACK;
}






