/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * compose.C - modulo para composicao de algoritmos.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    24.06.92  21:25 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "compose.h"
#include "basic.h"
#include "List.h"
#include "algorithms.h"
#include "animation.h"

typedef struct composed {
    Algorithm* alg;
    bool discard_input;
    bool unselect_input;
    bool output_selected;	
    char *function_name;
    char *module_name;
} ComposedAlgorithm;

typedef ComposedAlgorithm* ComposedAlgorithm_ptr;

#ifndef _ComposedAlgorithm_ptr_list
#define _ComposedAlgorithm_ptr_list
declare(list,ComposedAlgorithm_ptr)
#endif ~_ComposedAlgorithm_ptr_list;

bool COMPOSING = false;
bool LAST_PLAY = false;

list(ComposedAlgorithm_ptr) COMPOSITION;

void compose_start() {
    // libera composicao corrente
    ComposedAlgorithm_ptr calg;
    forall(calg, COMPOSITION) {
      delete calg;
    }
    COMPOSITION.clear();
    
    // ajusta variaveis exportadas
    COMPOSING = true;
}

void compose_add_algorithm(Algorithm *alg, bool di, bool ui, bool os, char *fn, char *mn) {
    ComposedAlgorithm_ptr calg = new ComposedAlgorithm;
    
    calg->alg = alg;
    calg->discard_input = di;
    calg->unselect_input = ui;
    calg->output_selected = os;
    calg->function_name = fn;
    calg->module_name = mn;
    
    COMPOSITION.append(calg);
}

void compose_finish() {
    // ajusta variaveis exportadas
    COMPOSING = false;
}

void compose_play(World *w) {
    LAST_PLAY = true;
    if (animation_on())
      notify_stop();
    else
      compose_play_dispatcher(w);
}

void compose_play_dispatcher(World *w) {
    ComposedAlgorithm_ptr calg;

    if (compose_exec_start(w, COMPOSITION.head()->alg)) {
      forall(calg, COMPOSITION) {
        compose_exec(w, calg->alg, calg->discard_input, calg->unselect_input, calg->output_selected, calg->function_name, calg->module_name);
      }
      compose_exec_finish(w);
    }
}




