/*
    Copyright (C) 1992-1994
                    Cesar N. Gon

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 * draw_string.C - implementacao das funcoes para desenho de XString.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    26.05.92  19:07 	Cesar N. Gon     	Arquivo criado
 *
 */

#include "draw_string.h"
#include "XString_ui.h"

#include <assert.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gdd.h>
#include <xview/notice.h>


#define XSTRING_CURSOR_XHOT  0
#define XSTRING_CURSOR_YHOT  15

int FONT_FAMILY_VALUE = 0;
int FONT_STYLE_VALUE = 0;
int FONT_SIZE_VALUE = 12; /* WIN_SCALE_MEDIUM */

XString_popup_xstring_objects	*XString_popup_xstring;

//
// Instance XV_KEY_DATA key.  An instance is a set of related
// user interface objects.  A pointer to an object's instance
// is stored under this key in every object.  This must be a
// global variable.
//
extern Attr_attribute	INSTANCE;

char view_example[] = {193, 202, 204, 213, 220, 199, 225, 234, 238, 245, 252, 231, '[', '(', '{', '+', '-', '/', '*'};

icon_bits xstring_cursor_icon[] = {
#include "ICONS/xstring.cursor"
};

icon_bits draw_string_icon[] = {
#include "ICONS/draw_string.icon"
};


//
// Auxiliar variables...
//     

static GraphicXString* g_XStr;
static bool first;
									      
//
// Auxiliar functions...
//


bool isaccent(int code)  {
    return (code == ACUTE_KEY || code == GRAVE_KEY || code == TILDE_KEY || code == DDOT_KEY || code == HAT_KEY); 
}


int inc_accent(int c, int accent)
{
    switch (accent) {
      case GRAVE_KEY: return 0;
      case ACUTE_KEY: return 1;
      case HAT_KEY:   return 2;
      case TILDE_KEY: return 3;
      case DDOT_KEY:  return 3+(c=='A'||c=='a'||c=='O'||c=='o');
    }
    return 0;
}


int put_accent(int c, int accent)
{
    if (accent == TILDE_KEY && (c=='E' || c=='e' || c=='I' || c=='i' || c=='U' || c=='u'))
	return c;
    switch ((char)c) {
      case ' ': return accent;
      case 'C':
      case 'c': return C_CEDILLHA+c-'C';	
      case 'A':
      case 'a': return A_ACCENTS+inc_accent(c,accent)+c-'A';
      case 'E':
      case 'e': return E_ACCENTS+inc_accent(c,accent)+c-'E';
      case 'I':
      case 'i': return I_ACCENTS+inc_accent(c,accent)+c-'I';
      case 'O':
      case 'o': return O_ACCENTS+inc_accent(c,accent)+c-'O';
      case 'U':
      case 'u': return U_ACCENTS+inc_accent(c,accent)+c-'U';
      case 'N':
      case 'n':
	if (accent == TILDE_KEY)
	  return N_TILDE+c-'N';
	else
	  return c;
      default: return c;
    }
}

void draw_string_finish(World *w) {
    g_XStr->set_handle(Default_Obj_Attributes.HANDLES == 0);
    g_XStr->draw_cursor(w, true);
    g_XStr->draw(w, CopyMode);
    if (((XString *)(g_XStr->obj))->XStr().length() == 0) // n~ao aceita string vazia
      delete g_XStr;
    else  
      EO_install_externally_created_object(g_XStr);
    first=true;
}

bool draw_string_events(World *w, Xv_Window, Event *event)
{
    static char buff[XSTRING_MAX_LEN+1];
    static int  i;

    static accent = NO_KEY;
    int c;
    bool end = false;
        
    double X = w->virtual_xcoord(event_x(event));
    double Y = w->virtual_ycoord(event_y(event));

    if (!first)
      if (event_is_ascii(event))
	if (event_is_down(event)) {
	    c = event_action(event);
	    if (c == CR_KEY) 
	      end = true;
	    else {
		if (c == BS_KEY) {
		    if (i>0) {
			g_XStr->draw(w, XorMode);			
			buff[--i]='\0';		  
		    }
		}
		else {
		    if (isaccent(c)) 
			if (accent != c) {
			    accent=c;
			    return true;
			}
			else
			  accent = NO_KEY;		    
		    if (i == XSTRING_MAX_LEN) {
			cerr << "String too long!\n";
			return true;
		    }
		    if (accent != NO_KEY) {
		      c = put_accent (c, accent);
		      accent = NO_KEY;
		    }  
		    buff[i++]=(char)c;
		    buff[i]='\0';
		}
		g_XStr->draw_cursor(w, true);
		g_XStr->set_xstring(buff);
		g_XStr->draw_cursor(w, false);
		g_XStr->draw(w, CopyMode);
		return true;
	    }
	}
    
    
    switch (event_action(event)) {
      case ACTION_SELECT:
      case ACTION_ADJUST:
        if (!event_is_down(event))
	  if (first) {	      
	      g_XStr = new GraphicXString();
	      assert(g_XStr);
	      if (!g_XStr->set_font(w,FONT_FAMILY_VALUE,FONT_STYLE_VALUE, FONT_SIZE_VALUE)) {
		  notice_font_not_found (w);
		  g_XStr->set_font_default(w);
	      }		
	      g_XStr->set_all (w, FOREGROUND, BACKGROUND, 0, 0, HandlesOn);
	      g_XStr->setP(X,Y);	      
	      g_XStr->set_xstring("");
	      g_XStr->set_handle(false);
	      i=0;
	      buff[0]='\0';
	      g_XStr->draw_cursor (w,false);
	      first=false;
	      return true;
	  }
	  else
	    end = true;
    }

    if (end) {
	draw_string_finish(w);
	return true;
    }
    
    // retorna true APENAS se for necessario CONSUMIR o evento
    return false;
}

//
// Notify callback function for `setting_xstring_family'.
//
void
xstring_family_handler(Panel_item item, int	 value, Event *event)
{
    FONT_FAMILY_VALUE = value;
}

//
// Notify callback function for `setting_xstring_style'.
//
void
xstring_style_handler(Panel_item item, int	 value, Event *event)
{
    FONT_STYLE_VALUE = value;
}

//
// Notify callback function for `slider_xstring_size'.
//
void
xstring_size_handler(Panel_item item, int value, Event *event)
{
    FONT_SIZE_VALUE = value;
}

//
// Notify callback function for `button_xstring_view'.
//
void
xstring_view_example_handler(Panel_item item, Event *event)
{
	XString_popup_xstring_objects	*ip = (XString_popup_xstring_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	World *w = ip->world;
	w->ClearWindow();
	static first = true;
	static GraphicXString *g_XStr;
	if (first) {
	    g_XStr = new GraphicXString();
	    assert(g_XStr);
	    first = false;
	}
	if (!g_XStr->set_font(w,FONT_FAMILY_VALUE,FONT_STYLE_VALUE, FONT_SIZE_VALUE)) {
	    (void)notice_prompt(XString_popup_xstring->popup_xstring, NULL,
				NOTICE_MESSAGE_STRINGS,
				  "Could not find the specified font.",
				  NULL,
				NOTICE_BUTTON_YES,
				  "Continue",
				  NULL);
	}
	else {
	    g_XStr->set_dimensions();
	    g_XStr->set_all (w, FOREGROUND, BACKGROUND, 0, 0, HandlesOn);
	    g_XStr->set_handle(false);	    
	    g_XStr->setP(w->virtual_xcoord(0),w->virtual_ycoord(g_XStr->heigth()));
	    g_XStr->set_xstring("ABCabc123DEFdef456");
	    g_XStr->draw (w, CopyMode);
	    g_XStr->setP(w->virtual_xcoord(0),w->virtual_ycoord(2*g_XStr->heigth()));
	    g_XStr->set_xstring(view_example);
	    g_XStr->draw (w, CopyMode);
	  }      
}




//
// ExternalObjectMode methods & functions...
//

icon_bits *draw_string::icon() {return draw_string_icon;}

icon_bits *draw_string::cursor(int& xhot, int& yhot) {
    xhot = XSTRING_CURSOR_XHOT;
    yhot = XSTRING_CURSOR_YHOT;
    return xstring_cursor_icon;
}


void draw_string::installer(World *w) {
    // Initialize user interface components.
    XString_popup_xstring = new XString_popup_xstring_objects;
    assert(XString_popup_xstring);
    XString_popup_xstring->objects_initialize((Frame)xv_get(w->get_canvas(), XV_OWNER));	  
}

void draw_string::uninstaller(World *w) {
    (void)xv_set(XString_popup_xstring->popup_xstring, FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, 0);
    delete XString_popup_xstring;
}

void draw_string::activator(World *w) {
    (void)xv_set(XString_popup_xstring->popup_xstring, XV_SHOW, TRUE, 0);
    w->add_event_proc(draw_string_events);
    w->add_event_proc(fixed_movement_events);
    first = true;
}

void draw_string::deactivator(World *w) {
    if (!first)
      draw_string_finish(w);
    (void)xv_set(XString_popup_xstring->popup_xstring, XV_SHOW, FALSE, WIN_SHOW, FALSE, 0);
}
    

Algorithm* draw_string_link() {
    return new draw_string();
}
