/*
    Copyright (C) 1992-1994
                    Cesar N. Gon

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 * draw_string.h - implementacao de algoritmo externo para o desenho de
 *		   um tipo externo ao geolab (XString). Deve servir como
 *		   exemplo para outras implementacoes.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    26.05.92  19:06 	Cesar N. Gon     	Arquivo criado
 *
 */

#ifndef _DRAW_STRING_DEFINED
#define _DRAW_STRING_DEFINED

#include "External.h"
#include "Algorithm.h"
#include "XString.h"

#define XSTRING_MAX_LEN 80

#define CR_KEY         13
#define BS_KEY      31745
#define ACUTE_KEY      39
#define GRAVE_KEY      96
#define TILDE_KEY     126
#define DDOT_KEY       34
#define HAT_KEY        94
#define NO_KEY          0

#define C_CEDILLHA    199
#define N_TILDE       209
#define A_ACCENTS     192
#define E_ACCENTS     200
#define I_ACCENTS     204
#define O_ACCENTS     210
#define U_ACCENTS     217      

/******************************************************************************
   funcao que instala os tratadores de eventos ao ser acionado o "draw string"
   mode.
*******************************************************************************/
  
void install_draw_string_events(World *w);

/******************************************************************************
   funcao tratadora de eventos para insercao de XString.
   
   Nota: 
         Funcoes externas para tratamento de eventos tendem a ser muito
         semelhantes (a exemplo de insert_simple_events() para os objetos
         internos ao GeoLab). Assim sendo, recomenda-se que estas funcoes
         sejam compartilhadas entre varios objetos, de forma a diminuir
         a quantidade de codigo 'redundante' a ser gerado.
*******************************************************************************/

bool draw_string_events(World *, Xv_Window, Event *);

// Modelagem

class draw_string : public ExternalObjectMode {
  public:

    void installer(World *);
    void uninstaller(World *);

    void activator(World *);
    void deactivator(World *);
    
    icon_bits *icon();
    icon_bits *cursor(int& xhot, int& yhot);
    
    Obj* instantiation(ObjType t) {
    	if (t == XString_t)
          return new XString();
        else 
          return 0;
    }
};

Algorithm* draw_string_link();

#endif ~_DRAW_STRING_DEFINED;
