/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti
                    Cesar N. Gon

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * edit_mode.C - modulo de gerencia dos diversos "modes" de edicao no GeoLab.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    26.09.91  18:25 	Welson R. Jacometti  	Arquivo criado
 *    10.01.93  13:10   C'esar N. Gon		Modos externos
 *
 */

#include "utilities.h"
#include "edit_mode.h"
#include "select.h"
#include "move_copy.h"
#include "resize.h"
#include "zoom.h"
#include "scroll.h"
#include "dynamic.h"
#include "insert_simple.h"
#include "instantiation.h"
#include "algorithms.h"
#include "objects.h"
#include "trapsignal.h"

/*
 * Variaveis exportadas.
 *
 */
 
ExternalObjectMode *EXTERNAL_OBJECT_MODE = 0;
ExternalFunctionMode *EXTERNAL_FUNCTION_MODE = 0;

/*
 * Constantes e variaveis internas ao modulo.
 *
 */

#define DRAG_ACCURACY	15
 
static int EDIT_MODE = POINT;

#define CURSOR_WIDTH	16
#define CURSOR_HEIGHT	16

#define INSERT_CURSOR_XHOT	7
#define INSERT_CURSOR_YHOT	7

#define ZOOM_CURSOR_XHOT	6
#define ZOOM_CURSOR_YHOT	6

#define DYNAMIC_CURSOR_XHOT	8
#define DYNAMIC_CURSOR_YHOT	8


icon_bits clean_icon[] = {
#include "ICONS/clean.icon"
};

icon_bits EF_icon[] = {
#include "ICONS/EF.icon"
};

icon_bits EO_icon[] = {
#include "ICONS/EO.icon"
};

icon_bits EF_cursor_icon[] = {
#include "ICONS/EF.cursor"
};

icon_bits insert_cursor_icon[] = {
#include "ICONS/insert.cursor"
};

icon_bits select_cursor_icon[] = {
#include "ICONS/select.cursor"
};

icon_bits move_cursor_icon[] = {
#include "ICONS/move.cursor"
};

icon_bits copy_cursor_icon[] = {
#include "ICONS/copy.cursor"
};

icon_bits resizeop_cursor_icon[] = {
#include "ICONS/resizeop.cursor"
};

icon_bits resizecl_cursor_icon[] = {
#include "ICONS/resizecl.cursor"
};

icon_bits zoom_cursor_icon[] = {
#include "ICONS/zoom.cursor"
};

icon_bits scroll_cursor_icon[] = {
#include "ICONS/scroll.cursor"
};

icon_bits dynamic_cursor_icon[] = {
#include "ICONS/dynamic.cursor"
};

Server_image
EF_cursor_image = (Server_image)xv_create(NULL, SERVER_IMAGE,
					XV_WIDTH,		CURSOR_WIDTH,
					XV_HEIGHT,		CURSOR_HEIGHT,
					SERVER_IMAGE_BITS,	EF_cursor_icon,
 					NULL);

Server_image
insert_cursor_image = (Server_image)xv_create(NULL, SERVER_IMAGE,
					XV_WIDTH,		CURSOR_WIDTH,
					XV_HEIGHT,		CURSOR_HEIGHT,
					SERVER_IMAGE_BITS,	insert_cursor_icon,
 					NULL);
					
Server_image
select_cursor_image = (Server_image)xv_create(NULL, SERVER_IMAGE,
					XV_WIDTH,		CURSOR_WIDTH,
					XV_HEIGHT,		CURSOR_HEIGHT,
					SERVER_IMAGE_BITS,	select_cursor_icon,
					NULL);
					
Server_image
move_cursor_image = (Server_image)xv_create(NULL, SERVER_IMAGE,
					XV_WIDTH,		CURSOR_WIDTH,
					XV_HEIGHT,		CURSOR_HEIGHT,
					SERVER_IMAGE_BITS,	move_cursor_icon,
					NULL);
					
Server_image
copy_cursor_image = (Server_image)xv_create(NULL, SERVER_IMAGE,
					XV_WIDTH,		CURSOR_WIDTH,
					XV_HEIGHT,		CURSOR_HEIGHT,
					SERVER_IMAGE_BITS,	copy_cursor_icon,
					NULL);
					
Server_image
resizeop_cursor_image = (Server_image)xv_create(NULL, SERVER_IMAGE,
					XV_WIDTH,		CURSOR_WIDTH,
					XV_HEIGHT,		CURSOR_HEIGHT,
					SERVER_IMAGE_BITS,	resizeop_cursor_icon,
					NULL);
					
Server_image
resizecl_cursor_image = (Server_image)xv_create(NULL, SERVER_IMAGE,
					XV_WIDTH,		CURSOR_WIDTH,
					XV_HEIGHT,		CURSOR_HEIGHT,
					SERVER_IMAGE_BITS,	resizecl_cursor_icon,
					NULL);
					
Server_image
zoom_cursor_image = (Server_image)xv_create(NULL, SERVER_IMAGE,
					XV_WIDTH,		CURSOR_WIDTH,
					XV_HEIGHT,		CURSOR_HEIGHT,
					SERVER_IMAGE_BITS,	zoom_cursor_icon,
					NULL);

Server_image
scroll_cursor_image = (Server_image)xv_create(NULL, SERVER_IMAGE,
					XV_WIDTH,		CURSOR_WIDTH,
					XV_HEIGHT,		CURSOR_HEIGHT,
					SERVER_IMAGE_BITS,	scroll_cursor_icon,
					NULL);

Server_image
dynamic_cursor_image = (Server_image)xv_create(NULL, SERVER_IMAGE,
					XV_WIDTH,		CURSOR_WIDTH,
					XV_HEIGHT,		CURSOR_HEIGHT,
					SERVER_IMAGE_BITS,	dynamic_cursor_icon,
					NULL);
					
Xv_Cursor 
insert_cursor = (Xv_Cursor)xv_create(NULL, CURSOR,
					CURSOR_IMAGE,		insert_cursor_image,
					CURSOR_XHOT,		INSERT_CURSOR_XHOT,
					CURSOR_YHOT,		INSERT_CURSOR_YHOT,
					NULL);

Xv_Cursor
EO_cursor = insert_cursor;
					
					
Xv_Cursor 
EF_cursor = (Xv_Cursor)xv_create(NULL, CURSOR,
					CURSOR_IMAGE,		EF_cursor_image,
					NULL);									       					
Xv_Cursor 
select_cursor = (Xv_Cursor)xv_create(NULL, CURSOR,
					CURSOR_IMAGE,		select_cursor_image,
					NULL);
					
Xv_Cursor 
move_cursor = (Xv_Cursor)xv_create(NULL, CURSOR,
					CURSOR_IMAGE,		move_cursor_image,
					NULL);
					
Xv_Cursor 
copy_cursor = (Xv_Cursor)xv_create(NULL, CURSOR,
					CURSOR_IMAGE,		copy_cursor_image,
					NULL);
					
Xv_Cursor 
resizeop_cursor = (Xv_Cursor)xv_create(NULL, CURSOR,
					CURSOR_IMAGE,		resizeop_cursor_image,
					NULL);
					
Xv_Cursor 
resizecl_cursor = (Xv_Cursor)xv_create(NULL, CURSOR,
					CURSOR_IMAGE,		resizecl_cursor_image,
					NULL);
					
Xv_Cursor 
zoom_cursor = (Xv_Cursor)xv_create(NULL, CURSOR,
					CURSOR_IMAGE,		zoom_cursor_image,
					CURSOR_XHOT,		ZOOM_CURSOR_XHOT,
					CURSOR_YHOT,		ZOOM_CURSOR_YHOT,
					NULL);
					
Xv_Cursor 
scroll_cursor = (Xv_Cursor)xv_create(NULL, CURSOR,
					CURSOR_IMAGE,		scroll_cursor_image,
					NULL);
					
Xv_Cursor 
dynamic_cursor = (Xv_Cursor)xv_create(NULL, CURSOR,
					CURSOR_IMAGE,		dynamic_cursor_image,
					CURSOR_XHOT,		DYNAMIC_CURSOR_XHOT,
					CURSOR_YHOT,		DYNAMIC_CURSOR_YHOT,
					NULL);


/****************************************************************************
 *
 *            SUPORTE PARA ALGORITMOS EXTERNOS INTERATIVOS
 *
 ****************************************************************************/

Xv_Cursor EXTERNAL_FUNCTION_CURSOR = 0;
Xv_Cursor EXTERNAL_OBJECT_CURSOR = 0;
bool EF_capture_events;
bool EO_capture_events;
					
Xv_opaque mode_panel = 0;
Xv_opaque extra_object_image;
Xv_opaque extra_function_image;

bool EF_events(World *world, Xv_Window, Event *event) {
    bool ans = false;
    double X = world->virtual_xcoord(event_x(event));
    double Y = world->virtual_ycoord(event_y(event));

    static bool has_double_click = false;
    
    switch (event_action(event)) {
	
      case ACTION_SELECT:
	if (event_is_down(event))
	  if (double_click(event)) {
	      ans = EXTERNAL_FUNCTION_MODE->double_click(world,X,Y);
	      has_double_click = true;
	  }
	  else
	    ans = EXTERNAL_FUNCTION_MODE->click_select_down(world,X,Y);
	else {
	    if (has_double_click) 
	      has_double_click = false;
	    else
	      ans = EXTERNAL_FUNCTION_MODE->click_select_up(world,X,Y);
	}
	break;
	
      case ACTION_ADJUST:
	if (event_is_down(event)) 
	  ans = EXTERNAL_FUNCTION_MODE->click_adjust_down(world,X,Y);       
	else 
	  ans = EXTERNAL_FUNCTION_MODE->click_adjust_up(world,X,Y);	
	break;

      case LOC_DRAG:
	ans = EXTERNAL_FUNCTION_MODE->loc_drag(world,X,Y);
	break;

      case LOC_MOVE:
	ans = EXTERNAL_FUNCTION_MODE->loc_move(world,X,Y);
	break;
    }


    if (event_is_ascii(event))
      if (event_is_down(event))
	ans = EXTERNAL_FUNCTION_MODE->ascii_key(world, event_action(event));
    
    return ans;
}

void EF_call_activator(World *w) {
    EXTERNAL_FUNCTION_MODE->activator(w);
    if (EF_capture_events)
      w->add_event_proc(EF_events);    
}

void EF_call_deactivator(World *w) {
    EXTERNAL_FUNCTION_MODE->deactivator(w);
    w->reset_event_proc();
}



bool EO_events(World *world, Xv_Window, Event *event) {
    bool ans = false;
    double X = world->virtual_xcoord(event_x(event));
    double Y = world->virtual_ycoord(event_y(event));

    static bool has_double_click = false;
    
    switch (event_action(event)) {
	
      case ACTION_SELECT:
	if (event_is_down(event))
	  if (double_click(event)) {
	      ans = EXTERNAL_OBJECT_MODE->double_click(world,X,Y);
	      has_double_click = true;
	  }
	  else
	    ans = EXTERNAL_OBJECT_MODE->click_select_down(world,X,Y);
	else {
	    if (has_double_click) 
	      has_double_click = false;
	    else
	      ans = EXTERNAL_OBJECT_MODE->click_select_up(world,X,Y);
	}
	break;
	
      case ACTION_ADJUST:
	if (event_is_down(event)) 
	  ans = EXTERNAL_OBJECT_MODE->click_adjust_down(world,X,Y);       
	else 
	  ans = EXTERNAL_OBJECT_MODE->click_adjust_up(world,X,Y);	
	break;

      case LOC_DRAG:
	ans = EXTERNAL_OBJECT_MODE->loc_drag(world,X,Y);
	break;

      case LOC_MOVE:
	ans = EXTERNAL_OBJECT_MODE->loc_move(world,X,Y);
	break;
    }


    if (event_is_ascii(event))
      if (event_is_down(event))
	ans = EXTERNAL_OBJECT_MODE->ascii_key(world, event_action(event));
    
    return ans;
}

void EO_call_activator(World *w) {
    EXTERNAL_OBJECT_MODE->activator(w);
    if (EO_capture_events)
      w->add_event_proc(EO_events);    
}

void EO_call_deactivator(World *w) {
    EXTERNAL_OBJECT_MODE->deactivator(w);
    w->reset_event_proc();
}



void install_external_object_mode(World *w, ExternalObjectMode *Mode) {
    icon_bits *icon, *cursor;
    int xhot, yhot;

    if (EXTERNAL_OBJECT_MODE) {
	set_external_signal_handler();
	if (get_mode() == EXTRA_OBJECT)
	  EO_call_deactivator(w);
	EXTERNAL_OBJECT_MODE->uninstaller(w);
	set_internal_signal_handler();
    }

    set_external_signal_handler();
    Mode->installer(w);    
    icon = Mode->icon();
    cursor = Mode->cursor(xhot, yhot);
    set_internal_signal_handler();
    
    if (!icon) icon = EO_icon;
    extra_object_image = xv_find(XV_NULL, SERVER_IMAGE,
				   SERVER_IMAGE_BITS, icon,
				   SERVER_IMAGE_DEPTH, 1, 
				   XV_WIDTH, 32,
				   XV_HEIGHT, 32,
				   NULL);
    (void)xv_set(mode_panel, PANEL_CHOICE_IMAGE, EXTRA_OBJECT, extra_object_image, XV_NULL);

        if (cursor) {
	Server_image
	  extra_object_image = (Server_image)xv_create(NULL, SERVER_IMAGE,
					XV_WIDTH,		CURSOR_WIDTH,
					XV_HEIGHT,		CURSOR_HEIGHT,
					SERVER_IMAGE_BITS,	cursor,
					NULL);

	EXTERNAL_OBJECT_CURSOR = (Xv_Cursor)xv_create(NULL, CURSOR,
					CURSOR_IMAGE,		extra_object_image,
					CURSOR_XHOT,		xhot,
					CURSOR_YHOT,		yhot,
					NULL);
    }
    else
      EXTERNAL_OBJECT_CURSOR = 0;

    EO_capture_events = Mode->capture_events();

    EXTERNAL_OBJECT_MODE = Mode;
    
    if (get_mode() == EXTRA_OBJECT) {
	set_external_signal_handler();
	if (Mode->is_able_to_activate(objects_get_selected_objects(), objects_get_graphic_objects(), w)) {
	    if (EXTERNAL_OBJECT_CURSOR)
	      set_cursor(w, EXTERNAL_OBJECT_CURSOR);
	    else
	      set_cursor(w, EO_cursor);
	    EO_call_activator(w);
	    set_internal_signal_handler();
	}
	else {
	    set_internal_signal_handler();
	    set_select_mode(w);
	}
    }
}


void uninstall_external_object_mode(World *w) {
    if (EXTERNAL_OBJECT_MODE) {
	set_select_mode(w);
	set_external_signal_handler();
	EXTERNAL_OBJECT_MODE->uninstaller(w);
	set_internal_signal_handler();
    
	extra_object_image = xv_find(XV_NULL, SERVER_IMAGE,
				   SERVER_IMAGE_BITS, clean_icon,
				   SERVER_IMAGE_DEPTH, 1, 
				   XV_WIDTH, 32,
				   XV_HEIGHT, 32,
				   NULL);
    (void)xv_set(mode_panel, PANEL_CHOICE_IMAGE, EXTRA_OBJECT, extra_object_image, XV_NULL);
    
	EXTERNAL_OBJECT_MODE = 0;
    }
}


void install_external_function_mode(World *w, ExternalFunctionMode *Mode) {
    icon_bits *icon, *cursor;
    int xhot, yhot;

    if (EXTERNAL_FUNCTION_MODE) {
	set_external_signal_handler();
	if (get_mode() == EXTRA_OPMODE)
	  EF_call_deactivator(w);
	EXTERNAL_FUNCTION_MODE->uninstaller(w);
	set_internal_signal_handler();
    }

    set_external_signal_handler();
    Mode->installer(w);    
    icon = Mode->icon();
    cursor = Mode->cursor(xhot, yhot);
    set_internal_signal_handler();
    
    if (!icon) icon = EF_icon;
    extra_function_image = xv_find(XV_NULL, SERVER_IMAGE,
				   SERVER_IMAGE_BITS, icon,
				   SERVER_IMAGE_DEPTH, 1, 
				   XV_WIDTH, 32,
				   XV_HEIGHT, 32,
				   NULL);
    (void)xv_set(mode_panel, PANEL_CHOICE_IMAGE, EXTRA_OPMODE, extra_function_image, XV_NULL);

    if (cursor) {
	Server_image
	  extra_function_image = (Server_image)xv_create(NULL, SERVER_IMAGE,
					XV_WIDTH,		CURSOR_WIDTH,
					XV_HEIGHT,		CURSOR_HEIGHT,
					SERVER_IMAGE_BITS,	cursor,
					NULL);

	EXTERNAL_FUNCTION_CURSOR = (Xv_Cursor)xv_create(NULL, CURSOR,
					CURSOR_IMAGE,		extra_function_image,
					CURSOR_XHOT,		xhot,
					CURSOR_YHOT,		yhot,
					NULL);
    }
    else
      EXTERNAL_FUNCTION_CURSOR = 0;

    EF_capture_events = Mode->capture_events();

    EXTERNAL_FUNCTION_MODE = Mode;
    
    if (get_mode() == EXTRA_OPMODE) {
	set_external_signal_handler();
	if (Mode->is_able_to_activate(objects_get_selected_objects(), objects_get_graphic_objects(), w)) {
	    if (EXTERNAL_FUNCTION_CURSOR)
	      set_cursor(w, EXTERNAL_FUNCTION_CURSOR);
	    else
	      set_cursor(w, EF_cursor);
	    EF_call_activator(w);
	    set_internal_signal_handler();
	}
	else {
	    set_internal_signal_handler();
	    set_select_mode(w);
	}
    }    
}


void uninstall_external_function_mode(World *w) {
    if (EXTERNAL_FUNCTION_MODE) {
	set_select_mode(w);
	set_external_signal_handler();
	EXTERNAL_FUNCTION_MODE->uninstaller(w);
	set_internal_signal_handler();
    
	extra_function_image = xv_find(XV_NULL, SERVER_IMAGE,
				   SERVER_IMAGE_BITS, clean_icon,
				   SERVER_IMAGE_DEPTH, 1, 
				   XV_WIDTH, 32,
				   XV_HEIGHT, 32,
				   NULL);
    (void)xv_set(mode_panel, PANEL_CHOICE_IMAGE, EXTRA_OPMODE, extra_function_image, XV_NULL);
    
	EXTERNAL_FUNCTION_MODE = 0;
    }
}


void select_applyable_mode(World *w) {
    if (get_mode() == EXTRA_OBJECT && EXTERNAL_OBJECT_MODE) {
	set_external_signal_handler();
	if (! EXTERNAL_OBJECT_MODE->objects_list_has_changed(objects_get_selected_objects(), objects_get_graphic_objects(), w)) {
	    set_internal_signal_handler();
	    set_select_mode(w);
	}
	else
	  set_internal_signal_handler();
    }
    
    if (get_mode() == EXTRA_OPMODE && EXTERNAL_FUNCTION_MODE) {
	set_external_signal_handler();
	if (! EXTERNAL_FUNCTION_MODE->objects_list_has_changed(objects_get_selected_objects(), objects_get_graphic_objects(), w)) {
	    set_internal_signal_handler();
	    set_select_mode(w);
	}
	else
	  set_internal_signal_handler();
    }
	    
}

void set_cursor(World *w, Xv_Cursor cursor) {
    (void)xv_set(w->get_paint(), WIN_CURSOR, cursor, NULL);
}


void EF_exec_binder (World *w) {
    exec_binder(w, EXTERNAL_FUNCTION_MODE);
}

void EO_exec_binder (World *w) {
    exec_binder(w, EXTERNAL_OBJECT_MODE);
}

void EF_install_externally_created_object(GraphicObj_ptr obj) {
    install_externally_created_object(obj, EXTERNAL_FUNCTION_MODE);
}

void EO_install_externally_created_object(GraphicObj_ptr obj) {
    install_externally_created_object(obj, EXTERNAL_OBJECT_MODE);
}


/****************************************************************************/

void init_mode(Xv_opaque panel_choice, World *w, int value) {
    mode_panel = panel_choice;
    set_mode(w, value);
}

void set_select_mode(World *world) {
    // notifica algoritmos externos (modos de desenho e modos funcionais)
    // caso estejam sendo desativados atraves do metodo deactivator().    
    if (EXTERNAL_OBJECT_MODE && get_mode() == EXTRA_OBJECT) {
      set_external_signal_handler();      
      EO_call_deactivator(world);
      set_internal_signal_handler();
    }
    
    if (EXTERNAL_FUNCTION_MODE && get_mode() == EXTRA_OPMODE) {
      set_external_signal_handler();
      EF_call_deactivator(world);
      set_internal_signal_handler();
    }
    
    world->reset_event_proc();
    world->add_event_proc(select_events);
    set_cursor(world, select_cursor);
    set_mode(SELECT);
    (void)xv_set(mode_panel, PANEL_VALUE, SELECT, XV_NULL);
}

void set_mode(World *world, int value) {
    // notifica algoritmos externos (modos de desenho e modos funcionais)
    // caso estejam sendo desativados atraves do metodo deactivator().
    
    if (EXTERNAL_OBJECT_MODE && get_mode() == EXTRA_OBJECT) {
      set_external_signal_handler();      
      EO_call_deactivator(world);
      set_internal_signal_handler();
    }
    
    if (EXTERNAL_FUNCTION_MODE && get_mode() == EXTRA_OPMODE) {
      set_external_signal_handler();
      EF_call_deactivator(world);
      set_internal_signal_handler();
    }    
    
    world->reset_event_proc();
    set_cursor(world, insert_cursor);
    set_mode(value);

    switch (value) {
      case POINT:
        world->add_event_proc(insert_simple_events);
        world->add_event_proc(fixed_movement_events_points);
        break;

      case SEGMENT:
      case RAY:
      case LINE:
        world->add_event_proc(insert_simple_events);
        world->add_event_proc(fixed_movement_events);
        break;

      case RECTANGLE:
        world->add_event_proc(insert_simple_events);
        world->add_event_proc(change_mode_square_events);
        break;

      case ELLIPSE:
        world->add_event_proc(insert_simple_events);
        world->add_event_proc(change_mode_circle_events);
        break;

      case EXTRA_OBJECT:
        if (EXTERNAL_OBJECT_MODE) {
	    set_external_signal_handler();
	    if (EXTERNAL_OBJECT_MODE->is_able_to_activate(objects_get_selected_objects(), objects_get_graphic_objects(), world)) {
		if (EXTERNAL_OBJECT_CURSOR)
		  set_cursor(world, EXTERNAL_OBJECT_CURSOR);
		else
		  set_cursor(world, EO_cursor);
		EO_call_activator(world);
		set_internal_signal_handler();
	    }
	    else {
		set_internal_signal_handler();
		set_mode(NOMODE);
		set_select_mode(world);
	    }
	}
	break;
        
      case SELECT:
        world->add_event_proc(select_events);
        set_cursor(world, select_cursor);
        break;

      case MOVE_COPY:
        world->add_event_proc(move_copy_events);      
        world->add_event_proc(fixed_movement_events);
        world->add_event_proc(change_cursor_move_copy_events);
        set_cursor(world, move_cursor);
        world->add_event_proc(compress_drag_events);
        break;
         
      case RESIZE:
        world->add_event_proc(resize_events);    
        world->add_event_proc(fixed_movement_events);
        world->add_event_proc(change_cursor_resize_events);
        set_cursor(world, resizeop_cursor);
        world->add_event_proc(compress_drag_events);
        break;
        
      case ZOOM:
        zoom_init(world);
        world->add_event_proc(zoom_events);
        set_cursor(world, zoom_cursor);
        world->add_event_proc(compress_drag_events);
        break;
        
      case SCROLL:
        world->add_event_proc(scroll_events);
        world->add_event_proc(fixed_movement_events);
        set_cursor(world, scroll_cursor);
        world->add_event_proc(compress_drag_events);
        break;

      case DYNAMIC:
	world->add_event_proc(dynamic_toggle_events);
        world->add_event_proc(fixed_movement_events);
        set_cursor(world, dynamic_cursor);
        world->add_event_proc(compress_drag_events);
        break;

      case EXTRA_OPMODE:
        if (EXTERNAL_FUNCTION_MODE) {
	    set_external_signal_handler();
	    if (EXTERNAL_FUNCTION_MODE->is_able_to_activate(objects_get_selected_objects(), objects_get_graphic_objects(), world)) {
		if (EXTERNAL_FUNCTION_CURSOR)
		  set_cursor(world, EXTERNAL_FUNCTION_CURSOR);
		else
		  set_cursor(world, EF_cursor);
	    	EF_call_activator(world);
		set_internal_signal_handler();
	    }
	    else {
		set_internal_signal_handler();
		set_mode(NOMODE);
		set_select_mode(world);
	    }	    
        }
        break;
    }            
}

void set_mode(int value) {
    EDIT_MODE = value;
}

int get_mode() {
    return EDIT_MODE;
}

GraphicObj *new_object() {
  switch (get_mode()) {
    case POINT:
      return geolab_instantiation(Point2D_t)->new_GraphicObj();
      
    case SEGMENT:
      return geolab_instantiation(Segment2D_t)->new_GraphicObj();

    case RAY:
      return geolab_instantiation(Ray2D_t)->new_GraphicObj();
      
    case LINE:
      return geolab_instantiation(Line2D_t)->new_GraphicObj();
            
    case RECTANGLE:
      return geolab_instantiation(Rectangle_t)->new_GraphicObj();
      
    case SQUARE:
      return geolab_instantiation(Square_t)->new_GraphicObj();
      
    case ELLIPSE:
      return geolab_instantiation(Ellipse_t)->new_GraphicObj();

    case CIRCLE:
      return geolab_instantiation(Circle_t)->new_GraphicObj();
  }
}

bool compress_drag_events(World *world, Xv_Window, Event *event) {
    if (event_action(event) == LOC_DRAG)
      world->compress_drag(event);
      
    return false;
}

bool fixed_movement_events(World *, Xv_Window, Event *event) {
    static bool vert = false, horz = false;
    static int fixed_X, fixed_Y;
        
    switch (event_action(event)) {
      case SHIFT_LEFT:
      case SHIFT_RIGHT:
        fixed_X = event_x(event);
        fixed_Y = event_y(event);
        if (event_is_down(event))
          vert = horz = true;
        else
          vert = horz = false;
        break;
        
      case ACTION_SELECT:
      case ACTION_ADJUST:
      case ACTION_MENU:
        if (event_is_down(event) && event_shift_is_down(event)) {
          fixed_Y = event_y(event);
          fixed_X = event_x(event);
          vert = horz = true;
        }
        break;
        
      case LOC_DRAG:
        if (vert && horz) {
          int dx = abs(fixed_X - event_x(event));
          int dy = abs(fixed_Y - event_y(event));
          
          if (dx > dy && dx > DRAG_ACCURACY)
            vert = false;
          else
            if (dy > dx && dy > DRAG_ACCURACY)
              horz = false; 
        }
        break;
    }
    
    // Warning:
    //		O codigo abaixo e' possivelmente NAO-PORTAVEL e NAO-ESTAVEL
    //		dependendo de mudancas na versao do X11 ou XView

    if (event_shift_is_down(event)) {
      if (vert)
        event->ie_locx = fixed_X;
      if (horz)
        event->ie_locy = fixed_Y;
    }
      
    return false;
}

bool fixed_movement_events_points(World *, Xv_Window, Event *event) {
    static bool vert = false, horz = false;
    static int fixed_X, fixed_Y;
        
    switch (event_action(event)) {
      case SHIFT_LEFT:
      case SHIFT_RIGHT:
        fixed_X = event_x(event);
        fixed_Y = event_y(event);
        if (event_is_down(event))
          vert = horz = true;
        else
          vert = horz = false;
        break;
        
      case ACTION_SELECT:
      case ACTION_ADJUST:
      case ACTION_MENU:
        if (event_is_down(event) && event_shift_is_down(event)) {
          if (vert)
            fixed_Y = event_y(event);
          if (horz)
            fixed_X = event_x(event);
          vert = horz = true;
        }
        break;
        
      case LOC_DRAG:
        if (vert && horz) {
          int dx = abs(fixed_X - event_x(event));
          int dy = abs(fixed_Y - event_y(event));
          
          if (dx > dy && dx > DRAG_ACCURACY)
            vert = false;
          else
            if (dy > dx && dy > DRAG_ACCURACY)
              horz = false; 
        }
        break;
    }
    
    // Warning:
    //		O codigo abaixo e' possivelmente NAO-PORTAVEL e NAO-ESTAVEL
    //		dependendo de mudancas na versao do X11 ou XView

    if (event_shift_is_down(event)) {
      if (vert)
        event->ie_locx = fixed_X;
      if (horz)
        event->ie_locy = fixed_Y;
    }
      
    return false;
}

bool change_mode_square_events(World *, Xv_Window, Event *event) {
    switch (event_action(event)) {
      case SHIFT_LEFT:
      case SHIFT_RIGHT:
        if (event_is_down(event))
          set_mode(SQUARE);
        else
          set_mode(RECTANGLE);
        break;
    }
    
    return false;
}

bool change_mode_circle_events(World *, Xv_Window, Event *event) {
    switch (event_action(event)) {
      case SHIFT_LEFT:
      case SHIFT_RIGHT:
        if (event_is_down(event))
          set_mode(CIRCLE);
        else
          set_mode(ELLIPSE);
        break;
    }
    
    return false;
}

bool change_cursor_move_copy_events(World *w, Xv_Window, Event *event) {
    switch (event_action(event)) {
      case ACTION_ADJUST:
        if (event_is_down(event))
          set_cursor(w, copy_cursor);
        else
          set_cursor(w, move_cursor);
      break;
    }        

    return false;
}

bool change_cursor_resize_events(World *w, Xv_Window, Event *event) {
    switch (event_action(event)) {
      case ACTION_SELECT:
      case ACTION_ADJUST:
        if (event_is_down(event))
          set_cursor(w, resizecl_cursor);
        else
          set_cursor(w, resizeop_cursor);
      break;
    }        

    return false;
}

