/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti
                    Cesar N. Gon

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * geolab_stubs.C - Ligacao entre a interface e os componentes de aplicacao do
 * 		    ambiente GeoLab.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    09.12.91  19:00 	Welson R. Jacometti  	Arquivo criado
 *    13.01.93  14:00   C'esar N. Gon           M'odos Externos
 *
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <xview/notice.h>
#include <gdd.h>
#include "geolab_ui.h"
#include "algorithms.h"
#include "animation.h"
#include "edit_mode.h"
#include "line_settings.h"
#include "color.h"
#include "objects.h"
#include "input_tools.h"
#include "load_save.h"
#include "trapsignal.h"
#include "zoom.h"
#include "quit.h"
#include "compose.h"
#include "attributes.h"
#include "goodies_file.h"

//
// Instance XV_KEY_DATA key.  An instance is a set of related
// user interface objects.  A pointer to an object's instance
// is stored under this key in every object.  This must be a
// global variable.
//
Attr_attribute INSTANCE;

// Popup windows
geolab_popup_color_settings_objects		geolab_popup_color_settings;
geolab_popup_zoom_settings_objects		geolab_popup_zoom_settings;
geolab_popup_line_settings_objects		geolab_popup_line_settings;
geolab_popup_random_points_settings_objects	geolab_popup_random_points_settings;
geolab_popup_random_cover_settings_objects	geolab_popup_random_cover_settings;
geolab_popup_animation_settings_objects		geolab_popup_animation_settings;
geolab_popup_random_polygon_settings_objects	geolab_popup_random_polygon_settings;
geolab_popup_color_settings_bw_objects		geolab_popup_color_settings_bw;
geolab_popup_algorithms_settings_objects 	geolab_popup_algorithms_settings;
geolab_popup_algorithms_statistics_objects	geolab_popup_algorithms_statistics;
geolab_popup_algorithms_hackers_dream_objects	geolab_popup_algorithms_hackers_dream;
geolab_popup_edit_string_objects		geolab_popup_edit_string;
geolab_popup_object_attributes_objects		geolab_popup_object_attributes;
geolab_popup_compose_objects			geolab_popup_compose;

// interface
geolab_geolab_base_window_objects geolab_geolab_base_window;

void treat_parameters(int argc, char **argv);

void 
main(int argc, char **argv) {
    //
    // Initialize XView.
    //
    xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, 0);
    INSTANCE = xv_unique_key();

    //
    // Initialize user interface components.
    //
    geolab_geolab_base_window.objects_initialize(NULL);
    geolab_popup_color_settings.objects_initialize(geolab_geolab_base_window.geolab_base_window);
    //
    // Initialize color information
    //
    init_colors(geolab_popup_color_settings.controls_color_settings);

    //
    // Initialize default object attributes
    //

    Default_Obj_Attributes.set_default();
    FOREGROUND = Default_Obj_Attributes.FOREGROUND;
    BACKGROUND = Default_Obj_Attributes.BACKGROUND;
    LINE_WIDTH = Default_Obj_Attributes.LINE_WIDTH;
    LINE_STYLE = Default_Obj_Attributes.LINE_STYLE;

    //
    // Continue user interface components initilization.
    //
    geolab_popup_zoom_settings.objects_initialize(geolab_geolab_base_window.geolab_base_window);
    geolab_popup_line_settings.objects_initialize(geolab_geolab_base_window.geolab_base_window);
    geolab_popup_random_points_settings.objects_initialize(geolab_geolab_base_window.geolab_base_window);
    geolab_popup_random_cover_settings.objects_initialize(geolab_geolab_base_window.geolab_base_window);
    geolab_popup_animation_settings.objects_initialize(geolab_geolab_base_window.geolab_base_window);
    geolab_popup_random_polygon_settings.objects_initialize(geolab_geolab_base_window.geolab_base_window);
    geolab_popup_color_settings_bw.objects_initialize(geolab_geolab_base_window.geolab_base_window);
    geolab_popup_algorithms_settings.objects_initialize(geolab_geolab_base_window.geolab_base_window);
    geolab_popup_algorithms_statistics.objects_initialize(geolab_geolab_base_window.geolab_base_window);
    geolab_popup_algorithms_hackers_dream.objects_initialize(geolab_geolab_base_window.geolab_base_window);
    geolab_popup_edit_string.objects_initialize(geolab_geolab_base_window.geolab_base_window);
    geolab_popup_object_attributes.objects_initialize(geolab_geolab_base_window.geolab_base_window);
    geolab_popup_compose.objects_initialize(geolab_geolab_base_window.geolab_base_window);

    //
    // Desabilita,cao da fun,c~ao "Compose" (especial p/ o IMPA)
    //
    (void)xv_set(geolab_geolab_base_window.button_compose, PANEL_INACTIVE, TRUE, 0);

    
    //
    // Initialize color information
    //
    init_colors(geolab_popup_color_settings.controls_color_settings);

    //
    // Initialize the default mode
    //
    init_mode(geolab_geolab_base_window.setting_mode, geolab_geolab_base_window.world, 0);

    // 
    // Install signal handler
    //
    set_internal_signal_handler();

    //
    // Install interpose quit function
    //
    notify_interpose_destroy_func(geolab_geolab_base_window.geolab_base_window, (Notify_func)interpose_quit);

    //
    // Load file as parameter...
    //    

    treat_parameters(argc, argv);
    
    // 
    // Turn control over to XView.
    //
    xv_main_loop(geolab_geolab_base_window.geolab_base_window);

    //
    // Note: The code below refers to algorithm animation and the mechanisms
    //       to bypass the notifier (in order to provide event handling while
    //       executing algorithm's code).
    // 
    //       xv_main_loop is terminated either by Quit (normal termination) or
    //       by a call to an algorithm while animation option is active (the
    //       instruction used is notify_stop).
    //
    while(!TERMINATED) {
      animation_disable_interface(geolab_geolab_base_window.world);
	geolab_geolab_base_window.world->Flush();
      if (LAST_PLAY)
        compose_play_dispatcher(geolab_geolab_base_window.world);
      else {
	  dispatcher(geolab_geolab_base_window.world, LAST_ALGORITHM);
      }
	geolab_geolab_base_window.world->Flush();
	animation_restore_interface(geolab_geolab_base_window.world);
	geolab_geolab_base_window.world->Flush();
	notify_start();
    }

    uninstall_external_object_mode(geolab_geolab_base_window.world);
    uninstall_external_function_mode(geolab_geolab_base_window.world);
    geolab_geolab_base_window.world->reset_event_proc();
    objects_clear();
    close_dynamic_modules();
    delete geolab_geolab_base_window.world;
    exit(0);
}


//
// treat parameter "-f <file>"
//
void treat_parameters(int argc, char **argv) {
    register unsigned i;
    for (i=1; i<argc; i++) {
	if (!strcmp(argv[i],"-f")) {
	    if (file_exists("",argv[i+1]) ) {
		if (!load(geolab_geolab_base_window.world,"",argv[i+1]))
		  cerr << "load error in file " << argv[i+1] << endl;
	    }
	    else	    
	      cerr << "file " << argv[i+1] << " doesn't exist" << endl;
	    break;
	}
    }	
}


//
// Menu handler for `menu_file (New)'.
//
Menu_item
file_new_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    int new_op;
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        new_op = notice_prompt(ip->geolab_base_window, NULL,
          		NOTICE_MESSAGE_STRINGS,		"Warning: This operation will cause",
          						"all existing objects to be destroyed.", 
          						NULL,
          		NOTICE_BUTTON_NO,		"Confirm",
          		NOTICE_BUTTON_YES,		"Cancel",
          		NULL);
        if (new_op == NOTICE_NO) {
      	  objects_clear();
    	  objects_refresh(ip->world);
    	}
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Function load (supplied for goodies_file).
//
void
function_load(char *file, char *dir) {
    objects_clear();
    
    if (!load(geolab_geolab_base_window.world, dir, file)) {
	(void)notice_prompt(geolab_geolab_base_window.geolab_base_window, NULL,
			    NOTICE_MESSAGE_STRINGS,		"Ooops! Problems during load operation.",
			    NULL,
			NOTICE_BUTTON_YES,		"Continue",
			NULL);
    	objects_clear();
    }
    
    CURRENT_DIRECTORY_NAME = dir;
    CURRENT_FILE_NAME = file;
    
    objects_refresh(geolab_geolab_base_window.world);
    select_applyable_algorithms();
    select_applyable_mode(geolab_geolab_base_window.world);
}

//
// Menu handler for `menu_file (Load...)'.
//
Menu_item
file_load_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
      	break;

      case MENU_NOTIFY:
        goodies_choose_file(ip->geolab_base_window, 0, CURRENT_DIRECTORY_NAME, "****", "GeoLab: Load", "Load File", "Load", "Cancel", function_load);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_file (Save)'.
//
Menu_item
file_save_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    int save_op;
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        if (file_exists(CURRENT_DIRECTORY_NAME, CURRENT_FILE_NAME))
          save_op = notice_prompt(ip->geolab_base_window, NULL,
          		NOTICE_MESSAGE_STRINGS,		"File already exists. You can:",
          						NULL,
          		NOTICE_BUTTON_NO,		"Overwrite",
          		NOTICE_BUTTON_YES,		"Cancel",
          		NULL);
	else
	  save_op = NOTICE_NO;
	  
	if (save_op == NOTICE_NO && !save(CURRENT_DIRECTORY_NAME, CURRENT_FILE_NAME))
          (void)notice_prompt(ip->geolab_base_window, NULL,
          		NOTICE_MESSAGE_STRINGS,		"Ooops! Problems during save operation.",
          						NULL,
			NOTICE_BUTTON_YES,		"Continue",
			NULL);
    	break;
  
      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Function saveas (supplied for goodies_file).
//
void
function_saveas(char *file, char *dir) {
    if (!save(dir, file))
      (void)notice_prompt(geolab_geolab_base_window.geolab_base_window, NULL,
          		NOTICE_MESSAGE_STRINGS,		"Ooops! Problems during save operation.",
          						NULL,
			NOTICE_BUTTON_YES,		"Continue",
			NULL);
			
    CURRENT_DIRECTORY_NAME = dir;
    CURRENT_FILE_NAME = file;
}

//
// Menu handler for `menu_file (Save As...)'.
//
Menu_item
file_saveas_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
      	break;

      case MENU_NOTIFY:
        goodies_choose_file(ip->geolab_base_window, 1, CURRENT_DIRECTORY_NAME, "****", "GeoLab: Save As", "Save As", "Save", "Cancel", function_saveas);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Function include (supplied for goodies_file).
//
void
function_include(char *file, char *dir) {
    if (!load(geolab_geolab_base_window.world, dir, file)) {
      (void)notice_prompt(geolab_geolab_base_window.geolab_base_window, NULL,
	       		NOTICE_MESSAGE_STRINGS,		"Ooops! Problems during include operation.",
          						NULL,
			NOTICE_BUTTON_YES,		"Continue",
			NULL);
    	objects_clear();
    }
    
    CURRENT_DIRECTORY_NAME = dir;
    CURRENT_FILE_NAME = file;

    objects_repaint(geolab_geolab_base_window.world);
    select_applyable_algorithms();
    select_applyable_mode(geolab_geolab_base_window.world);
}

//
// Menu handler for `menu_file (Include...)'.
//
Menu_item
file_include_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
      	break;

      case MENU_NOTIFY:
        goodies_choose_file(ip->geolab_base_window, 0, CURRENT_DIRECTORY_NAME, "****", "GeoLab: Include", "Include File", "Include", "Cancel", function_include);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}


//
// Menu handler for `menu_file (About)'.
//
Menu_item
file_about_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        (void)notice_prompt(ip->geolab_base_window, NULL,
	       		NOTICE_MESSAGE_STRINGS,	
			"GeoLab is an environment for the development of",
			"algorithms for computational geometry, including",
			"facilities for algorithm animation.",
			"",
			"It is a result of work done by the following people:",
			"",
			"Design: ................. Pedro J. de Rezende",
			"            ................. Welson R. Jacometti",
			"",
			"Specification: .......... Pedro J. de Rezende",
			"                    .......... Welson R. Jacometti",
			"",
			"Implementation: ......... Welson R. Jacometti",
			"                ......... Cesar N. Gon",
			"",
                        "The environment as a whole is Copyrighted (1992-95)",
			"by Pedro J. de Rezende and Welson R. Jacometti;",
			"while individual source code files are Copyrighted",
			"by their respective authors as indicated therein.",
			"",
			"For more information, see the file named COPYRIGHT",
			"which describes the guarantees and conditions for free",
			"distribution.",
			"",
			"If that file is not available, you may obtain a copy",
			"by writing to",
			"geolab-request@dcc.unicamp.br",
          		NULL,
			NOTICE_BUTTON_YES,		"Continue",
			NULL);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_file (Quit)'.
//
Menu_item
file_quit_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        quit(ip->geolab_base_window);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_view (Refresh)'.
//
Menu_item
view_refresh_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        objects_refresh(ip->world);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_view (Zoom...)'.
//
Menu_item
view_zoom_settings_handler(Menu_item item, Menu_generate op) {
    static bool FIRST = true;
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        if (FIRST) {
          (void)xv_set(geolab_popup_zoom_settings.setting_zoom_center, PANEL_VALUE, ZOOM_CENTERED, 0); 
          (void)xv_set(geolab_popup_zoom_settings.setting_zoom_stack, PANEL_VALUE, ZOOM_STACKED, 0); 
          (void)xv_set(geolab_popup_zoom_settings.slider_zoom_factor, PANEL_VALUE, (int)((ZOOM_FACTOR - 1) * 100), 0); 
          (void)xv_set(geolab_popup_zoom_settings.slider_glass1_factor, PANEL_VALUE, (int)((GLASS1_FACTOR - 1) * 100), 0); 
          (void)xv_set(geolab_popup_zoom_settings.slider_glass2_factor, PANEL_VALUE, (int)((GLASS2_FACTOR - 1) * 100), 0);  
          FIRST = false;
        }
        (void)xv_set(geolab_popup_zoom_settings.popup_zoom_settings, WIN_SHOW, TRUE, WIN_FRONT, 0);
        break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_edit (Select All)'.
//
Menu_item
edit_select_all_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
    	objects_select_all(ip->world);
        if (objects_selected_empty())
          enable_all_algorithms();
        else
          select_applyable_algorithms();

        select_applyable_mode(ip->world);
        break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_edit (Delete)'.
//
Menu_item
edit_delete_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
    	objects_delete_selected(ip->world);
    	if (objects_selected_empty())
    	  enable_all_algorithms();
    	else
    	  select_applyable_algorithms();
    	  
    	select_applyable_mode(ip->world);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_edit (Undelete)'.
//
Menu_item
edit_undelete_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
    	objects_undelete_selected(ip->world);
    	if (objects_selected_empty())
    	  enable_all_algorithms();
    	else
    	  select_applyable_algorithms();
    	  
    	select_applyable_mode(ip->world);
    	break;
 
      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_input_tools'.
//
Menu
tools_handler(Menu menu, Menu_generate op) {
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return menu;
}


//
// Show object attributes popup.
// This function is used here and in module select.C 
//
void show_popup_object_attributes(Attributes& attr)
{
    (void)xv_set(geolab_popup_object_attributes.setting_object_attributes_deletable, PANEL_VALUE, attr.DELETABLE, 0);
    (void)xv_set(geolab_popup_object_attributes.setting_object_attributes_glue, PANEL_VALUE, attr.GLUE, 0);
    (void)xv_set(geolab_popup_object_attributes.setting_object_attributes_handles, PANEL_VALUE, attr.HANDLES, 0);
    (void)xv_set(geolab_popup_object_attributes.setting_object_attributes_line_width, PANEL_VALUE, get_index_from_line_width(attr.LINE_WIDTH), 0);
    (void)xv_set(geolab_popup_object_attributes.setting_object_attributes_line_style, PANEL_VALUE, get_index_from_line_style(attr.LINE_STYLE), 0);
    (void)xv_set(geolab_popup_object_attributes.setting_object_attributes_foreground_color, PANEL_VALUE, get_value_from_pixel(attr.FOREGROUND), 0);
    (void)xv_set(geolab_popup_object_attributes.setting_object_attributes_background_color, PANEL_VALUE, get_value_from_pixel(attr.BACKGROUND), 0);
    (void)xv_set(geolab_popup_object_attributes.setting_object_attributes_increment, PANEL_VALUE, attr.INCREMENT, 0);
    (void)xv_set(geolab_popup_object_attributes.textfield_object_attributes_increment, PANEL_VALUE, attr.INCREMENT_VALUE, 0);
    (void)xv_set(geolab_popup_object_attributes.textfield_object_attributes_increment, PANEL_INACTIVE, (attr.INCREMENT == 0), 0);
    (void)xv_set(geolab_popup_object_attributes.textfield_object_attributes_string, PANEL_VALUE, attr.LABEL.value(), 0);
    (void)xv_set(geolab_popup_object_attributes.popup_object_attributes, WIN_SHOW, TRUE, WIN_FRONT, 0);
}


//
// Menu handler for `menu_properties (Attributes...)'.
//
Menu_item
show_popup_object_attributes_handler(Menu_item item, Menu_generate op)
{
	geolab_geolab_base_window_objects * ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
		Obj_Attributes = Default_Obj_Attributes;
		show_popup_object_attributes(Obj_Attributes);
		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

//
// Menu handler for `menu_properties (Colors...)'.
//
Menu_item
settings_colors_properties_handler(Menu_item item, Menu_generate op) {
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        if (SCREEN_DEPTH == 1)
          (void)xv_set(geolab_popup_color_settings_bw.popup_color_settings_bw, WIN_SHOW, TRUE, WIN_FRONT, 0);
        else
          (void)xv_set(geolab_popup_color_settings.popup_color_settings, WIN_SHOW, TRUE, WIN_FRONT, 0);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_properties (Line Settings...)'.
//
Menu_item
settings_line_properties_handler(Menu_item item, Menu_generate op) {
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        (void)xv_set(geolab_popup_line_settings.popup_line_settings, WIN_SHOW, TRUE, WIN_FRONT, 0);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_random_points (Go  )'.
//
Menu_item
random_points_go_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects  *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);    
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
    	random_points_schedule (ip->world);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_random_points (Properties...)'.
//
Menu_item
random_points_properties_handler(Menu_item item, Menu_generate op) {
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        (void)xv_set(geolab_popup_random_points_settings.popup_random_points_settings, WIN_SHOW, TRUE, WIN_FRONT, 0);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_random_cover (Go)'.
//
Menu_item
random_cover_go_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects  *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
    	random_cover_schedule (ip->world);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_random_cover (Properties...)'.
//
Menu_item
random_cover_properties_handler(Menu_item item, Menu_generate op) {
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        (void)xv_set(geolab_popup_random_cover_settings.popup_random_cover_settings, WIN_SHOW, TRUE, WIN_FRONT, 0);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_view_handles_toggle (Toggle All)'.
//
Menu_item
menu_handles_toggle_all_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects  *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        HandlesOn = !HandlesOn;
  	objects_set_handle(ip->world, true, HandlesOn);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_view_handles_toggle (Selected On)'.
//
Menu_item
menu_handles_selected_on_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects  *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        objects_set_handle(ip->world, false, true);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_view_handles_toggle (Selected Off)'.
//
Menu_item
menu_handles_selected_off_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects  *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        objects_set_handle(ip->world, false, false);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_random_polygon (Go)'.
//
Menu_item
random_polygon_go_handler(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects  *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
	random_polygon_schedule (ip->world);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}    


//
// Menu handler for `menu_random_polygon (Properties...)'.
//
Menu_item
random_polygon_properties_handler(Menu_item item, Menu_generate op) {
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        (void)xv_set(geolab_popup_random_polygon_settings.popup_random_polygon_settings, WIN_SHOW, TRUE, WIN_FRONT, 0);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_view_menu_layers (Rotate All)'.
//
Menu_item
menu_view_menu_layers_rotate_all(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects  *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
        break;

      case MENU_DISPLAY_DONE:
        break;

      case MENU_NOTIFY:
        objects_rotate_all(ip->world);
	break;

      case MENU_NOTIFY_DONE:
	break;
    }
    return item;
}

//
// Menu handler for `menu_view_menu_layers (Selected Top)'.
//
Menu_item
menu_view_menu_layers_selected_top(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects  *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
	break;

      case MENU_DISPLAY_DONE:
	break;

      case MENU_NOTIFY:
        objects_selected_top(ip->world);
	break;

      case MENU_NOTIFY_DONE:
	break;
    }
    return item;
}

//
// Menu handler for `menu_view_menu_layers (Selected Bottom)'.
//
Menu_item
menu_view_menu_layers_selected_bottom(Menu_item item, Menu_generate op) {
    geolab_geolab_base_window_objects  *ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
	break;

      case MENU_DISPLAY_DONE:
 	break;

      case MENU_NOTIFY:
        objects_selected_bottom(ip->world);
	break;

      case MENU_NOTIFY_DONE:
 	break;
    }
    return item;
}

//
// Menu handler for `menu_properties_menu_objects (Glue)'.
//
Menu_item
edit_objects_glue_handler(Menu_item item, Menu_generate op) {
    switch (op) {
      case MENU_DISPLAY:
	break;

      case MENU_DISPLAY_DONE:
	break;

      case MENU_NOTIFY:
	objects_glue();
	break;

      case MENU_NOTIFY_DONE:
 	break;
    }
    return item;
}

//
// Menu handler for `menu_properties_menu_objects (Unglue)'.
//
Menu_item
edit_objects_unglue_handler(Menu_item item, Menu_generate op) {
    switch (op) {
      case MENU_DISPLAY:
	break;

      case MENU_DISPLAY_DONE:
	break;

      case MENU_NOTIFY:
        objects_unglue();
	break;

      case MENU_NOTIFY_DONE:
	break;
    }
    return item;
}

//
// Menu handler for `menu_properties_menu_objects (Not deletable)'.
//
edit_objects_anchor_handler(Menu_item item, Menu_generate op) {
    switch (op) {
      case MENU_DISPLAY:
	break;

      case MENU_DISPLAY_DONE:
	break;

      case MENU_NOTIFY:
        objects_anchor();
	break;

      case MENU_NOTIFY_DONE:
 	break;
    }
    return item;
}

//
// Menu handler for `menu_properties_menu_objects (Deletable)'.
//
Menu_item
edit_objects_unanchor_handler(Menu_item item, Menu_generate op) {
    switch (op) {
      case MENU_DISPLAY:
	break;

      case MENU_DISPLAY_DONE:
	break;

      case MENU_NOTIFY:
        objects_unanchor();
	break;

      case MENU_NOTIFY_DONE:
	break;
    }
    return item;
}

//
// Menu handler for `menu_properties (String...)'.
//
Menu_item
edit_objects_string_handler(Menu_item item, Menu_generate op) {
    Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        (void)xv_set(geolab_popup_edit_string.popup_edit_string, WIN_SHOW, TRUE, WIN_FRONT, 0);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_properties_menu_debug (Dump)'.
//
Menu_item
edit_objects_dump_handler(Menu_item item, Menu_generate op) {
    Xv_opaque ip = (Xv_opaque) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    switch (op) {
      case MENU_DISPLAY:
     	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
    	objects_dump();
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_algorithms (Settings...)'.
//
Menu_item
algorithms_settings_handler(Menu_item item, Menu_generate op) {
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        (void)xv_set(geolab_popup_algorithms_settings.popup_algorithms_settings, WIN_SHOW, TRUE, WIN_FRONT, 0);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_algorithms (Statistics...)'.
//
Menu_item
algorithms_statistics_handler(Menu_item item, Menu_generate op) {
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        (void)xv_set(geolab_popup_algorithms_statistics.popup_algorithms_statistics, WIN_SHOW, TRUE, WIN_FRONT, 0);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_properties_menu_debug (Hackers's Dream...)'.
//
Menu_item
algorithms_hackers_dream_handler(Menu_item item, Menu_generate op) {
    switch (op) {
      case MENU_DISPLAY:
    	break;

      case MENU_DISPLAY_DONE:
    	break;

      case MENU_NOTIFY:
        (void)xv_set(geolab_popup_algorithms_hackers_dream.popup_algorithms_hackers_dream, WIN_SHOW, TRUE, WIN_FRONT, 0);
    	break;

      case MENU_NOTIFY_DONE:
    	break;
    }
    return item;
}

//
// Menu handler for `menu_external_modes (Uninstall External Object Mode)'.
//
Menu_item
uninstall_EOM_handler(Menu_item item, Menu_generate op)
{
        geolab_geolab_base_window_objects	*ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);    	
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:		
		break;

	case MENU_NOTIFY:
		uninstall_external_object_mode(ip->world);
		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

//
// Menu handler for `menu_external_modes (Uninstall External Function Mode)'.
//
Menu_item
uninstall_EFM_handler(Menu_item item, Menu_generate op)
{
        geolab_geolab_base_window_objects	*ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);    	
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
		uninstall_external_function_mode(ip->world);
		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}


//
// Notify callback function for `button_animation'.
//
void
button_animation_handler(Panel_item item, Event *event)
{
    static bool FIRST = true;
    if (FIRST) {
	(void)xv_set(geolab_popup_animation_settings.button_animation_settings_pause_continue, PANEL_INACTIVE, TRUE, 0);
	(void)xv_set(geolab_popup_animation_settings.button_animation_settings_step, PANEL_INACTIVE, TRUE, 0);
	(void)xv_set(geolab_popup_animation_settings.button_animation_settings_fast_forward, PANEL_INACTIVE, TRUE, 0);
	FIRST = false;
    }
    (void)xv_set(geolab_popup_animation_settings.popup_animation_settings, WIN_SHOW, TRUE, WIN_FRONT, 0); 
}

//
// Notify callback function for `button_compose'.
//
void
button_compose_handler(Panel_item, Event*) {
    static bool FIRST = true;
	
    if (FIRST) {
      (void)xv_set(geolab_popup_compose.button_compose_stop, PANEL_INACTIVE, TRUE, 0);
      (void)xv_set(geolab_popup_compose.button_compose_play, PANEL_INACTIVE, TRUE, 0);
      FIRST = false;
    }
    (void)xv_set(geolab_popup_compose.popup_compose, WIN_SHOW, TRUE, WIN_FRONT, 0);
}

//
// Notify callback function for `setting_mode'.
//
void
mode_handler(Panel_item item, int value, Event *) {
    geolab_geolab_base_window_objects	*ip = (geolab_geolab_base_window_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);

    set_mode(ip->world, value);
}

//
// Event callback function for `geolab_canvas'.
//
bool
root_canvas_events(World *world, Xv_window win, Event *event) {
    geolab_geolab_base_window_objects	*ip = (geolab_geolab_base_window_objects *) xv_get(win, XV_KEY_DATA, INSTANCE);

    int ev_action = event_action(event);
    
    if (ev_action == LOC_MOVE || ev_action == LOC_DRAG) {
      static char virtual_xcoord[20];
      static char virtual_ycoord[20];	// to display the current coordinates

      int x = event_x(event);
      int y = event_y(event);
      (void)sprintf(virtual_xcoord, "%.2f", world->virtual_xcoord(x));
      (void)sprintf(virtual_ycoord, "%.2f", world->virtual_ycoord(y));
        	
      (void)xv_set(ip->const_message_virtualXposition_place_holder, PANEL_LABEL_STRING, virtual_xcoord, 0);
      (void)xv_set(ip->const_message_virtualYposition_place_holder, PANEL_LABEL_STRING, virtual_ycoord, 0);
    }
    
    if (ev_action == ACTION_DRAG_LOAD) {
      char name[MAXPATHLEN];

      if (gdd_get_drag_name(win, name) != -1) {
	fprintf(stderr, "geolab: root_canvas_events: load %s\n", name);
      }
    } 
    else 
      if (ev_action == ACTION_MENU && event_is_down(event)) {
        Menu menu = (Menu) xv_get(win, WIN_MENU);

	if (menu)
  	  menu_show(menu, win, event, 0);
      }
      else
        if (ev_action == WIN_RESIZE) {
          world->restart();
          objects_refresh(world);
        }

    // keyboard hot-keys facilities
    if (event_is_ascii(event)) {
      // cerr << "KEY:" << event_action(event) << endl;
      switch(event_action(event)) {	  
      	case 4:	// CTRL-D (delete)
      	  if (event_is_down(event)) {
            objects_delete_selected(world);
    	    if (objects_selected_empty())
    	      enable_all_algorithms();
    	    else
    	      select_applyable_algorithms();
  	 
  	    select_applyable_mode(world);
          }
          break;
   
        case 7:	// CTRL-G (undelete)
      	  if (event_is_down(event)) {
            objects_undelete_selected(world);
    	    if (objects_selected_empty())
    	      enable_all_algorithms();
    	    else
    	      select_applyable_algorithms();
  	  
  	    select_applyable_mode(world);
          }
          break;
          
        case 19:	// CTRL-S (select_all)
      	  if (event_is_down(event)) {
            objects_select_all(world);
    	    if (objects_selected_empty())
    	      enable_all_algorithms();
    	    else
    	      select_applyable_algorithms();
    	      
    	    select_applyable_mode(world);
    	  }
          break;
          
        case 18:	// CTRL-R (refresh)
      	  if (event_is_down(event))
            objects_refresh(world);
          break;
          
        case 8:		// CTRL-H (toggle handles)
      	  if (event_is_down(event)) {
            HandlesOn = !HandlesOn;
            objects_set_handle(world, true, HandlesOn);
          }
          break;
          
        case 9:		// TAB (rotate all)
      	  if (event_is_down(event))
            objects_rotate_all(world);
          break;
	case 17:        // CTRL-Q (quit quickly)
	  TERMINATED = true;
	  notify_stop();
      }
  }
    
    
    return false;
}

//
// Repaint callback function for `geolab_canvas'.
//
void
root_canvas_repaint_proc(World *world) {
    objects_repaint(world);
}

//
// Notify callback function for `setting_color'.
//
void
popup_color_handler(Panel_item, int value, Event *) {
    unsigned long selected_color = get_pixel_from_value(value);
    
    if (SET_FOREGROUND)
      Default_Obj_Attributes.FOREGROUND = FOREGROUND = selected_color;
    else 
      Default_Obj_Attributes.BACKGROUND = BACKGROUND = selected_color;
}

//
// Notify callback function for `setting_foreback'.
//
void
popup_foreback_handler(Panel_item item, int value, Event *) {
    geolab_popup_color_settings_objects	*ip = (geolab_popup_color_settings_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);

    SET_FOREGROUND = (value == 0);
    if (SET_FOREGROUND)
      (void)xv_set(ip->setting_color, PANEL_VALUE, get_value_from_pixel(Default_Obj_Attributes.FOREGROUND), 0);
    else
      (void)xv_set(ip->setting_color, PANEL_VALUE, get_value_from_pixel(Default_Obj_Attributes.BACKGROUND), 0);
}
    
//
// Notify callback function for `button_color_apply_to_selected'.
//
void
button_color_apply_to_selected_handler(Panel_item item, Event *) {
    geolab_popup_color_settings_objects	*ip = (geolab_popup_color_settings_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    Frame frame = (Frame)xv_get(ip->popup_color_settings, XV_OWNER);
    geolab_geolab_base_window_objects  *igp = (geolab_geolab_base_window_objects *) xv_get(frame, XV_KEY_DATA, INSTANCE);

    objects_set_color_selected(igp->world, Default_Obj_Attributes.FOREGROUND, Default_Obj_Attributes.BACKGROUND);
}

//
// Notify callback function for `button_zoom_reset'.
//
void
popup_zoom_reset(Panel_item item, Event *) {
    geolab_popup_zoom_settings_objects	*ip = (geolab_popup_zoom_settings_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    Frame frame = (Frame)xv_get(ip->popup_zoom_settings, XV_OWNER);
    geolab_geolab_base_window_objects  *igp = (geolab_geolab_base_window_objects *) xv_get(frame, XV_KEY_DATA, INSTANCE);

    zoom_reset(igp->world);	
}

//
// Notify callback function for `button_zoom_in'.
//
void
popup_zoom_in(Panel_item item, Event *) {
    geolab_popup_zoom_settings_objects	*ip = (geolab_popup_zoom_settings_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    Frame frame = (Frame)xv_get(ip->popup_zoom_settings, XV_OWNER);
    geolab_geolab_base_window_objects  *igp = (geolab_geolab_base_window_objects *) xv_get(frame, XV_KEY_DATA, INSTANCE);

    zoom_in(igp->world);	
}

//
// Notify callback function for `button_zoom_out'.
//
void
popup_zoom_out(Panel_item item, Event *) {
    geolab_popup_zoom_settings_objects	*ip = (geolab_popup_zoom_settings_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    Frame frame = (Frame)xv_get(ip->popup_zoom_settings, XV_OWNER);
    geolab_geolab_base_window_objects  *igp = (geolab_geolab_base_window_objects *) xv_get(frame, XV_KEY_DATA, INSTANCE);

    zoom_out(igp->world);	
}

//
// Notify callback function for `setting_zoom_center'.
//
void
popup_zoom_center_handler(Panel_item, int value, Event *) {
    ZOOM_CENTERED = (bool)value;
}

//
// Notify callback function for `setting_zoom_stack'.
//
void
popup_zoom_stack_handler(Panel_item, int value, Event *) {
    ZOOM_STACKED = (bool)value;
}

//
// Notify callback function for `slider_zoom_factor'.
//
void
popup_zoom_factor_handler(Panel_item, int value, Event *) {
    ZOOM_FACTOR = 1 + ((double)value / 100);
}

//
// Notify callback function for `slider_glass1_factor'.
//
void
popup_glass1_factor_handler(Panel_item, int value, Event *) {
    GLASS1_FACTOR = 1 + ((double)value / 100);
}

//
// Notify callback function for `slider_glass2_factor'.
//
void
popup_glass2_factor_handler(Panel_item, int value, Event *) {
    GLASS2_FACTOR = 1 + ((double)value / 100);
}

//
// Notify callback function for `setting_line_width'.
//
void
popup_line_width_handler(Panel_item, int value, Event *) {
    Default_Obj_Attributes.LINE_WIDTH = LINE_WIDTH = get_line_width_from_index(value);
}

//
// Notify callback function for `setting_line_style'.
//
void
popup_line_style_handler(Panel_item, int value, Event *) {
    Default_Obj_Attributes.LINE_STYLE = LINE_STYLE = get_line_style_from_index(value);
}

//
// Notify callback function for `button_line_settings_apply_to_selected'.
//
void
button_line_settings_apply_to_selected_handler(Panel_item item, Event *) {
    geolab_popup_line_settings_objects	*ip = (geolab_popup_line_settings_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    Frame frame = (Frame)xv_get(ip->popup_line_settings, XV_OWNER);
    geolab_geolab_base_window_objects  *igp = (geolab_geolab_base_window_objects *) xv_get(frame, XV_KEY_DATA, INSTANCE);

    objects_set_line_attr_selected(igp->world, Default_Obj_Attributes.LINE_WIDTH, Default_Obj_Attributes.LINE_STYLE);
}

//
// Notify callback function for `setting_all_region'.
//
void
popup_all_region_handler(Panel_item, int value, Event *) {
    RANDOM_POINTS_ALL_REGION = value;
}

//
// Notify callback function for `setting_discard_input_random_points'.
//
void
popup_points_discard_input_handler(Panel_item, int value, Event *) {
    RANDOM_POINTS_DISCARD_INPUT = value;
}

//
// Notify callback function for `setting_unselect_input_random_points'.
//
void
popup_unselect_input_region(Panel_item, int value, Event *) {
       RANDOM_POINTS_UNSELECT_INPUT = value;
}

//
// Notify callback function for `setting_points_selected_random_points'.
//
void
popup_points_selected_region_handler(Panel_item, int value, Event *) {
    RANDOM_POINTS_SELECTED = value;
}

//
// Notify callback function for `slider_scale_region'.
//
void
popup_scale_region_handler(Panel_item item, int value, Event *) {
    geolab_popup_random_points_settings_objects	*ip = (geolab_popup_random_points_settings_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    int number, max;
    
    RANDOM_POINTS_SCALE = value;
    number = (int) xv_get (ip->slider_number_points_region, PANEL_VALUE);
    max = (int) xv_get (ip->slider_number_points_region, PANEL_MAX_VALUE);
    xv_set (ip->slider_number_points_region, PANEL_MAX_VALUE, value*100, 0);
    RANDOM_POINTS_NUMBER = (number*value*100)/max;
    xv_set (ip->slider_number_points_region, PANEL_VALUE,RANDOM_POINTS_NUMBER, 0);
}

//
// Notify callback function for `slider_number_points_region'.
//
void
popup_number_points_region_handler(Panel_item, int value, Event *) {
    RANDOM_POINTS_NUMBER = value;
}

//
// Notify callback function for `slider_random_points_timeout'.
//
void
popup_timeout_handler(Panel_item, int value, Event *) {
    RANDOM_POINTS_TIMEOUT = value;
}

//
// Notify callback function for `button_random_points_go'.
//
void
popup_random_points_go(Panel_item item, Event *) {
    geolab_popup_random_points_settings_objects	*ip = (geolab_popup_random_points_settings_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    Frame frame = (Frame)xv_get(ip->popup_random_points_settings, XV_OWNER);
    geolab_geolab_base_window_objects  *igp = (geolab_geolab_base_window_objects *) xv_get(frame, XV_KEY_DATA, INSTANCE);

    random_points_schedule(igp->world);

}

//
// Notify callback function for `setting_discard_input_cover'.
//
void
popup_discard_input_cover_handler(Panel_item, int value, Event *) {
    COVER_DISCARD_INPUT = value;
}

//
// Notify callback function for `setting_unselect_input_cover'.
//
void
popup_unselect_input_cover_handler(Panel_item, int value, Event *) {
    COVER_UNSELECT_INPUT = value;
}

//
// Notify callback function for `setting_points_selected_cover'.
//
void
popup_points_selected_cover_handler(Panel_item, int value, Event *) {
    COVER_SELECTED = value;
}

//
// Notify callback function for `slider_scale_cover'.
//
void
popup_scale_cover_handler(Panel_item item, int value, Event *) {
    geolab_popup_random_cover_settings_objects	*ip = (geolab_popup_random_cover_settings_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);

    int number, max;
    
    COVER_SCALE = value;
    number = (int) xv_get (ip->slider_number_points_cover, PANEL_VALUE);
    max = (int) xv_get (ip->slider_number_points_cover, PANEL_MAX_VALUE);
    xv_set (ip->slider_number_points_cover, PANEL_MAX_VALUE, value*100, 0);
    COVER_POINTS = (number*value*100)/max;
    xv_set (ip->slider_number_points_cover, PANEL_VALUE,COVER_POINTS, 0);
}

//
// Notify callback function for `slider_number_points_cover'.
//
void
popup_number_points_cover_handler(Panel_item, int value, Event *) { 
    COVER_POINTS = value;
}

//
// Notify callback function for `slider_random_cover_timeout'.
//
void
popup_random_cover_timeout_handler(Panel_item, int value, Event *) {
    COVER_TIMEOUT = value;
}

//
// Notify callback function for `button_random_cover_go'.
//
void
popup_random_cover_go(Panel_item item, Event *) {
    geolab_popup_random_cover_settings_objects *ip = (geolab_popup_random_cover_settings_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    Frame frame = (Frame)xv_get(ip->popup_random_cover_settings, XV_OWNER);
    geolab_geolab_base_window_objects  *igp = (geolab_geolab_base_window_objects *) xv_get(frame, XV_KEY_DATA, INSTANCE);

    random_cover_schedule(igp->world);
}

//
// Notify callback function for `setting_animation_on_off'.
//
void
popup_animation_on_off_handler(Panel_item item, int value, Event *) {
    geolab_popup_animation_settings_objects *ip = (geolab_popup_animation_settings_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);

    if (value) {
      (void)xv_set(ip->button_animation_settings_pause_continue, PANEL_INACTIVE, FALSE, 0);
      if (PAUSE)
        (void)xv_set(geolab_popup_animation_settings.button_animation_settings_step, PANEL_INACTIVE, FALSE, 0);
      (void)xv_set(ip->button_animation_settings_fast_forward, PANEL_INACTIVE, FALSE, 0);
      REFERENCE_DEPTH = (int)xv_get(ip->slider_animation_depth, PANEL_VALUE);
      disable_algorithms_by_animation();
    }
    else {
      (void)xv_set(ip->button_animation_settings_pause_continue, PANEL_INACTIVE, TRUE, 0);
      (void)xv_set(ip->button_animation_settings_step, PANEL_INACTIVE, TRUE, 0);
      (void)xv_set(ip->button_animation_settings_fast_forward, PANEL_INACTIVE, TRUE, 0);
      REFERENCE_DEPTH = 0;
      select_applyable_algorithms();
    }
}

//
// Notify callback function for `slider_animation_depth'.
//
void
popup_animation_depth_handler(Panel_item item, int value, Event *) {
    geolab_popup_animation_settings_objects *ip = (geolab_popup_animation_settings_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    if ((int)xv_get(ip->setting_animation_on_off, PANEL_VALUE))
      REFERENCE_DEPTH = value;
}

//
// Notify callback function for `button_animation_settings_pause_continue'.
//
void
button_animation_settings_pause_continue_handler(Panel_item, Event *) {
    PAUSE = !PAUSE;
    FRAME_DELAY = (int)xv_get(geolab_popup_animation_settings.slider_animation_frame_delay, PANEL_VALUE);

    if (PAUSE) {
      (void)xv_set(geolab_popup_animation_settings.button_animation_settings_pause_continue, PANEL_LABEL_STRING, "Continue", 0);
      (void)xv_set(geolab_popup_animation_settings.button_animation_settings_step, PANEL_INACTIVE, FALSE, 0);
    }
    else {
      (void)xv_set(geolab_popup_animation_settings.button_animation_settings_pause_continue, PANEL_LABEL_STRING, "  Pause   ", 0); 
      (void)xv_set(geolab_popup_animation_settings.button_animation_settings_step, PANEL_INACTIVE, TRUE, 0);
    }
}

//
// Notify callback function for `button_animation_settings_step'.
//
void
button_animation_settings_step_handler(Panel_item, Event *) {
   STEP++;
}

//
// Notify callback function for `button_animation_settings_fast_forward'.
//
void
button_animation_settings_fast_forward(Panel_item, Event *) {
    FRAME_DELAY = 0;
    PAUSE = false;
    (void)xv_set(geolab_popup_animation_settings.button_animation_settings_pause_continue, PANEL_LABEL_STRING, "  Pause   ", 0); 
    (void)xv_set(geolab_popup_animation_settings.button_animation_settings_step, PANEL_INACTIVE, TRUE, 0);
}

//
// Notify callback function for `slider_animation_frame_delay'.
//
void
popup_frame_delay_handler(Panel_item, int value, Event *) {
    FRAME_DELAY = value;
}

//
// Notify callback function for `setting_random_polygon_discard_input'.
//
void
popup_polygon_discard_input(Panel_item, int value, Event *) {
    POLYGON_DISCARD_INPUT = value;
}

//
// Notify callback function for `setting_polygon_type'.
//
void
popup_polygon_type_handler(Panel_item, int value, Event *) {
    POLYGON_TYPE = value;
}

//
// Notify callback function for `setting_random_polygon_unselect_input'.
//
void
popup_polygon_unselect_input(Panel_item, int value, Event *) {
    POLYGON_UNSELECT_INPUT = value;
}

//
// Notify callback function for `setting_random_polygon_selected'.
//
void
popup_polygon_selected_handler(Panel_item, int value, Event *) {
    POLYGON_SELECTED = value;
}

//
// Notify callback function for `slider_random_polygon_timeout'.
//
void
popup_random_polygon_timeout_handler(Panel_item, int value, Event *) {
}

//  
// Notify callback function for `button_random_polygon_go'.
//
void
popup_random_polygon_go(Panel_item item, Event *) {
    geolab_popup_random_polygon_settings_objects *ip = (geolab_popup_random_polygon_settings_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    Frame frame = (Frame)xv_get(ip->popup_random_polygon_settings, XV_OWNER);
    geolab_geolab_base_window_objects  *igp = (geolab_geolab_base_window_objects *) xv_get(frame, XV_KEY_DATA, INSTANCE);

    random_polygon_schedule (igp->world);
}
	
//
// Notify callback function for `setting_algorithms_discard_input'.
//
void
popup_algorithms_discard_input_handler(Panel_item, int value, Event *) {
    DISCARD_INPUT = value;
}

//
// Notify callback function for `setting_algorithms_unselect_input'.
//
void
popup_algorithms_unselect_input_handler(Panel_item, int value, Event *) {
    UNSELECT_INPUT = value;
}

//
// Notify callback function for `setting_algorithms_output_selected'.
//
void
popup_algorithms_output_selected_handler(Panel_item, int value, Event *) {
    OUTPUT_SELECTED = value;
}

//
// Notify callback function for `setting_algorithms_discard_output'.
//
void
popup_algorithms_discard_output_handler(Panel_item, int value, Event *) {
    DYNAMIC_DISCARD_OUTPUT = value;
}

//
// Notify callback function for `setting_algorithms_dump_heap'.
//
void
popup_algorithms_dump_heap_handler(Panel_item, int value, Event *) {
    DUMP_HEAP = value;
}

//
// Notify callback function for `textfield_edit_string'.
//
Panel_setting
popup_edit_string_handler(Panel_item item, Event *event) {
    geolab_popup_edit_string_objects *ip = (geolab_popup_edit_string_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    Frame frame = (Frame)xv_get(ip->popup_edit_string, XV_OWNER);
    geolab_geolab_base_window_objects  *igp = (geolab_geolab_base_window_objects *) xv_get(frame, XV_KEY_DATA, INSTANCE);
    char *value = (char *) xv_get(item, PANEL_VALUE);

    objects_set_string(igp->world, value);
    
    return panel_text_notify(item, event);
}

//
// Notify callback function for `setting_object_attributes_deletable'.
//
void
popup_object_attributes_deletable_handler(Panel_item item, int	 value, Event *event)
{
    Obj_Attributes.DELETABLE = value;
}

//
// Notify callback function for `setting_object_attributes_line_width'.
//
void
popup_object_attributes_line_width_handler(Panel_item item, int	 value, Event *event)
{
    Obj_Attributes.LINE_WIDTH = get_line_width_from_index(value);
}

//
// Notify callback function for `setting_object_attributes_line_style'.
//
void
popup_object_attributes_line_style_handler(Panel_item item, int	 value, Event *event)
{
    Obj_Attributes.LINE_STYLE = get_line_style_from_index(value);
}

//
// Notify callback function for `setting_object_attributes_glue'.
//
void
popup_object_attributes_glue_handler(Panel_item item, int	 value, Event *event)
{
    Obj_Attributes.GLUE = value;
}

//
// Notify callback function for `setting_object_attributes_handles'.
//
void
popup_object_attributes_handles_handler(Panel_item item, int	 value, Event *event)
{
    Obj_Attributes.HANDLES = value;
}

//
// Notify callback function for `setting_object_attributes_foreground_color'.
//
void
popup_object_attributes_foreground_handler(Panel_item item, int	 value, Event *event)
{
    Obj_Attributes.FOREGROUND = get_pixel_from_value(value);    
}

//
// Notify callback function for `setting_object_attributes_background_color'.
//
void
popup_object_attributes_background_handler(Panel_item item, int	 value, Event *event)
{
    Obj_Attributes.BACKGROUND = get_pixel_from_value(value);
}


//
// Notify callback function for `setting_object_attributes_increment'.
//
void
popup_object_attributes_increment_handler(Panel_item item, int	 value, Event *event)
{
    geolab_popup_object_attributes_objects	*ip = (geolab_popup_object_attributes_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    
    Obj_Attributes.INCREMENT = value;
    if (value == 0) 
	(void)xv_set(ip->textfield_object_attributes_increment, PANEL_INACTIVE, TRUE, 0);
    else
      (void)xv_set(ip->textfield_object_attributes_increment, PANEL_INACTIVE, FALSE, 0);

}

//
// Notify callback function for `textfield_object_attributes_string'.
//
Panel_setting
popup_object_attributes_string_textfield_handler(Panel_item item, Event *event)
{
	geolab_popup_object_attributes_objects	*ip = (geolab_popup_object_attributes_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	char *	value = (char *) xv_get(item, PANEL_VALUE);

	Obj_Attributes.LABEL = value;
	
	return panel_text_notify(item, event);
}

//
// Notify callback function for `textfield_object_attributes_increment'.
//
Panel_setting
popup_object_attributes_increment_textfield_handler(Panel_item item, Event *event)
{
	geolab_popup_object_attributes_objects	*ip = (geolab_popup_object_attributes_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	int	value = (int) xv_get(item, PANEL_VALUE);
	
	Obj_Attributes.INCREMENT_VALUE = value;
	return panel_text_notify(item, event);
}

//
// Notify callback function for `button_object_attributes_apply_to_selected'.
//
void
popup_object_attributes_apply_handler(Panel_item item, Event *event)
{
    geolab_popup_object_attributes_objects	*ip = (geolab_popup_object_attributes_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    Frame frame = (Frame)xv_get(ip->popup_object_attributes, XV_OWNER);
    geolab_geolab_base_window_objects  *igp = (geolab_geolab_base_window_objects *) xv_get(frame, XV_KEY_DATA, INSTANCE);

    list(list_item)* SO=&objects_get_selected_objects();
    list(GraphicObj_ptr)* GO=&objects_get_graphic_objects();
    list_item it;
    GraphicObj_ptr obj;
    int count;
    String save_label(Obj_Attributes.LABEL);

    count = 0;
    forall(it, *SO) {
	obj = (*GO)[it];	
	if (Obj_Attributes.INCREMENT) {
	    char num[10];
	    sprintf (num, "%d", count);
	    count+=Obj_Attributes.INCREMENT_VALUE;
	    Obj_Attributes.LABEL = save_label + num;
	}
	obj->set_attributes(igp->world, Obj_Attributes);	
    }

    Obj_Attributes.LABEL = save_label;
	    
    objects_refresh(igp->world);
}

//
// Notify callback function for `button_object_attributes_set_default'.
//
void
popup_object_attributes_set_default_handler(Panel_item item, Event *event)
{
    Default_Obj_Attributes = Obj_Attributes;
    FOREGROUND = Default_Obj_Attributes.FOREGROUND;
    BACKGROUND = Default_Obj_Attributes.BACKGROUND;
    LINE_WIDTH = Default_Obj_Attributes.LINE_WIDTH;
    LINE_STYLE = Default_Obj_Attributes.LINE_STYLE;
}


//
// Notify callback function for `button_compose_start'.
//
void
popup_compose_start_handler(Panel_item item, Event*) {
    geolab_popup_compose_objects *ip = (geolab_popup_compose_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
	
    (void)xv_set(ip->button_compose_start, PANEL_INACTIVE, TRUE, 0);
    (void)xv_set(ip->button_compose_stop, PANEL_INACTIVE, FALSE, 0);
    (void)xv_set(ip->button_compose_play, PANEL_INACTIVE, TRUE, 0);
    
    compose_start();
}

//
// Notify callback function for `button_compose_stop'.
//
void
popup_compose_stop_handler(Panel_item item, Event*) {
    geolab_popup_compose_objects *ip = (geolab_popup_compose_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);

    (void)xv_set(ip->button_compose_start, PANEL_INACTIVE, FALSE, 0);
    (void)xv_set(ip->button_compose_stop, PANEL_INACTIVE, TRUE, 0);
    (void)xv_set(ip->button_compose_play, PANEL_INACTIVE, FALSE, 0);
    
    compose_finish();
}

//
// Notify callback function for `button_compose_play'.
//
void
popup_compose_play_handler(Panel_item item, Event *) {
    geolab_popup_compose_objects *ip = (geolab_popup_compose_objects *) xv_get(item, XV_KEY_DATA, INSTANCE);
    Frame frame = (Frame)xv_get(ip->popup_compose, XV_OWNER);
    geolab_geolab_base_window_objects  *igp = (geolab_geolab_base_window_objects *) xv_get(frame, XV_KEY_DATA, INSTANCE);
    
    compose_play(igp->world);
}

