/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * geolab_ui.C - implementacao dos metodos de inicializacao da interface 
 *		 do GeoLab.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    09.12.91  19:25 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "geolab_ui.h"
#include "color.h"
#include "algorithms.h"

char *GEOLAB_VERSION = "GeoLab 1.0";

//
// Create object `menu_file' in the specified instance.
//
Xv_opaque
geolab_menu_file_create(caddr_t *ip, Xv_opaque owner) {
    extern Menu_item file_new_handler(Menu_item, Menu_generate);
    extern Menu_item file_load_handler(Menu_item, Menu_generate);
    extern Menu_item file_save_handler(Menu_item, Menu_generate);
    extern Menu_item file_saveas_handler(Menu_item, Menu_generate);
    extern Menu_item file_include_handler(Menu_item, Menu_generate);
    extern Menu_item file_about_handler(Menu_item, Menu_generate);
    extern Menu_item file_quit_handler(Menu_item, Menu_generate);
    Xv_opaque obj;
	
    obj = xv_create(XV_NULL, MENU_COMMAND_MENU,
		XV_KEY_DATA, INSTANCE, ip,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "About...",
			MENU_GEN_PROC, file_about_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "New",
			MENU_GEN_PROC, file_new_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Load...",
			MENU_GEN_PROC, file_load_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Save",
			MENU_GEN_PROC, file_save_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Save As...",
			MENU_GEN_PROC, file_saveas_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Include...",
			MENU_GEN_PROC, file_include_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Quit",
			MENU_GEN_PROC, file_quit_handler,
			NULL,
		MENU_DEFAULT, 3,
		MENU_GEN_PIN_WINDOW, owner, "File",
		NULL);
    return obj;
}

//
// Create object `menu_view' in the specified instance.
//
Xv_opaque
geolab_menu_view_create(caddr_t *ip, Xv_opaque owner) {
    extern Menu_item view_refresh_handler(Menu_item, Menu_generate);
    extern Menu_item view_zoom_settings_handler(Menu_item, Menu_generate);
    Xv_opaque obj;
	
    obj = xv_create(XV_NULL, MENU_COMMAND_MENU,
		XV_KEY_DATA, INSTANCE, ip,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Refresh",
			MENU_GEN_PROC, view_refresh_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Handles",
			MENU_PULLRIGHT, geolab_menu_view_handles_toggle_create(ip, owner),
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Layers",
			MENU_PULLRIGHT, geolab_menu_view_menu_layers_create(ip, owner),
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Zoom...",
			MENU_GEN_PROC, view_zoom_settings_handler,
			NULL,
		MENU_DEFAULT, 2,
		MENU_GEN_PIN_WINDOW, owner, "View",
		NULL);
    return obj;
}

//
// Create object `menu_edit' in the specified instance.
//
Xv_opaque
geolab_menu_edit_create(caddr_t *ip, Xv_opaque owner) {
    extern Menu_item edit_select_all_handler(Menu_item, Menu_generate);
    extern Menu_item edit_delete_handler(Menu_item, Menu_generate);
    extern Menu_item edit_undelete_handler(Menu_item, Menu_generate);
    Xv_opaque obj;
	
    obj = xv_create(XV_NULL, MENU_COMMAND_MENU,
		XV_KEY_DATA, INSTANCE, ip,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Select All",
			MENU_GEN_PROC, edit_select_all_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Delete",
			MENU_GEN_PROC, edit_delete_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Undelete",
			MENU_GEN_PROC, edit_undelete_handler,
			NULL,
		MENU_GEN_PIN_WINDOW, owner, "Edit",
		NULL);
    return obj;
}

//
// Create object `menu_input_tools' in the specified instance.
//
Xv_opaque
geolab_menu_input_tools_create(caddr_t *ip, Xv_opaque owner) {
    extern Menu tools_handler(Menu, Menu_generate);
    Xv_opaque obj;
	
    obj = xv_create(XV_NULL, MENU_COMMAND_MENU,
		XV_KEY_DATA, INSTANCE, ip,
		MENU_GEN_PROC, tools_handler,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Random Points",
			MENU_PULLRIGHT, geolab_menu_random_points_create(ip, owner),
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Random Cover",
			MENU_PULLRIGHT, geolab_menu_random_cover_create(ip, owner),
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Random Polygon",
			MENU_PULLRIGHT, geolab_menu_random_polygon_create(ip, owner),
			NULL,
		MENU_DEFAULT, 2,
		MENU_GEN_PIN_WINDOW, owner, "Input Tools",
		NULL);
    return obj;
}


//
// Create object `menu_properties' in the specified instance.
//
Xv_opaque
geolab_menu_properties_create(caddr_t *ip, Xv_opaque owner)
{
	extern Menu_item	show_popup_object_attributes_handler(Menu_item, Menu_generate);
	extern Menu_item	settings_colors_properties_handler(Menu_item, Menu_generate);
	extern Menu_item	settings_line_properties_handler(Menu_item, Menu_generate);
	extern Menu_item	edit_objects_string_handler(Menu_item, Menu_generate);
	Xv_opaque	obj;
	
	obj = xv_create(XV_NULL, MENU_COMMAND_MENU,
		XV_KEY_DATA, INSTANCE, ip,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Attributes...",
			MENU_GEN_PROC, show_popup_object_attributes_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Quick Settings",
			MENU_PULLRIGHT, geolab_menu_properties_menu_objects_create(ip, owner),
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Colors...",
			MENU_GEN_PROC, settings_colors_properties_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Line Settings...",
			MENU_GEN_PROC, settings_line_properties_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "String...",
			MENU_GEN_PROC, edit_objects_string_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Debug",
			MENU_PULLRIGHT, geolab_menu_properties_menu_debug_create(ip, owner),
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "External Modes",
			MENU_PULLRIGHT, geolab_menu_external_modes_create(ip, owner),
			NULL,			
		MENU_DEFAULT, 2,
		MENU_GEN_PIN_WINDOW, owner, "Properties",
		NULL);
	return obj;
}

//
// Create object `menu_random_points' in the specified instance.
//
Xv_opaque
geolab_menu_random_points_create(caddr_t *ip, Xv_opaque) {
    extern Menu_item random_points_go_handler(Menu_item, Menu_generate);
    extern Menu_item random_points_properties_handler(Menu_item, Menu_generate);
    Xv_opaque obj;
	
    obj = xv_create(XV_NULL, MENU_COMMAND_MENU,
		XV_KEY_DATA, INSTANCE, ip,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Go  ",
			MENU_GEN_PROC, random_points_go_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Properties...",
			MENU_GEN_PROC, random_points_properties_handler,
			NULL,
		MENU_DEFAULT, 1,
		NULL);
    return obj;
}

//
// Create object `menu_random_cover' in the specified instance.
//
Xv_opaque
geolab_menu_random_cover_create(caddr_t *ip, Xv_opaque) {
    extern Menu_item random_cover_go_handler(Menu_item, Menu_generate);
    extern Menu_item random_cover_properties_handler(Menu_item, Menu_generate);
    Xv_opaque obj;
	
    obj = xv_create(XV_NULL, MENU_COMMAND_MENU,
		XV_KEY_DATA, INSTANCE, ip,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Go",
			MENU_GEN_PROC, random_cover_go_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Properties...",
			MENU_GEN_PROC, random_cover_properties_handler,
			NULL,
		MENU_DEFAULT, 1,
		NULL);
    return obj;
}

//
// Create object `menu_view_handles_toggle' in the specified instance.
//
Xv_opaque
geolab_menu_view_handles_toggle_create(caddr_t *ip, Xv_opaque owner) {
    extern Menu_item menu_handles_toggle_all_handler(Menu_item, Menu_generate);
    extern Menu_item menu_handles_selected_on_handler(Menu_item, Menu_generate);
    extern Menu_item menu_handles_selected_off_handler(Menu_item, Menu_generate);
    Xv_opaque obj;
	
    obj = xv_create(XV_NULL, MENU_COMMAND_MENU,
		XV_KEY_DATA, INSTANCE, ip,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Toggle All",
			MENU_GEN_PROC, menu_handles_toggle_all_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Selected On",
			MENU_GEN_PROC, menu_handles_selected_on_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Selected Off",
			MENU_GEN_PROC, menu_handles_selected_off_handler,
			NULL,
		MENU_DEFAULT, 2,
		MENU_GEN_PIN_WINDOW, owner, "",
		NULL);
    return obj;
}

//
// Create object `menu_random_polygon' in the specified instance.
//
Xv_opaque
geolab_menu_random_polygon_create(caddr_t *ip, Xv_opaque) {
    extern Menu_item random_polygon_go_handler(Menu_item, Menu_generate);
    extern Menu_item random_polygon_properties_handler(Menu_item, Menu_generate);
    Xv_opaque obj;
	
    obj = xv_create(XV_NULL, MENU_COMMAND_MENU,
		XV_KEY_DATA, INSTANCE, ip,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Go",
			MENU_GEN_PROC, random_polygon_go_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Properties...",
			MENU_GEN_PROC, random_polygon_properties_handler,
			NULL,
		MENU_DEFAULT, 1,
		NULL);
    return obj;
}

//
// Create object `menu_algorithms' in the specified instance.
//
Xv_opaque
geolab_menu_algorithms_create(caddr_t *ip, Xv_opaque owner) {
    return menu_algorithms(ip, owner);
}

//
// Create object `menu_view_menu_layers' in the specified instance.
//
Xv_opaque
geolab_menu_view_menu_layers_create(caddr_t *ip, Xv_opaque owner) {
    extern Menu_item menu_view_menu_layers_rotate_all(Menu_item, Menu_generate);
    extern Menu_item menu_view_menu_layers_selected_top(Menu_item, Menu_generate);
    extern Menu_item menu_view_menu_layers_selected_bottom(Menu_item, Menu_generate);
    Xv_opaque obj;
	
    obj = xv_create(XV_NULL, MENU_COMMAND_MENU,
		XV_KEY_DATA, INSTANCE, ip,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Rotate All",
			MENU_GEN_PROC, menu_view_menu_layers_rotate_all,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Selected Top",
			MENU_GEN_PROC, menu_view_menu_layers_selected_top,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Selected Bottom",
			MENU_GEN_PROC, menu_view_menu_layers_selected_bottom,
			NULL,
		MENU_GEN_PIN_WINDOW, owner, "Layers",
		NULL);
    return obj;
}


//
// Create object `menu_properties_menu_objects' in the specified instance.
//
Xv_opaque
geolab_menu_properties_menu_objects_create(caddr_t *ip, Xv_opaque owner)
{
	extern Menu_item	edit_objects_glue_handler(Menu_item, Menu_generate);
	extern Menu_item	edit_objects_unglue_handler(Menu_item, Menu_generate);
	extern Menu_item	edit_objects_anchor_handler(Menu_item, Menu_generate);
	extern Menu_item	edit_objects_unanchor_handler(Menu_item, Menu_generate);
	Xv_opaque	obj;
	
	obj = xv_create(XV_NULL, MENU_COMMAND_MENU,
		XV_KEY_DATA, INSTANCE, ip,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Glue",
			MENU_GEN_PROC, edit_objects_glue_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Unglue",
			MENU_GEN_PROC, edit_objects_unglue_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Not deletable",
			MENU_GEN_PROC, edit_objects_anchor_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Deletable",
			MENU_GEN_PROC, edit_objects_unanchor_handler,
			NULL,
		MENU_GEN_PIN_WINDOW, owner, "Attributes",
		NULL);
	return obj;
}

//
// Create object `menu_properties_menu_debug' in the specified instance.
//
Xv_opaque
geolab_menu_properties_menu_debug_create(caddr_t *ip, Xv_opaque owner)
{
	extern Menu_item	algorithms_hackers_dream_handler(Menu_item, Menu_generate);
	extern Menu_item	edit_objects_dump_handler(Menu_item, Menu_generate);
	Xv_opaque	obj;
	
	obj = xv_create(XV_NULL, MENU_COMMAND_MENU,
		XV_KEY_DATA, INSTANCE, ip,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Hackers's Dream...",
			MENU_GEN_PROC, algorithms_hackers_dream_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Dump",
			MENU_GEN_PROC, edit_objects_dump_handler,
			NULL,
		MENU_GEN_PIN_WINDOW, owner, "",
		NULL);
	return obj;
}

//
// Create object `menu_external_modes' in the specified instance.

//
Xv_opaque
geolab_menu_external_modes_create(caddr_t *ip, Xv_opaque owner)
{
	extern Menu_item	uninstall_EOM_handler(Menu_item, Menu_generate);
	extern Menu_item	uninstall_EFM_handler(Menu_item, Menu_generate);
	Xv_opaque	obj;
	
	obj = xv_create(XV_NULL, MENU_COMMAND_MENU,
		XV_KEY_DATA, INSTANCE, ip,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Uninstall External Object Mode",
			MENU_GEN_PROC, uninstall_EOM_handler,
			NULL,
		MENU_ITEM,
			XV_KEY_DATA, INSTANCE, ip,
			MENU_STRING, "Uninstall External Function Mode",
			MENU_GEN_PROC, uninstall_EFM_handler,
			NULL,
		MENU_GEN_PIN_WINDOW, owner, "External Modes",
		NULL);
	return obj;
}

//
// Initialize an instance of object `geolab_base_window'.
//
void
geolab_geolab_base_window_objects::objects_initialize(Xv_opaque owner) {
    geolab_base_window = geolab_base_window_create(owner);
    controls_base_panel_horizontal = controls_base_panel_horizontal_create(geolab_base_window);
    button_file = button_file_create(controls_base_panel_horizontal);
    button_view = button_view_create(controls_base_panel_horizontal);
    button_edit = button_edit_create(controls_base_panel_horizontal);
    button_properties = button_properties_create(controls_base_panel_horizontal);
    button_input_tools = button_input_tools_create(controls_base_panel_horizontal);
    button_algorithms = button_algorithms_create(controls_base_panel_horizontal);
    button_animation = button_animation_create(controls_base_panel_horizontal);
    button_compose = button_compose_create(controls_base_panel_horizontal);
    controls_base_panel_vertical = controls_base_panel_vertical_create(geolab_base_window);
    setting_mode = setting_mode_create(controls_base_panel_vertical);
    mouse_position_message = mouse_position_message_create(controls_base_panel_vertical);
    mouse_virtualXposition = mouse_virtualXposition_create(controls_base_panel_vertical);
    const_message_virtualXposition_place_holder = const_message_virtualXposition_place_holder_create(controls_base_panel_vertical);
    mouse_virtualYposition = mouse_virtualYposition_create(controls_base_panel_vertical);
    const_message_virtualYposition_place_holder = const_message_virtualYposition_place_holder_create(controls_base_panel_vertical);
    geolab_canvas = geolab_canvas_create(geolab_base_window);
    world = geolab_world_create(geolab_canvas);
}

//
// Create object `geolab_base_window' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::geolab_base_window_create(Xv_opaque owner) {
    Xv_opaque obj;
    Xv_opaque geolab_base_window_image;

    static unsigned short geolab_base_window_bits[] = {
#include "ICONS/geolab.icon"
};

    geolab_base_window_image = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, geolab_base_window_bits,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 64,
		XV_HEIGHT, 64,
		NULL);

    obj = xv_create(owner, FRAME,
		XV_KEY_DATA, INSTANCE, this,
		XV_WIDTH, 800,
		XV_HEIGHT, 600,
		XV_LABEL, GEOLAB_VERSION,
		FRAME_CLOSED, FALSE,
		FRAME_SHOW_FOOTER, TRUE,
		FRAME_SHOW_RESIZE_CORNER, TRUE,
		FRAME_ICON, xv_create(XV_NULL, ICON,
			ICON_IMAGE, geolab_base_window_image,
			NULL),
		NULL);
    return obj;
}

//
// Create object `controls_base_panel_horizontal' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::controls_base_panel_horizontal_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 0,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, 40,
		WIN_BORDER, FALSE,
		NULL);
    return obj;
}

//
// Create object `button_file' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::button_file_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 16,
		XV_Y, 8,
		XV_WIDTH, 54,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "File",
		PANEL_ITEM_MENU, geolab_menu_file_create((caddr_t *) this, geolab_base_window),
		NULL);
    return obj;
}
       
//
// Create object `button_view' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::button_view_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 80,
		XV_Y, 8,
		XV_WIDTH, 62,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "View",
		PANEL_ITEM_MENU, geolab_menu_view_create((caddr_t *) this, geolab_base_window),
		NULL);
    return obj;
}

//
// Create object `button_edit' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::button_edit_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 152,
		XV_Y, 8,
		XV_WIDTH, 56,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "Edit",
		PANEL_ITEM_MENU, geolab_menu_edit_create((caddr_t *) this, geolab_base_window),
		NULL);
    return obj;
}


//
// Create object `button_properties' in the specified instance.

//
Xv_opaque
geolab_geolab_base_window_objects::button_properties_create(Xv_opaque owner)
{
	Xv_opaque	obj;
	
	obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 220,
		XV_Y, 8,
		XV_WIDTH, 94,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "Properties",
		PANEL_ITEM_MENU, geolab_menu_properties_create((caddr_t *) this, geolab_base_window),
		NULL);
	return obj;
}


//
// Create object `button_input_tools' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::button_input_tools_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 376,
		XV_Y, 8,
		XV_WIDTH, 101,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "Input Tools",
		PANEL_ITEM_MENU, geolab_menu_input_tools_create((caddr_t *) this, geolab_base_window),
		NULL);
    return obj;
}

//
// Create object `button_algorithms' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::button_algorithms_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 488,
		XV_Y, 8,
		XV_WIDTH, 98,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "Algorithms",
		PANEL_ITEM_MENU, geolab_menu_algorithms_create((caddr_t *) this, geolab_base_window),
		NULL);
    return obj;
}


//
// Create object `button_animation' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::button_animation_create(Xv_opaque owner)
{
	extern void		button_animation_handler(Panel_item, Event *);
	Xv_opaque	obj;
	
	obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 596,
		XV_Y, 8,
		XV_WIDTH, 90,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "Animation...",
		PANEL_NOTIFY_PROC, button_animation_handler,
		NULL);
	return obj;
}


//
// Create object `button_compose' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::button_compose_create(Xv_opaque owner) {
    extern void button_compose_handler(Panel_item, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 696,
		XV_Y, 8,
		XV_WIDTH, 83,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "Compose...",
		PANEL_NOTIFY_PROC, button_compose_handler,
		NULL);
    return obj;
}

//
// Create object `controls_base_panel_vertical' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::controls_base_panel_vertical_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 40,
		XV_WIDTH, 116,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		WIN_BORDER, FALSE,
		NULL);
    return obj;
}

//
// Create object `setting_mode' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::setting_mode_create(Xv_opaque owner) {
    extern void mode_handler(Panel_item, int, Event *);
    Xv_opaque obj;

    Xv_opaque setting_mode_image0;
    static unsigned short setting_mode_bits0[] = {
#include "ICONS/point.icon"
};

    Xv_opaque setting_mode_image1;
    static unsigned short setting_mode_bits1[] = {
#include "ICONS/segment.icon"
};

    Xv_opaque setting_mode_image2;
    static unsigned short setting_mode_bits2[] = {
#include "ICONS/ray.icon"
};

    Xv_opaque setting_mode_image3;
    static unsigned short setting_mode_bits3[] = {
#include "ICONS/line.icon"
};

    Xv_opaque setting_mode_image4;
    static unsigned short setting_mode_bits4[] = {
#include "ICONS/rectangle.icon"
};

    Xv_opaque setting_mode_image5;
    static unsigned short setting_mode_bits5[] = {
#include "ICONS/ellipse.icon"
};

    Xv_opaque setting_mode_image6;
    static unsigned short setting_mode_bits6[] = {
#include "ICONS/place_holder.icon"
};

    Xv_opaque setting_mode_image7;
    static unsigned short setting_mode_bits7[] = {
#include "ICONS/select.icon"
};

    Xv_opaque setting_mode_image8;
    static unsigned short setting_mode_bits8[] = {
#include "ICONS/move_copy.icon"
};

    Xv_opaque setting_mode_image9;
    static unsigned short setting_mode_bits9[] = {
#include "ICONS/resize.icon"
};

    Xv_opaque setting_mode_image10;
    static unsigned short setting_mode_bits10[] = {
#include "ICONS/zoom.icon"
};

    Xv_opaque setting_mode_image11;
    static unsigned short setting_mode_bits11[] = {
#include "ICONS/scroll.icon"
};

    Xv_opaque setting_mode_image12;
    static unsigned short setting_mode_bits12[] = {
#include "ICONS/dynamic.icon"
};

    setting_mode_image0 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_mode_bits0,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 32,
		XV_HEIGHT, 32,
		NULL);

    setting_mode_image1 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_mode_bits1,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 32,
		XV_HEIGHT, 32,
		NULL);

    setting_mode_image2 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_mode_bits2,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 32,
		XV_HEIGHT, 32,
		NULL);

    setting_mode_image3 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_mode_bits3,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 32,
		XV_HEIGHT, 32,
		NULL);

    setting_mode_image4 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_mode_bits4,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 32,
		XV_HEIGHT, 32,
		NULL);

    setting_mode_image5 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_mode_bits5,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 32,
		XV_HEIGHT, 32,
		NULL);

    setting_mode_image6 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_mode_bits6,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 32,
		XV_HEIGHT, 32,
		NULL);

    setting_mode_image7 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_mode_bits7,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 32,
		XV_HEIGHT, 32,
		NULL);

    setting_mode_image8 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_mode_bits8,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 32,
		XV_HEIGHT, 32,
		NULL);

    setting_mode_image9 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_mode_bits9,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 32,
		XV_HEIGHT, 32,
		NULL);

    setting_mode_image10 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_mode_bits10,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 32,
		XV_HEIGHT, 32,
		NULL);

    setting_mode_image11 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_mode_bits11,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 32,
		XV_HEIGHT, 32,
		NULL);

    setting_mode_image12 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_mode_bits12,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 32,
		XV_HEIGHT, 32,
		NULL);

    obj = xv_create(owner, PANEL_CHOICE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 16,
		XV_Y, 12,
		XV_WIDTH, 84,
		XV_HEIGHT, 311,
		PANEL_VALUE_X, 16,
		PANEL_VALUE_Y, 29,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOICE_NROWS, 7,
		PANEL_NOTIFY_PROC, mode_handler,
		PANEL_CHOICE_IMAGES,
			setting_mode_image0,
			setting_mode_image1,
			setting_mode_image2,
			setting_mode_image3,
			setting_mode_image4,
			setting_mode_image5,
			setting_mode_image6,
			setting_mode_image7,
			setting_mode_image8,
			setting_mode_image9,
			setting_mode_image10,
			setting_mode_image11,
			setting_mode_image12,
			setting_mode_image6,
			0,
		NULL);
    return obj;
}

//
// Create object `mouse_position_message' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::mouse_position_message_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 15,
		XV_Y, 480,
		XV_WIDTH, 53,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Pointer:",
		PANEL_LABEL_BOLD, TRUE,
		NULL);
    return obj;
}

//
// Create object `mouse_virtualXposition' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::mouse_virtualXposition_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 10,
		XV_Y, 504,
		XV_WIDTH, 11,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "X:",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_virtualXposition_place_holder' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::const_message_virtualXposition_place_holder_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 25,
		XV_Y, 504,
		XV_WIDTH, 0,
		XV_HEIGHT, 0,
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `mouse_virtualYposition' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::mouse_virtualYposition_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 10,
		XV_Y, 524,
		XV_WIDTH, 11,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Y:",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_virtualYposition_place_holder' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::const_message_virtualYposition_place_holder_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 25,
		XV_Y, 524,
		XV_WIDTH, 0,
		XV_HEIGHT, 0,
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `geolab_canvas' in the specified instance.
//
Xv_opaque
geolab_geolab_base_window_objects::geolab_canvas_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, CANVAS,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 116,
		XV_Y, 40,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		CANVAS_X_PAINT_WINDOW, TRUE,
		NULL);

    xv_set(canvas_paint_window(obj), WIN_MENU, (Menu)xv_get(button_algorithms, PANEL_ITEM_MENU), NULL);

    xv_set(canvas_paint_window(obj), WIN_CONSUME_EVENTS,
    		WIN_NO_EVENTS,
    	   	WIN_ASCII_EVENTS, KBD_USE, KBD_DONE,
    		WIN_REPAINT,
    		WIN_RESIZE,
		WIN_MOUSE_BUTTONS,
		LOC_WINENTER,
		LOC_WINEXIT,
		LOC_DRAG,
		LOC_MOVE,
		NULL, NULL);

    xv_set(canvas_paint_window(obj), XV_KEY_DATA, INSTANCE, this, NULL);

    return obj;
}

World*
geolab_geolab_base_window_objects::geolab_world_create(Xv_opaque owner) {
    extern bool root_canvas_events(World *, Xv_window, Event *);
    extern void	root_canvas_repaint_proc(World *);

    World *wobj = new World((Canvas)owner);
    assert (wobj);

    wobj->set_event_proc(root_canvas_events);
    wobj->set_repaint_proc(root_canvas_repaint_proc);
        
    return wobj; 
}

//
// Initialize an instance of object `popup_color_settings'.
//
void
geolab_popup_color_settings_objects::objects_initialize(Xv_opaque owner) {
    popup_color_settings = popup_color_settings_create(owner);
    controls_color_settings = controls_color_settings_create(popup_color_settings);
    setting_color = setting_color_create(controls_color_settings);
    setting_foreback = setting_foreback_create(controls_color_settings);
    button_color_apply_to_selected = button_color_apply_to_selected_create(controls_color_settings);
}

//
// Create object `popup_color_settings' in the specified instance.
//
Xv_opaque
geolab_popup_color_settings_objects::popup_color_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, FRAME_CMD,
		XV_KEY_DATA, INSTANCE, this,
		XV_WIDTH, 274,
		XV_HEIGHT, 193,
		XV_LABEL, "GeoLab: Color Settings",
		XV_SHOW, FALSE,
		FRAME_SHOW_FOOTER, FALSE,
		FRAME_SHOW_RESIZE_CORNER, FALSE,
		FRAME_CMD_PUSHPIN_IN, TRUE,
		NULL);

    xv_set(xv_get(obj, FRAME_CMD_PANEL), WIN_SHOW, FALSE, NULL);
    return obj;
}

//
// Create object `controls_color_settings' in the specified instance.
//
Xv_opaque
geolab_popup_color_settings_objects::controls_color_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 0,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		WIN_BORDER, FALSE,
		NULL);
    gcm_initialize_colors(obj, NULL, NULL);
    return obj;
}

//
// Create object `setting_color' in the specified instance.
//
Xv_opaque
geolab_popup_color_settings_objects::setting_color_create(Xv_opaque owner) {
    extern void popup_color_handler(Panel_item, int, Event *);
    Xv_opaque obj;
    
    Xv_opaque setting_color_image0;
    static unsigned short setting_color_bits0[] = {
#include "ICONS/color.icon"
};
	
    setting_color_image0 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_color_bits0,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 16,
		XV_HEIGHT, 16,
		NULL);

    obj = xv_create(owner, PANEL_CHOICE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 12,
		XV_Y, 20,
		XV_WIDTH, 110,
		XV_HEIGHT, 101,
		PANEL_VALUE_X, 21,
		PANEL_VALUE_Y, 20,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_CHOICE_NROWS, 4,
		PANEL_NOTIFY_PROC, popup_color_handler,
		PANEL_CHOICE_IMAGES,
			setting_color_image0,
			setting_color_image0,
			setting_color_image0,
			setting_color_image0,
			setting_color_image0,
			setting_color_image0,
			setting_color_image0,
			setting_color_image0,
			setting_color_image0,
			setting_color_image0,
			setting_color_image0,
			setting_color_image0,
			setting_color_image0,
			setting_color_image0,
			setting_color_image0,
			setting_color_image0,
			0,
		PANEL_CHOICE_COLOR,  0, gcm_color_index("Black"),
		PANEL_CHOICE_COLOR,  1, gcm_color_index("Gray"),
		PANEL_CHOICE_COLOR,  2, gcm_color_index("Cyan"),
		PANEL_CHOICE_COLOR,  3, gcm_color_index("Blue"),
		PANEL_CHOICE_COLOR,  4, gcm_color_index("Sienna"),
		PANEL_CHOICE_COLOR,  5, gcm_color_index("Red"),
		PANEL_CHOICE_COLOR,  6, gcm_color_index("Magenta"),
		PANEL_CHOICE_COLOR,  7, gcm_color_index("Yellow"),
		PANEL_CHOICE_COLOR,  8, gcm_color_index("Orange"),
		PANEL_CHOICE_COLOR,  9, gcm_color_index("Medium Sea Green"),
		PANEL_CHOICE_COLOR,  10, gcm_color_index("Dark Green"),
		PANEL_CHOICE_COLOR,  11, gcm_color_index("Green"),
		PANEL_CHOICE_COLOR,  12, gcm_color_index("Goldenrod"),
		PANEL_CHOICE_COLOR,  13, gcm_color_index("Gold"),
		PANEL_CHOICE_COLOR,  14, gcm_color_index("Khaki"),
		PANEL_CHOICE_COLOR,  15, gcm_color_index("White"),
		NULL);
    return obj;
}

//
// Create object `setting_foreback' in the specified instance.
//
Xv_opaque
geolab_popup_color_settings_objects::setting_foreback_create(Xv_opaque owner) {
    extern void popup_foreback_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 144,
		XV_Y, 48,
		XV_WIDTH, 113,
		XV_HEIGHT, 52,
		PANEL_VALUE_X, 153,
		PANEL_VALUE_Y, 48,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NCOLS, 1,
		PANEL_NOTIFY_PROC, popup_foreback_handler,
		PANEL_CHOICE_STRINGS,
			"Foreground",
			"Background",
			0,
		NULL);
    return obj;
}

//
// Create object `button_color_apply_to_selected' in the specified instance.
//
Xv_opaque
geolab_popup_color_settings_objects::button_color_apply_to_selected_create(Xv_opaque owner) {
    extern void button_color_apply_to_selected_handler(Panel_item, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 76,
		XV_Y, 152,
		XV_WIDTH, 124,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "Apply to Selected",
		PANEL_NOTIFY_PROC, button_color_apply_to_selected_handler,
		NULL);
    return obj;
}

//
// Initialize an instance of object `popup_zoom_settings'.
//
void
geolab_popup_zoom_settings_objects::objects_initialize(Xv_opaque owner) {
    popup_zoom_settings = popup_zoom_settings_create(owner);
    controls_zoom_settings = controls_zoom_settings_create(popup_zoom_settings);
    button_zoom_reset = button_zoom_reset_create(controls_zoom_settings);
    button_zoom_in = button_zoom_in_create(controls_zoom_settings);
    button_zoom_out = button_zoom_out_create(controls_zoom_settings);
    setting_zoom_center = setting_zoom_center_create(controls_zoom_settings);
    setting_zoom_stack = setting_zoom_stack_create(controls_zoom_settings);
    slider_zoom_factor = slider_zoom_factor_create(controls_zoom_settings);
    slider_glass1_factor = slider_glass1_factor_create(controls_zoom_settings);
    slider_glass2_factor = slider_glass2_factor_create(controls_zoom_settings);
}

//
// Create object `popup_zoom_settings' in the specified instance.
//
Xv_opaque
geolab_popup_zoom_settings_objects::popup_zoom_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, FRAME_CMD,
		XV_KEY_DATA, INSTANCE, this,
		XV_WIDTH, 361,
		XV_HEIGHT, 220,
		XV_LABEL, "GeoLab: Zoom settings",
		XV_SHOW, FALSE,
		FRAME_SHOW_FOOTER, TRUE,
		FRAME_SHOW_RESIZE_CORNER, FALSE,
		FRAME_CMD_PUSHPIN_IN, TRUE,
		NULL);

    xv_set(xv_get(obj, FRAME_CMD_PANEL), WIN_SHOW, FALSE, NULL);
    return obj;
}

//
// Create object `controls_zoom_settings' in the specified instance.
//
Xv_opaque
geolab_popup_zoom_settings_objects::controls_zoom_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 0,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		WIN_BORDER, FALSE,
		NULL);
    return obj;
}

//
// Create object `button_zoom_reset' in the specified instance.
//
Xv_opaque
geolab_popup_zoom_settings_objects::button_zoom_reset_create(Xv_opaque owner) {
    extern void popup_zoom_reset(Panel_item, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 24,
		XV_Y, 12,
		XV_WIDTH, 51,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "Reset",
		PANEL_NOTIFY_PROC, popup_zoom_reset,
		NULL);
    return obj;
}

//
// Create object `button_zoom_in' in the specified instance.
//
Xv_opaque
geolab_popup_zoom_settings_objects::button_zoom_in_create(Xv_opaque owner) {
    extern void popup_zoom_in(Panel_item, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 88,
		XV_Y, 12,
		XV_WIDTH, 68,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "Zoom In",
		PANEL_NOTIFY_PROC, popup_zoom_in,
		NULL);
    return obj;
}

//
// Create object `button_zoom_out' in the specified instance.
//
Xv_opaque
geolab_popup_zoom_settings_objects::button_zoom_out_create(Xv_opaque owner) {
    extern void popup_zoom_out(Panel_item, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 168,
		XV_Y, 12,
		XV_WIDTH, 79,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "Zoom Out",
		PANEL_NOTIFY_PROC, popup_zoom_out,
		NULL);
    return obj;
}

//
// Create object `setting_zoom_center' in the specified instance.
//
Xv_opaque
geolab_popup_zoom_settings_objects::setting_zoom_center_create(Xv_opaque owner) {
    extern void popup_zoom_center_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 40,
		XV_Y, 52,
		XV_WIDTH, 128,
		XV_HEIGHT, 69,
		PANEL_VALUE_X, 40,
		PANEL_VALUE_Y, 69,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NCOLS, 1,
		PANEL_LABEL_STRING, "Zoom Center",
		PANEL_NOTIFY_PROC, popup_zoom_center_handler,
		PANEL_CHOICE_STRINGS,
			"Mouse Position",
			"Window Center",
			0,
		NULL);
    return obj;
}

//
// Create object `setting_zoom_stack' in the specified instance.
//
Xv_opaque
geolab_popup_zoom_settings_objects::setting_zoom_stack_create(Xv_opaque owner) {
    extern void	popup_zoom_stack_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 192,
		XV_Y, 52,
		XV_WIDTH, 125,
		XV_HEIGHT, 69,
		PANEL_VALUE_X, 192,
		PANEL_VALUE_Y, 69,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NCOLS, 1,
		PANEL_LABEL_STRING, "Save Zoom Values",
		PANEL_NOTIFY_PROC, popup_zoom_stack_handler,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes, please",
			0,
		NULL);
    return obj;
}

//
// Create object `slider_zoom_factor' in the specified instance.
//
Xv_opaque
geolab_popup_zoom_settings_objects::slider_zoom_factor_create(Xv_opaque owner) {
    extern void	popup_zoom_factor_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_SLIDER,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 16,
		XV_Y, 136,
		XV_WIDTH, 330,
		XV_HEIGHT, 20,
		PANEL_VALUE_X, 132,
		PANEL_VALUE_Y, 136,
		PANEL_SLIDER_WIDTH, 100,
		PANEL_LABEL_STRING, "Zoom Factor (%)",
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_DIRECTION, PANEL_HORIZONTAL,
		PANEL_SLIDER_END_BOXES, FALSE,
		PANEL_SHOW_RANGE, TRUE,
		PANEL_SHOW_VALUE, TRUE,
		PANEL_MIN_VALUE, 0,
		PANEL_MAX_VALUE, 800,
		PANEL_TICKS, 9,
		PANEL_NOTIFY_PROC, popup_zoom_factor_handler,
		NULL);
    return obj;
}

//
// Create object `slider_glass1_factor' in the specified instance.
//
Xv_opaque
geolab_popup_zoom_settings_objects::slider_glass1_factor_create(Xv_opaque owner) {
    extern void popup_glass1_factor_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_SLIDER,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 17,
		XV_Y, 164,
		XV_WIDTH, 329,
		XV_HEIGHT, 20,
		PANEL_VALUE_X, 132,
		PANEL_VALUE_Y, 164,
		PANEL_SLIDER_WIDTH, 100,
		PANEL_LABEL_STRING, "Small Glass (%)",
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_DIRECTION, PANEL_HORIZONTAL,
		PANEL_SLIDER_END_BOXES, FALSE,
		PANEL_SHOW_RANGE, TRUE,
		PANEL_SHOW_VALUE, TRUE,
		PANEL_MIN_VALUE, 0,
		PANEL_MAX_VALUE, 800,
		PANEL_TICKS, 9,
		PANEL_NOTIFY_PROC, popup_glass1_factor_handler,
		NULL);
    return obj;
}

//
// Create object `slider_glass2_factor' in the specified instance.
//
Xv_opaque
geolab_popup_zoom_settings_objects::slider_glass2_factor_create(Xv_opaque owner) {
    extern void popup_glass2_factor_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_SLIDER,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 34,
		XV_Y, 192,
		XV_WIDTH, 312,
		XV_HEIGHT, 20,
		PANEL_VALUE_X, 132,
		PANEL_VALUE_Y, 192,
		PANEL_SLIDER_WIDTH, 100,
		PANEL_LABEL_STRING, "Big Glass (%)",
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_DIRECTION, PANEL_HORIZONTAL,
		PANEL_SLIDER_END_BOXES, FALSE,
		PANEL_SHOW_RANGE, TRUE,
		PANEL_SHOW_VALUE, TRUE,
		PANEL_MIN_VALUE, 0,
		PANEL_MAX_VALUE, 800,
		PANEL_TICKS, 9,
		PANEL_NOTIFY_PROC, popup_glass2_factor_handler,
		NULL);
    return obj;
}

//
// Initialize an instance of object `popup_line_settings'.
//
void
geolab_popup_line_settings_objects::objects_initialize(Xv_opaque owner) {
    popup_line_settings = popup_line_settings_create(owner);
    controls_line_settings = controls_line_settings_create(popup_line_settings);
    setting_line_width = setting_line_width_create(controls_line_settings);
    setting_line_style = setting_line_style_create(controls_line_settings);
    button_line_settings_apply_to_selected = button_line_settings_apply_to_selected_create(controls_line_settings);
}

//
// Create object `popup_line_settings' in the specified instance.
//
Xv_opaque
geolab_popup_line_settings_objects::popup_line_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, FRAME_CMD,
		XV_KEY_DATA, INSTANCE, this,
		XV_WIDTH, 333,
		XV_HEIGHT, 181,
		XV_LABEL, "GeoLab: Line Settings",
		XV_SHOW, FALSE,
		FRAME_SHOW_FOOTER, FALSE,
		FRAME_SHOW_RESIZE_CORNER, FALSE,
		FRAME_CMD_PUSHPIN_IN, TRUE,
		NULL);

    xv_set(xv_get(obj, FRAME_CMD_PANEL), WIN_SHOW, FALSE, NULL);
    return obj;
}

//
// Create object `controls_line_settings' in the specified instance.
//
Xv_opaque
geolab_popup_line_settings_objects::controls_line_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 0,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		WIN_BORDER, FALSE,
		NULL);
    return obj;
}

//
// Create object `setting_line_width' in the specified instance.
//
Xv_opaque
geolab_popup_line_settings_objects::setting_line_width_create(Xv_opaque owner) {
    extern void popup_line_width_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 40,
		XV_Y, 24,
		XV_WIDTH, 78,
		XV_HEIGHT, 69,
		PANEL_VALUE_X, 40,
		PANEL_VALUE_Y, 41,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NCOLS, 1,
		PANEL_LABEL_STRING, "Line Width:",
		PANEL_NOTIFY_PROC, popup_line_width_handler,
		PANEL_CHOICE_STRINGS, 
		        "Normal",
			"Bold",
			0,
		NULL);
    return obj;
}

//
// Create object `setting_line_style' in the specified instance.
//
Xv_opaque
geolab_popup_line_settings_objects::setting_line_style_create(Xv_opaque owner) {
    extern void popup_line_style_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 168,
		XV_Y, 24,
		XV_WIDTH, 126,
		XV_HEIGHT, 98,
		PANEL_VALUE_X, 168,
		PANEL_VALUE_Y, 41,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0, 
		PANEL_CHOICE_NCOLS, 1,
		PANEL_LABEL_STRING, "Line Style:",
		PANEL_NOTIFY_PROC, popup_line_style_handler,
		PANEL_CHOICE_STRINGS,
			"Solid",
			"Dashed",
			"Double Dashed",
			0,
		NULL);
    return obj;
}

//
// Create object `button_line_settings_apply_to_selected' in the specified instance.
//
Xv_opaque
geolab_popup_line_settings_objects::button_line_settings_apply_to_selected_create(Xv_opaque owner) {
    extern void button_line_settings_apply_to_selected_handler(Panel_item, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 104,
		XV_Y, 144,
		XV_WIDTH, 124,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "Apply to Selected",
		PANEL_NOTIFY_PROC, button_line_settings_apply_to_selected_handler,
		NULL);
    return obj;
}

//
// Initialize an instance of object `popup_random_points_settings'.
//
void
geolab_popup_random_points_settings_objects::objects_initialize(Xv_opaque owner) {
    popup_random_points_settings = popup_random_points_settings_create(owner);
    controls_random_points_settings = controls_random_points_settings_create(popup_random_points_settings);
    setting_all_region = setting_all_region_create(controls_random_points_settings);
    setting_discard_input_random_points = setting_discard_input_random_points_create(controls_random_points_settings);
    setting_unselect_input_random_points = setting_unselect_input_random_points_create(controls_random_points_settings);
    setting_points_selected_random_points = setting_points_selected_random_points_create(controls_random_points_settings);
    const_message_random_points = const_message_random_points_create(controls_random_points_settings);
    slider_scale_region = slider_scale_region_create(controls_random_points_settings);
    slider_number_points_region = slider_number_points_region_create(controls_random_points_settings);
    const_message_random_points_timeout = const_message_random_points_timeout_create(controls_random_points_settings);
    slider_random_points_timeout = slider_random_points_timeout_create(controls_random_points_settings);
    button_random_points_go = button_random_points_go_create(controls_random_points_settings);
}

//
// Create object `popup_random_points_settings' in the specified instance.
//
Xv_opaque
geolab_popup_random_points_settings_objects::popup_random_points_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, FRAME_CMD,
		XV_KEY_DATA, INSTANCE, this,
		XV_WIDTH, 325,
		XV_HEIGHT, 380,
		XV_LABEL, "GeoLab: Random Points Settings",
		XV_SHOW, FALSE,
		FRAME_SHOW_FOOTER, FALSE,
		FRAME_SHOW_RESIZE_CORNER, FALSE,
		FRAME_CMD_PUSHPIN_IN, TRUE,
		NULL);

    xv_set(xv_get(obj, FRAME_CMD_PANEL), WIN_SHOW, FALSE, NULL);
    return obj;
}

//
// Create object `controls_random_points_settings' in the specified instance.
//
Xv_opaque
geolab_popup_random_points_settings_objects::controls_random_points_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 0,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		WIN_BORDER, FALSE,
		NULL);
    return obj;
}

//
// Create object `setting_all_region' in the specified instance.
//
Xv_opaque
geolab_popup_random_points_settings_objects::setting_all_region_create(Xv_opaque owner) {
    extern void popup_all_region_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 36,
		XV_Y, 16,
		XV_WIDTH, 116,
		XV_HEIGHT, 40,
		PANEL_VALUE_X, 36,
		PANEL_VALUE_Y, 33,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 1,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "All Region",
		PANEL_NOTIFY_PROC, popup_all_region_handler,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes",
			0,
		NULL);
    return obj;
}

//
// Create object `setting_discard_input_random_points' in the specified instance.
//
Xv_opaque
geolab_popup_random_points_settings_objects::setting_discard_input_random_points_create(Xv_opaque owner) {
    extern void popup_points_discard_input_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 180,
		XV_Y, 16,
		XV_WIDTH, 116,
		XV_HEIGHT, 40,
		PANEL_VALUE_X, 180,
		PANEL_VALUE_Y, 33,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Discard Input",
		PANEL_NOTIFY_PROC, popup_points_discard_input_handler,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes",
			0,
		NULL);
    return obj;
}

//
// Create object `setting_unselect_input_random_points' in the specified instance.
//
Xv_opaque
geolab_popup_random_points_settings_objects::setting_unselect_input_random_points_create(Xv_opaque owner) {
    extern void popup_unselect_input_region(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 36,
		XV_Y, 70,
		XV_WIDTH, 116,
		XV_HEIGHT, 40,
		PANEL_VALUE_X, 36,
		PANEL_VALUE_Y, 87,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Unselect Input",
		PANEL_NOTIFY_PROC, popup_unselect_input_region,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes",
			0,
		NULL);
    return obj;
}

//
// Create object `setting_points_selected_random_points' in the specified instance.
//
Xv_opaque
geolab_popup_random_points_settings_objects::setting_points_selected_random_points_create(Xv_opaque owner) {
    extern void popup_points_selected_region_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 180,
		XV_Y, 70,
		XV_WIDTH, 116,
		XV_HEIGHT, 40,
		PANEL_VALUE_X, 180,
		PANEL_VALUE_Y, 87,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Points Selected",
		PANEL_NOTIFY_PROC, popup_points_selected_region_handler,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes",
			0,
		NULL);
    return obj;
}

//
// Create object `const_message_random_points' in the specified instance.
//
Xv_opaque
geolab_popup_random_points_settings_objects::const_message_random_points_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 80,
		XV_Y, 132,
		XV_WIDTH, 161,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Number of Random Points",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `slider_scale_region' in the specified instance.
//
Xv_opaque
geolab_popup_random_points_settings_objects::slider_scale_region_create(Xv_opaque owner) {
    extern void popup_scale_region_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_SLIDER,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 81,
		XV_Y, 172,
		XV_WIDTH, 161,
		XV_HEIGHT, 20,
		PANEL_VALUE_X, 126,
		PANEL_VALUE_Y, 172,
		PANEL_SLIDER_WIDTH, 100,
		PANEL_LABEL_STRING, "Scale:",
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_DIRECTION, PANEL_HORIZONTAL,
		PANEL_SLIDER_END_BOXES, FALSE,
		PANEL_SHOW_RANGE, FALSE,
		PANEL_SHOW_VALUE, FALSE,
		PANEL_MIN_VALUE, 1,
		PANEL_MAX_VALUE, 100,
		PANEL_TICKS, 10,
		PANEL_NOTIFY_PROC, popup_scale_region_handler,
		NULL);
    return obj;
}

//
// Create object `slider_number_points_region' in the specified instance.
//
Xv_opaque
geolab_popup_random_points_settings_objects::slider_number_points_region_create(Xv_opaque owner) {
    extern void popup_number_points_region_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_SLIDER,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 16,
		XV_Y, 216,
		XV_WIDTH, 281,
		XV_HEIGHT, 20,
		PANEL_VALUE_X, 83,
		PANEL_VALUE_Y, 216,
		PANEL_SLIDER_WIDTH, 100,
		PANEL_LABEL_STRING, "# Points:",
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_DIRECTION, PANEL_HORIZONTAL,
		PANEL_SLIDER_END_BOXES, FALSE,
		PANEL_SHOW_RANGE, TRUE,
		PANEL_SHOW_VALUE, TRUE,
		PANEL_MIN_VALUE, 1,
		PANEL_MAX_VALUE, 100,
		PANEL_TICKS, 10,
		PANEL_NOTIFY_PROC, popup_number_points_region_handler,
		PANEL_VALUE, 50, 
		NULL);
    return obj;
}

//
// Create object `const_message_random_points_timeout' in the specified instance.
//
Xv_opaque
geolab_popup_random_points_settings_objects::const_message_random_points_timeout_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 60,
		XV_Y, 264,
		XV_WIDTH, 214,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Warning: Timeout Value   (minutes)",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `slider_random_points_timeout' in the specified instance.
//
Xv_opaque
geolab_popup_random_points_settings_objects::slider_random_points_timeout_create(Xv_opaque owner) {
    extern void popup_timeout_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_SLIDER,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 56,
		XV_Y, 300,
		XV_WIDTH, 223,
		XV_HEIGHT, 20,
		PANEL_VALUE_X, 65,
		PANEL_VALUE_Y, 300,
		PANEL_SLIDER_WIDTH, 100,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_DIRECTION, PANEL_HORIZONTAL,
		PANEL_SLIDER_END_BOXES, FALSE,
		PANEL_SHOW_RANGE, TRUE,
		PANEL_SHOW_VALUE, TRUE,
		PANEL_MIN_VALUE, 0,
		PANEL_MAX_VALUE, 100,
		PANEL_TICKS, 10,
		PANEL_NOTIFY_PROC, popup_timeout_handler,
		PANEL_VALUE, 1,
		NULL);
    return obj;
}

//
// Create object `button_random_points_go' in the specified instance.
//
Xv_opaque
geolab_popup_random_points_settings_objects::button_random_points_go_create(Xv_opaque owner) {
    extern void popup_random_points_go(Panel_item, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 136,
		XV_Y, 348,
		XV_WIDTH, 50,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "  Go  ",
		PANEL_NOTIFY_PROC, popup_random_points_go,
		NULL);
    return obj;
}

//
// Initialize an instance of object `popup_random_cover_settings'.
//
void
geolab_popup_random_cover_settings_objects::objects_initialize(Xv_opaque owner) {
    popup_random_cover_settings = popup_random_cover_settings_create(owner);
    controls_random_cover_settings = controls_random_cover_settings_create(popup_random_cover_settings);
    setting_discard_input_cover = setting_discard_input_cover_create(controls_random_cover_settings);
    setting_unselect_input_cover = setting_unselect_input_cover_create(controls_random_cover_settings);
    setting_points_selected_cover = setting_points_selected_cover_create(controls_random_cover_settings);
    const_message_random_cover = const_message_random_cover_create(controls_random_cover_settings);
    slider_scale_cover = slider_scale_cover_create(controls_random_cover_settings);
    slider_number_points_cover = slider_number_points_cover_create(controls_random_cover_settings);
    const_message_random_cover_timeout = const_message_random_cover_timeout_create(controls_random_cover_settings);
    slider_random_cover_timeout = slider_random_cover_timeout_create(controls_random_cover_settings);
    button_random_cover_go = button_random_cover_go_create(controls_random_cover_settings);
}

//
// Create object `popup_random_cover_settings' in the specified instance.
//
Xv_opaque
geolab_popup_random_cover_settings_objects::popup_random_cover_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, FRAME_CMD,
		XV_KEY_DATA, INSTANCE, this,
		XV_WIDTH, 316,
		XV_HEIGHT, 354,
		XV_LABEL, "GeoLab: Random Cover Settings",
		XV_SHOW, FALSE,
		FRAME_SHOW_FOOTER, FALSE,
		FRAME_SHOW_RESIZE_CORNER, FALSE,
		FRAME_CMD_PUSHPIN_IN, TRUE,
		NULL);

    xv_set(xv_get(obj, FRAME_CMD_PANEL), WIN_SHOW, FALSE, NULL);
    return obj;
}

//
// Create object `controls_random_cover_settings' in the specified instance.
//
Xv_opaque
geolab_popup_random_cover_settings_objects::controls_random_cover_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 0,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		WIN_BORDER, FALSE,
		NULL);
    return obj;
}

//
// Create object `setting_discard_input_cover' in the specified instance.
//
Xv_opaque
geolab_popup_random_cover_settings_objects::setting_discard_input_cover_create(Xv_opaque owner) {
    extern void popup_discard_input_cover_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 56,
		XV_Y, 8,
		XV_WIDTH, 221,
		XV_HEIGHT, 23,
		PANEL_VALUE_X, 161,
		PANEL_VALUE_Y, 8,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Discard Input:",
		PANEL_NOTIFY_PROC, popup_discard_input_cover_handler,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes",
			0,
		NULL);
    return obj;
}

//
// Create object `setting_unselect_input_cover' in the specified instance.
//
Xv_opaque
geolab_popup_random_cover_settings_objects::setting_unselect_input_cover_create(Xv_opaque owner) {
    extern void popup_unselect_input_cover_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 48,
		XV_Y, 43,
		XV_WIDTH, 229,
		XV_HEIGHT, 23,
		PANEL_VALUE_X, 161,
		PANEL_VALUE_Y, 43,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Unselect Input:",
		PANEL_NOTIFY_PROC, popup_unselect_input_cover_handler,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes",
			0,
		NULL);
    return obj;
}

//
// Create object `setting_points_selected_cover' in the specified instance.
//
Xv_opaque
geolab_popup_random_cover_settings_objects::setting_points_selected_cover_create(Xv_opaque owner) {
    extern void popup_points_selected_cover_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 45,
		XV_Y, 78,
		XV_WIDTH, 232,
		XV_HEIGHT, 23,
		PANEL_VALUE_X, 161,
		PANEL_VALUE_Y, 78,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Points Selected:",
		PANEL_NOTIFY_PROC, popup_points_selected_cover_handler,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes",
			0,
		NULL);
    return obj;
}

//
// Create object `const_message_random_cover' in the specified instance.
//
Xv_opaque
geolab_popup_random_cover_settings_objects::const_message_random_cover_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 76,
		XV_Y, 120,
		XV_WIDTH, 161,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Number of Random Points",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `slider_scale_cover' in the specified instance.

//
Xv_opaque
geolab_popup_random_cover_settings_objects::slider_scale_cover_create(Xv_opaque owner) {
    extern void popup_scale_cover_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_SLIDER,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 76,
		XV_Y, 152,
		XV_WIDTH, 161,
		XV_HEIGHT, 20,
		PANEL_VALUE_X, 121,
		PANEL_VALUE_Y, 152,
		PANEL_SLIDER_WIDTH, 100,
		PANEL_LABEL_STRING, "Scale:",
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_DIRECTION, PANEL_HORIZONTAL,
		PANEL_SLIDER_END_BOXES, FALSE,
		PANEL_SHOW_RANGE, FALSE,
		PANEL_SHOW_VALUE, FALSE,
		PANEL_MIN_VALUE, 1,
		PANEL_MAX_VALUE, 100,
		PANEL_TICKS, 10,
		PANEL_NOTIFY_PROC, popup_scale_cover_handler,
		NULL);
    return obj;
}

//
// Create object `slider_number_points_cover' in the specified instance.
//
Xv_opaque
geolab_popup_random_cover_settings_objects::slider_number_points_cover_create(Xv_opaque owner) {
    extern void popup_number_points_cover_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_SLIDER,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 16,
		XV_Y, 196,
		XV_WIDTH, 281,
		XV_HEIGHT, 20,
		PANEL_VALUE_X, 83,
		PANEL_VALUE_Y, 196,
		PANEL_SLIDER_WIDTH, 100,
		PANEL_LABEL_STRING, "# Points:",
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_DIRECTION, PANEL_HORIZONTAL,
		PANEL_SLIDER_END_BOXES, FALSE,
		PANEL_SHOW_RANGE, TRUE,
		PANEL_SHOW_VALUE, TRUE,
		PANEL_MIN_VALUE, 1,
		PANEL_MAX_VALUE, 100,
		PANEL_TICKS, 10,
		PANEL_NOTIFY_PROC, popup_number_points_cover_handler,
		PANEL_VALUE, 50,
		NULL);
    return obj;
}

//
// Create object `const_message_random_cover_timeout' in the specified instance.
//
Xv_opaque
geolab_popup_random_cover_settings_objects::const_message_random_cover_timeout_create(Xv_opaque owner) {
    Xv_opaque  obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 56,
		XV_Y, 244,
		XV_WIDTH, 212,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Warning: Timeout Value  (minutes)",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `slider_random_cover_timeout' in the specified instance.
//
Xv_opaque
geolab_popup_random_cover_settings_objects::slider_random_cover_timeout_create(Xv_opaque owner) {
    extern void popup_random_cover_timeout_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_SLIDER,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 48,
		XV_Y, 280,
		XV_WIDTH, 223,
		XV_HEIGHT, 20,
		PANEL_VALUE_X, 57,
		PANEL_VALUE_Y, 280,
		PANEL_SLIDER_WIDTH, 100,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_DIRECTION, PANEL_HORIZONTAL,
		PANEL_SLIDER_END_BOXES, FALSE,
		PANEL_SHOW_RANGE, TRUE,
		PANEL_SHOW_VALUE, TRUE,
		PANEL_MIN_VALUE, 0,
		PANEL_MAX_VALUE, 100,
		PANEL_TICKS, 10,
		PANEL_NOTIFY_PROC, popup_random_cover_timeout_handler,
		NULL);
    return obj;
}

//
// Create object `button_random_cover_go' in the specified instance.
//
Xv_opaque
geolab_popup_random_cover_settings_objects::button_random_cover_go_create(Xv_opaque owner) {
    extern void popup_random_cover_go(Panel_item, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 136,
		XV_Y, 320,
		XV_WIDTH, 50,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "  Go  ",
		PANEL_NOTIFY_PROC, popup_random_cover_go,
		NULL);
    return obj;
}

//
// Initialize an instance of object `popup_animation_settings'.
//
void
geolab_popup_animation_settings_objects::objects_initialize(Xv_opaque owner) {
    popup_animation_settings = popup_animation_settings_create(owner);
    controls_animation_settings = controls_animation_settings_create(popup_animation_settings);
    setting_animation_on_off = setting_animation_on_off_create(controls_animation_settings);
    const_message_depth = const_message_depth_create(controls_animation_settings);
    slider_animation_depth = slider_animation_depth_create(controls_animation_settings);
    textfield_animation_settings = textfield_animation_settings_create(controls_animation_settings);
    const_message_animation_frame_control = const_message_animation_frame_control_create(controls_animation_settings);
    button_animation_settings_pause_continue = button_animation_settings_pause_continue_create(controls_animation_settings);
    button_animation_settings_step = button_animation_settings_step_create(controls_animation_settings);
    button_animation_settings_fast_forward = button_animation_settings_fast_forward_create(controls_animation_settings);
    const_message_animation_frame_delay = const_message_animation_frame_delay_create(controls_animation_settings);
    slider_animation_frame_delay = slider_animation_frame_delay_create(controls_animation_settings);
}

//
// Create object `popup_animation_settings' in the specified instance.
//
Xv_opaque
geolab_popup_animation_settings_objects::popup_animation_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, FRAME_CMD,
		XV_KEY_DATA, INSTANCE, this,
		XV_WIDTH, 284,
		XV_HEIGHT, 294,
		XV_LABEL, "GeoLab: Animation Settings",
		XV_SHOW, FALSE,
		FRAME_SHOW_FOOTER, FALSE,
		FRAME_SHOW_RESIZE_CORNER, FALSE,
		FRAME_CMD_PUSHPIN_IN, TRUE,
		NULL);

    xv_set(xv_get(obj, FRAME_CMD_PANEL), WIN_SHOW, FALSE, NULL);
    return obj;
}

//
// Create object `controls_animation_settings' in the specified instance.
//
Xv_opaque
geolab_popup_animation_settings_objects::controls_animation_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 0,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		WIN_BORDER, FALSE,
		NULL);
    return obj;
}

//
// Create object `setting_animation_on_off' in the specified instance.
//
Xv_opaque
geolab_popup_animation_settings_objects::setting_animation_on_off_create(Xv_opaque owner) {
    extern void popup_animation_on_off_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 40,
		XV_Y, 24,
		XV_WIDTH, 200,
		XV_HEIGHT, 23,
		PANEL_VALUE_X, 123,
		PANEL_VALUE_Y, 24,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Animation:",
		PANEL_NOTIFY_PROC, popup_animation_on_off_handler,
		PANEL_CHOICE_STRINGS,
			"Off",
			"On",
			0,
		NULL);
    return obj;
}

//      
// Create object `const_message_depth' in the specified instance.
//
Xv_opaque
geolab_popup_animation_settings_objects::const_message_depth_create(Xv_opaque owner) {
    Xv_opaque obj;
    
    obj = xv_create(owner, PANEL_MESSAGE,  
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 40,
		XV_Y, 64,
		XV_WIDTH, 45,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Depth:",
		PANEL_LABEL_BOLD, TRUE,
		NULL);
    return obj;
}
		
//
// Create object `slider_animation_depth' in the specified instance.
//
Xv_opaque
geolab_popup_animation_settings_objects::slider_animation_depth_create(Xv_opaque owner) {
    extern void popup_animation_depth_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_SLIDER,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 36,
		XV_Y, 86,
		XV_WIDTH, 215,
		XV_HEIGHT, 20,
		PANEL_VALUE_X, 36,
		PANEL_VALUE_Y, 86,
		PANEL_SLIDER_WIDTH, 100,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_DIRECTION, PANEL_HORIZONTAL,
		PANEL_SLIDER_END_BOXES, FALSE,
		PANEL_SHOW_RANGE, TRUE,
		PANEL_SHOW_VALUE, TRUE,
		PANEL_MIN_VALUE, 1,
		PANEL_MAX_VALUE, 50,
		PANEL_TICKS, 50,
		PANEL_NOTIFY_PROC, popup_animation_depth_handler,
		NULL);
    return obj;
}

//
// Create object `textfield_animation_settings' in the specified instance.
//
Xv_opaque
geolab_popup_animation_settings_objects::textfield_animation_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_TEXT,
	 	XV_KEY_DATA, INSTANCE, this,
		XV_X, 16,
		XV_Y, 120,
		XV_WIDTH, 329,
		XV_HEIGHT, 15,
		PANEL_VALUE_X, 25,
		PANEL_VALUE_Y, 120,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_VALUE_DISPLAY_LENGTH, 30,
		PANEL_READ_ONLY, TRUE,
		NULL);
    return obj;
}

//
// Create object `const_message_animation_frame_control' in the specified instance.
//
Xv_opaque
geolab_popup_animation_settings_objects::const_message_animation_frame_control_create(Xv_opaque owner) {
     Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 100,
		XV_Y, 156,
		XV_WIDTH, 97,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Frame Control",
		PANEL_LABEL_BOLD, TRUE,
		NULL);
    return obj;
}

//
// Create object `button_animation_settings_pause_continue' in the specified instance.
//
Xv_opaque
geolab_popup_animation_settings_objects::button_animation_settings_pause_continue_create(Xv_opaque owner) {
    extern void button_animation_settings_pause_continue_handler(Panel_item, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 20,
		XV_Y, 192,
		XV_WIDTH, 74,
		XV_HEIGHT, 19, 		PANEL_LABEL_STRING, "  Pause   ",
    		PANEL_NOTIFY_PROC, button_animation_settings_pause_continue_handler,
		NULL);
    return obj;
}

//
// Create object `button_animation_settings_step' in the specified instance.
//
Xv_opaque
geolab_popup_animation_settings_objects::button_animation_settings_step_create(Xv_opaque owner) {
    extern void button_animation_settings_step_handler(Panel_item, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 108,
		XV_Y, 192,
		XV_WIDTH, 45,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "Step",
		PANEL_NOTIFY_PROC, button_animation_settings_step_handler,
		NULL);
    return obj;
}

//
// Create object `button_animation_settings_fast_forward' in the specified instance.
//
Xv_opaque
geolab_popup_animation_settings_objects::button_animation_settings_fast_forward_create(Xv_opaque owner) {
    extern void button_animation_settings_fast_forward(Panel_item, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 168,
		XV_Y, 192,
		XV_WIDTH, 97,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "Fast Forward",
		PANEL_NOTIFY_PROC, button_animation_settings_fast_forward,
		NULL);
    return obj;
}

//
// Create object `const_message_animation_frame_delay' in the specified instance.
//
Xv_opaque
geolab_popup_animation_settings_objects::const_message_animation_frame_delay_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 24,
		XV_Y, 228,
		XV_WIDTH, 79,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Frame Delay:",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `slider_animation_frame_delay' in the specified instance.
//
Xv_opaque
geolab_popup_animation_settings_objects::slider_animation_frame_delay_create(Xv_opaque owner) {
    extern void popup_frame_delay_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_SLIDER,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 24,
		XV_Y, 256,
		XV_WIDTH, 239,
		XV_HEIGHT, 15,
		PANEL_VALUE_X, 33,
		PANEL_VALUE_Y, 256,
		PANEL_SLIDER_WIDTH, 100,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_DIRECTION, PANEL_HORIZONTAL,
		PANEL_SLIDER_END_BOXES, TRUE,
		PANEL_SHOW_RANGE, TRUE,
		PANEL_SHOW_VALUE, TRUE,
		PANEL_MIN_VALUE, 0,
		PANEL_MAX_VALUE, 100,
		PANEL_TICKS, 0,
		PANEL_NOTIFY_PROC, popup_frame_delay_handler,
		NULL);
    return obj;
}

//
// Initialize an instance of object `popup_random_polygon_settings'.
//
void
geolab_popup_random_polygon_settings_objects::objects_initialize(Xv_opaque owner) {
    popup_random_polygon_settings = popup_random_polygon_settings_create(owner);
    controls_random_polygon_settings = controls_random_polygon_settings_create(popup_random_polygon_settings);
    setting_random_polygon_discard_input = setting_random_polygon_discard_input_create(controls_random_polygon_settings);
    setting_polygon_type = setting_polygon_type_create(controls_random_polygon_settings);
    setting_random_polygon_unselect_input = setting_random_polygon_unselect_input_create(controls_random_polygon_settings);
    setting_random_polygon_selected = setting_random_polygon_selected_create(controls_random_polygon_settings);
    const_message_random_polygon_timeout = const_message_random_polygon_timeout_create(controls_random_polygon_settings);
    slider_random_polygon_timeout = slider_random_polygon_timeout_create(controls_random_polygon_settings);
    button_random_polygon_go = button_random_polygon_go_create(controls_random_polygon_settings);
}

//
// Create object `popup_random_polygon_settings' in the specified instance.
//
Xv_opaque
geolab_popup_random_polygon_settings_objects::popup_random_polygon_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, FRAME_CMD,
		XV_KEY_DATA, INSTANCE, this,
		XV_WIDTH, 326,
		XV_HEIGHT, 294,
		XV_LABEL, "GeoLab: Random Polygon Settings",
		XV_SHOW, FALSE,
		FRAME_SHOW_FOOTER, FALSE,
		FRAME_SHOW_RESIZE_CORNER, FALSE,
		FRAME_CMD_PUSHPIN_IN, TRUE,
		NULL);

    xv_set(xv_get(obj, FRAME_CMD_PANEL), WIN_SHOW, FALSE, NULL);
    return obj;
}

//
// Create object `controls_random_polygon_settings' in the specified instance.
//
Xv_opaque
geolab_popup_random_polygon_settings_objects::controls_random_polygon_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 0,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		WIN_BORDER, FALSE,
		NULL);
    return obj;
}

//
// Create object `setting_random_polygon_discard_input' in the specified instance.
//
Xv_opaque
geolab_popup_random_polygon_settings_objects::setting_random_polygon_discard_input_create(Xv_opaque owner) {
    extern void popup_polygon_discard_input(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 164,
		XV_Y, 20,
		XV_WIDTH, 116,
		XV_HEIGHT, 40,
		PANEL_VALUE_X, 164,
		PANEL_VALUE_Y, 37,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Discard Input",
		PANEL_NOTIFY_PROC, popup_polygon_discard_input,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes",
			0,
		NULL);
    return obj;
}

//
// Create object `setting_polygon_type' in the specified instance.
//
Xv_opaque
geolab_popup_random_polygon_settings_objects::setting_polygon_type_create(Xv_opaque owner) {
    extern void popup_polygon_type_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 28,
		XV_Y, 36,
		XV_WIDTH, 107,
		XV_HEIGHT, 127,
		PANEL_VALUE_X, 28,
		PANEL_VALUE_Y, 53,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 1,
		PANEL_CHOICE_NCOLS, 1,
		PANEL_LABEL_STRING, "Polygon Type",
		PANEL_NOTIFY_PROC, popup_polygon_type_handler,
		PANEL_CHOICE_STRINGS,
			"Generic",
			"Simple",
			"Star Shaped",
			"Convex",
			0,
		NULL);
    return obj;
}

//
// Create object `setting_random_polygon_unselect_input' in the specified instance.
//
Xv_opaque
geolab_popup_random_polygon_settings_objects::setting_random_polygon_unselect_input_create(Xv_opaque owner) {
    extern void popup_polygon_unselect_input(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 164,
		XV_Y, 72,
		XV_WIDTH, 116,
		XV_HEIGHT, 40,
		PANEL_VALUE_X, 164,
		PANEL_VALUE_Y, 89,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Unselect Input",
		PANEL_NOTIFY_PROC, popup_polygon_unselect_input,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes",
			0,
		NULL);
    return obj;
}

//
// Create object `setting_random_polygon_selected' in the specified instance.
//
Xv_opaque
geolab_popup_random_polygon_settings_objects::setting_random_polygon_selected_create(Xv_opaque owner) {
    extern void popup_polygon_selected_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 164,
		XV_Y, 126,
		XV_WIDTH, 116,
		XV_HEIGHT, 40,
		PANEL_VALUE_X, 164,
		PANEL_VALUE_Y, 143,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Polygon Selected",
		PANEL_NOTIFY_PROC, popup_polygon_selected_handler,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes",
			0,
		NULL);
    return obj;
}

//
// Create object `const_message_random_polygon_timeout' in the specified instance.
//
Xv_opaque
geolab_popup_random_polygon_settings_objects::const_message_random_polygon_timeout_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 60,
		XV_Y, 188,
		XV_WIDTH, 212,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Warning: Timeout Value  (minutes)",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `slider_random_polygon_timeout' in the specified instance.
//
Xv_opaque
geolab_popup_random_polygon_settings_objects::slider_random_polygon_timeout_create(Xv_opaque owner) {
    extern void popup_random_polygon_timeout_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_SLIDER,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 52,
		XV_Y, 220,
		XV_WIDTH, 223,
		XV_HEIGHT, 20,
		PANEL_VALUE_X, 61,
		PANEL_VALUE_Y, 220,
		PANEL_SLIDER_WIDTH, 100,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_DIRECTION, PANEL_HORIZONTAL,
		PANEL_SLIDER_END_BOXES, FALSE,
		PANEL_SHOW_RANGE, TRUE,
		PANEL_SHOW_VALUE, TRUE,
		PANEL_MIN_VALUE, 0,
		PANEL_MAX_VALUE, 100,
		PANEL_TICKS, 10,
		PANEL_NOTIFY_PROC, popup_random_polygon_timeout_handler,
		NULL);
    return obj;
}

// 
// Create object `button_random_polygon_go' in the specified instance.
//
Xv_opaque
geolab_popup_random_polygon_settings_objects::button_random_polygon_go_create(Xv_opaque owner) {
    extern void popup_random_polygon_go(Panel_item, Event *);
    Xv_opaque obj;
    
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 140,
		XV_Y, 260,
		XV_WIDTH, 50,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "  Go  ",
		PANEL_NOTIFY_PROC, popup_random_polygon_go,
		NULL);
    return obj;
}
  
//
// Initialize an instance of object `popup_color_settings_bw'.
//
void
geolab_popup_color_settings_bw_objects::objects_initialize(Xv_opaque owner) {
    popup_color_settings_bw = popup_color_settings_bw_create(owner);
    controls_color_settings_bw = controls_color_settings_bw_create(popup_color_settings_bw);
    const_message_color_settings_bw1 = const_message_color_settings_bw1_create(controls_color_settings_bw);
    const_message_color_settings_bw2 = const_message_color_settings_bw2_create(controls_color_settings_bw);
    const_message_color_settings_bw3 = const_message_color_settings_bw3_create(controls_color_settings_bw);
}

//
// Create object `popup_color_settings_bw' in the specified instance.
//
Xv_opaque
geolab_popup_color_settings_bw_objects::popup_color_settings_bw_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, FRAME_CMD,
		XV_KEY_DATA, INSTANCE, this,
		XV_WIDTH, 280,
		XV_HEIGHT, 149,
		XV_LABEL, "GeoLab: Color Settings",
		XV_SHOW, FALSE,
		FRAME_SHOW_FOOTER, FALSE,
		FRAME_SHOW_RESIZE_CORNER, FALSE,
		FRAME_CMD_PUSHPIN_IN, TRUE,
		NULL);
		
    xv_set(xv_get(obj, FRAME_CMD_PANEL), WIN_SHOW, FALSE, NULL);
    return obj;
}

//
// Create object `controls_color_settings_bw' in the specified instance.
//
Xv_opaque
geolab_popup_color_settings_bw_objects::controls_color_settings_bw_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 0,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		WIN_BORDER, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_color_settings_bw1' in the specified instance.
//
Xv_opaque
geolab_popup_color_settings_bw_objects::const_message_color_settings_bw1_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 32,
		XV_Y, 24,
		XV_WIDTH, 40,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Sorry:",
		PANEL_LABEL_BOLD, TRUE,
		NULL);
    return obj;
}

//
// Create object `const_message_color_settings_bw2' in the specified instance.
//
Xv_opaque
geolab_popup_color_settings_bw_objects::const_message_color_settings_bw2_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 32,
		XV_Y, 56,
		XV_WIDTH, 217,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "This is a Black & White display.",
		PANEL_LABEL_BOLD, TRUE,
		NULL);
    return obj;
}

//
// Create object `const_message_color_settings_bw3' in the specified instance.
//
Xv_opaque
geolab_popup_color_settings_bw_objects::const_message_color_settings_bw3_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 48,
		XV_Y, 88,
		XV_WIDTH, 182,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Color option not available.",
		PANEL_LABEL_BOLD, TRUE,
		NULL);
    return obj;
}

//
// Initialize an instance of object `popup_algorithms_settings'.
//
void
geolab_popup_algorithms_settings_objects::objects_initialize(Xv_opaque owner) {
    popup_algorithms_settings = popup_algorithms_settings_create(owner);
    controls_algorithms_settings = controls_algorithms_settings_create(popup_algorithms_settings);
    const_message_algorithms_settings_generic_calls = const_message_algorithms_settings_generic_calls_create(controls_algorithms_settings);
    setting_algorithms_discard_input = setting_algorithms_discard_input_create(controls_algorithms_settings);
    setting_algorithms_unselect_input = setting_algorithms_unselect_input_create(controls_algorithms_settings);
    setting_algorithms_output_selected = setting_algorithms_output_selected_create(controls_algorithms_settings);
    const_message_algorithms_settings_dynamic_calls = const_message_algorithms_settings_dynamic_calls_create(controls_algorithms_settings);
    setting_algorithms_discard_output = setting_algorithms_discard_output_create(controls_algorithms_settings);
}

//
// Create object `popup_algorithms_settings' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_settings_objects::popup_algorithms_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, FRAME_CMD,
		XV_KEY_DATA, INSTANCE, this,
		XV_WIDTH, 317,
		XV_HEIGHT, 338,
		XV_LABEL, "GeoLab: Algorithms Settings",
		XV_SHOW, FALSE,
		FRAME_SHOW_FOOTER, FALSE,
		FRAME_SHOW_RESIZE_CORNER, FALSE,
		FRAME_CMD_PUSHPIN_IN, TRUE,
		NULL);
    xv_set(xv_get(obj, FRAME_CMD_PANEL), WIN_SHOW, FALSE, NULL);
    return obj;
}

//
// Create object `controls_algorithms_settings' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_settings_objects::controls_algorithms_settings_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 0,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		WIN_BORDER, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_algorithms_settings_generic_calls' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_settings_objects::const_message_algorithms_settings_generic_calls_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 112,
		XV_Y, 28,
		XV_WIDTH, 92,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Generic Calls",
		PANEL_LABEL_BOLD, TRUE,
		NULL);
    return obj;
}

//
// Create object `setting_algorithms_discard_input' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_settings_objects::setting_algorithms_discard_input_create(Xv_opaque owner) {
    extern void	popup_algorithms_discard_input_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 52,
		XV_Y, 72,
		XV_WIDTH, 221,
		XV_HEIGHT, 23,
		PANEL_VALUE_X, 157,
		PANEL_VALUE_Y, 72,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Discard Input:",
		PANEL_NOTIFY_PROC, popup_algorithms_discard_input_handler,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes",
			0,
		NULL);
    return obj;
}

//
// Create object `setting_algorithms_unselect_input' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_settings_objects::setting_algorithms_unselect_input_create(Xv_opaque owner) {
    extern void popup_algorithms_unselect_input_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 44,
		XV_Y, 116,
		XV_WIDTH, 229,
		XV_HEIGHT, 23,
		PANEL_VALUE_X, 157,
		PANEL_VALUE_Y, 116,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Unselect Input:",
		PANEL_NOTIFY_PROC, popup_algorithms_unselect_input_handler,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes",
			0,
		NULL);
    return obj;
}

//
// Create object `setting_algorithms_output_selected' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_settings_objects::setting_algorithms_output_selected_create(Xv_opaque owner) {
    extern void	popup_algorithms_output_selected_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 36,
		XV_Y, 160,
		XV_WIDTH, 237,
		XV_HEIGHT, 23,
		PANEL_VALUE_X, 157,
		PANEL_VALUE_Y, 160,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Output Selected:",
		PANEL_NOTIFY_PROC, popup_algorithms_output_selected_handler,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes",
			0,
		NULL);
    return obj;
}

//
// Create object `const_message_algorithms_settings_dynamic_calls' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_settings_objects::const_message_algorithms_settings_dynamic_calls_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 111,
		XV_Y, 232,
		XV_WIDTH, 100,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Dynamic Calls",
		PANEL_LABEL_BOLD, TRUE,
		NULL);
    return obj;
}

//
// Create object `setting_algorithms_discard_output' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_settings_objects::setting_algorithms_discard_output_create(Xv_opaque owner) {
    extern void popup_algorithms_discard_output_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 40,
		XV_Y, 272,
		XV_WIDTH, 232,
		XV_HEIGHT, 23,
		PANEL_VALUE_X, 156,
		PANEL_VALUE_Y, 272,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 1,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Discard Output:",
		PANEL_NOTIFY_PROC, popup_algorithms_discard_output_handler,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes",
			0,
		NULL);
    return obj;
}

//
// Initialize an instance of object `popup_algorithms_statistics'.
//
void
geolab_popup_algorithms_statistics_objects::objects_initialize(Xv_opaque owner) {
    popup_algorithms_statistics = popup_algorithms_statistics_create(owner);
    controls_algorithms_statistics = controls_algorithms_statistics_create(popup_algorithms_statistics);
    textfield_algorithms_statistics_current = textfield_algorithms_statistics_current_create(controls_algorithms_statistics);
    const_message_algorithms_statistics_real = const_message_algorithms_statistics_real_create(controls_algorithms_statistics);
    const_message_algorithms_statistics_virtual = const_message_algorithms_statistics_virtual_create(controls_algorithms_statistics);
    const_message_algorithms_statistics_time1 = const_message_algorithms_statistics_time1_create(controls_algorithms_statistics);
    const_message_algorithms_statistics_real_place_holder = const_message_algorithms_statistics_real_place_holder_create(controls_algorithms_statistics);
    const_message_algorithms_statistics_virtual_place_holder = const_message_algorithms_statistics_virtual_place_holder_create(controls_algorithms_statistics);
    const_message_algorithms_statistics_time2 = const_message_algorithms_statistics_time2_create(controls_algorithms_statistics);
}

//
// Create object `popup_algorithms_statistics' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_statistics_objects::popup_algorithms_statistics_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, FRAME_CMD,
		XV_KEY_DATA, INSTANCE, this,
		XV_WIDTH, 400,
		XV_HEIGHT, 150,
		XV_LABEL, "GeoLab: Algorithms Statistics",
		XV_SHOW, FALSE,
		FRAME_SHOW_FOOTER, FALSE,
		FRAME_SHOW_RESIZE_CORNER, FALSE,
		FRAME_CMD_PUSHPIN_IN, TRUE,
		NULL);
    xv_set(xv_get(obj, FRAME_CMD_PANEL), WIN_SHOW, FALSE, NULL);
    return obj;
}    

//
// Create object `controls_algorithms_statistics' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_statistics_objects::controls_algorithms_statistics_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 0,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		WIN_BORDER, FALSE,
		NULL);
    return obj;
}

//
// Create object `textfield_algorithms_statistics_current' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_statistics_objects::textfield_algorithms_statistics_current_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_TEXT,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 20,
		XV_Y, 20,
		XV_WIDTH, 360,
		XV_HEIGHT, 32,
		PANEL_LABEL_STRING, "Current Algorithm:",
		PANEL_VALUE_X, 20,
		PANEL_VALUE_Y, 37,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_VALUE_DISPLAY_LENGTH, 45,
		PANEL_VALUE_STORED_LENGTH, 200,
		PANEL_READ_ONLY, TRUE,
		NULL);
    return obj;
}

//
// Create object `const_message_algorithms_statistics_real' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_statistics_objects::const_message_algorithms_statistics_real_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 172,
		XV_Y, 76,
		XV_WIDTH, 26,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Real",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_algorithms_statistics_virtual' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_statistics_objects::const_message_algorithms_statistics_virtual_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 264,
		XV_Y, 76,
		XV_WIDTH, 79,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Virtual (CPU)",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_algorithms_statistics_time1' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_statistics_objects::const_message_algorithms_statistics_time1_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 16,
		XV_Y, 92,
		XV_WIDTH, 97,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Execution Time:",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_algorithms_statistics_real_place_holder' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_statistics_objects::const_message_algorithms_statistics_real_place_holder_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 172,
		XV_Y, 104,
		XV_WIDTH, 0,
		XV_HEIGHT, 0,
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_algorithms_statistics_virtual_place_holder' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_statistics_objects::const_message_algorithms_statistics_virtual_place_holder_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 264,
		XV_Y, 104,
		XV_WIDTH, 0,
		XV_HEIGHT, 0,
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_algorithms_statistics_time2' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_statistics_objects::const_message_algorithms_statistics_time2_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 28,
		XV_Y, 112,
		XV_WIDTH, 72,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "(in seconds)",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Initialize an instance of object `popup_algorithms_hackers_dream'.
//
void
geolab_popup_algorithms_hackers_dream_objects::objects_initialize(Xv_opaque owner) {
    popup_algorithms_hackers_dream = popup_algorithms_hackers_dream_create(owner);
    controls_algorithms_hackers_dream = controls_algorithms_hackers_dream_create(popup_algorithms_hackers_dream);
    setting_algorithms_dump_heap = setting_algorithms_dump_heap_create(controls_algorithms_hackers_dream);
    textfield_algorihtms_hackers_dream = textfield_algorihtms_hackers_dream_create(controls_algorithms_hackers_dream);
    const_message_algorithms_hackers_dream_memory = const_message_algorithms_hackers_dream_memory_create(controls_algorithms_hackers_dream);
    const_message_algorithms_hackers_dream_total = const_message_algorithms_hackers_dream_total_create(controls_algorithms_hackers_dream);
    const_message_algorithms_hackers_dream_total_place_holder = const_message_algorithms_hackers_dream_total_place_holder_create(controls_algorithms_hackers_dream);
    const_message_algorithms_hackers_dream_allocated = const_message_algorithms_hackers_dream_allocated_create(controls_algorithms_hackers_dream);
    const_message_algorithms_hackers_dream_allocated_place_holder = const_message_algorithms_hackers_dream_allocated_place_holder_create(controls_algorithms_hackers_dream);
    const_message_algorithms_hackers_dream_free = const_message_algorithms_hackers_dream_free_create(controls_algorithms_hackers_dream);
    const_message_algorithms_hackers_dream_free_place_holder = const_message_algorithms_hackers_dream_free_place_holder_create(controls_algorithms_hackers_dream);
}

//
// Create object `popup_algorithms_hackers_dream' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_hackers_dream_objects::popup_algorithms_hackers_dream_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, FRAME_CMD,
		XV_KEY_DATA, INSTANCE, this,
		XV_WIDTH, 367,
		XV_HEIGHT, 228,
		XV_LABEL, "GeoLab: Hacker's Dream",
		XV_SHOW, FALSE,
		FRAME_SHOW_FOOTER, FALSE,
		FRAME_SHOW_RESIZE_CORNER, FALSE,
		FRAME_CMD_PUSHPIN_IN, TRUE,
		NULL);
    xv_set(xv_get(obj, FRAME_CMD_PANEL), WIN_SHOW, FALSE, NULL);
    return obj;
}

//
// Create object `controls_algorithms_hackers_dream' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_hackers_dream_objects::controls_algorithms_hackers_dream_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 0,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		WIN_BORDER, FALSE,
		NULL);
    return obj;
}

//
// Create object `setting_algorithms_dump_heap' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_hackers_dream_objects::setting_algorithms_dump_heap_create(Xv_opaque owner) {
    extern void popup_algorithms_dump_heap_handler(Panel_item, int, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_CHECK_BOX, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 28,
		XV_Y, 28,
		XV_WIDTH, 315,
		XV_HEIGHT, 40,
		PANEL_VALUE_X, 28,
		PANEL_VALUE_Y, 45,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 0,
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Heap Info:",
		PANEL_NOTIFY_PROC, popup_algorithms_dump_heap_handler,
		PANEL_CHOICE_STRINGS,
			"No",
			"Yes, dump it to the standard output",
			0,
		NULL);
    return obj;
}

//
// Create object `textfield_algorihtms_hackers_dream' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_hackers_dream_objects::textfield_algorihtms_hackers_dream_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_TEXT,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 12,
		XV_Y, 72,
		XV_WIDTH, 329,
		XV_HEIGHT, 15,
		PANEL_VALUE_X, 21,
		PANEL_VALUE_Y, 72,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_VALUE_DISPLAY_LENGTH, 40,
		PANEL_READ_ONLY, TRUE,
		NULL);
    return obj;
}

//
// Create object `const_message_algorithms_hackers_dream_memory' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_hackers_dream_objects::const_message_algorithms_hackers_dream_memory_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 124,
		XV_Y, 116,
		XV_WIDTH, 113,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Memory Allocation",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_algorithms_hackers_dream_total' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_hackers_dream_objects::const_message_algorithms_hackers_dream_total_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 20,
		XV_Y, 144,
		XV_WIDTH, 73,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Total Space:",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_algorithms_hackers_dream_total_place_holder' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_hackers_dream_objects::const_message_algorithms_hackers_dream_total_place_holder_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 100,
		XV_Y, 144,
		XV_WIDTH, 0,
		XV_HEIGHT, 0,
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_algorithms_hackers_dream_allocated' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_hackers_dream_objects::const_message_algorithms_hackers_dream_allocated_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 34,
		XV_Y, 168,
		XV_WIDTH, 59,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Allocated:",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_algorithms_hackers_dream_allocated_place_holder' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_hackers_dream_objects::const_message_algorithms_hackers_dream_allocated_place_holder_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 100,
		XV_Y, 168,
		XV_WIDTH, 0,
		XV_HEIGHT, 0,
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_algorithms_hackers_dream_free' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_hackers_dream_objects::const_message_algorithms_hackers_dream_free_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 64,
		XV_Y, 192,
		XV_WIDTH, 29,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Free:",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_algorithms_hackers_dream_free_place_holder' in the specified instance.
//
Xv_opaque
geolab_popup_algorithms_hackers_dream_objects::const_message_algorithms_hackers_dream_free_place_holder_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 100,
		XV_Y, 192,
		XV_WIDTH, 0,
		XV_HEIGHT, 0,
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Initialize an instance of object `popup_edit_string'.
//
void
geolab_popup_edit_string_objects::objects_initialize(Xv_opaque owner) {
    popup_edit_string = popup_edit_string_create(owner);
    controls_edit_string = controls_edit_string_create(popup_edit_string);
    textfield_edit_string = textfield_edit_string_create(controls_edit_string);
}

//
// Create object `popup_edit_string' in the specified instance.
//
Xv_opaque
geolab_popup_edit_string_objects::popup_edit_string_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, FRAME_CMD,
		XV_KEY_DATA, INSTANCE, this,
		XV_WIDTH, 344,
		XV_HEIGHT, 57,
		XV_LABEL, "GeoLab: Object String",
		XV_SHOW, FALSE,
		FRAME_SHOW_FOOTER, FALSE,
		FRAME_SHOW_RESIZE_CORNER, FALSE,
		FRAME_CMD_PUSHPIN_IN, TRUE,
		NULL);

    xv_set(xv_get(obj, FRAME_CMD_PANEL), WIN_SHOW, FALSE, NULL);
    return obj;
}

//
// Create object `controls_edit_string' in the specified instance.
//
Xv_opaque
geolab_popup_edit_string_objects::controls_edit_string_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 0,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		WIN_BORDER, FALSE,
		NULL);
    return obj;
}

//
// Create object `textfield_edit_string' in the specified instance.
//
Xv_opaque
geolab_popup_edit_string_objects::textfield_edit_string_create(Xv_opaque owner) {
    extern Panel_setting popup_edit_string_handler(Panel_item, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_TEXT,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 24,
		XV_Y, 20,
		XV_WIDTH, 293,
		XV_HEIGHT, 15,
		PANEL_LABEL_STRING, "String:",
		PANEL_VALUE_X, 77,
		PANEL_VALUE_Y, 20,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_VALUE_DISPLAY_LENGTH, 30,
		PANEL_VALUE_STORED_LENGTH, 80,
		PANEL_READ_ONLY, FALSE,
		PANEL_NOTIFY_PROC, popup_edit_string_handler,
		NULL);
    return obj;
}

//
// Initialize an instance of object `popup_object_attributes'.
//
void
geolab_popup_object_attributes_objects::objects_initialize(Xv_opaque owner)
{
	popup_object_attributes = popup_object_attributes_create(owner);
	control_object_attributes = control_object_attributes_create(popup_object_attributes);
	setting_object_attributes_deletable = setting_object_attributes_deletable_create(control_object_attributes);
	setting_object_attributes_line_width = setting_object_attributes_line_width_create(control_object_attributes);
	setting_object_attributes_line_style = setting_object_attributes_line_style_create(control_object_attributes);
	setting_object_attributes_glue = setting_object_attributes_glue_create(control_object_attributes);
	setting_object_attributes_handles = setting_object_attributes_handles_create(control_object_attributes);
	setting_object_attributes_foreground_color = setting_object_attributes_foreground_color_create(control_object_attributes);
	setting_object_attributes_background_color = setting_object_attributes_background_color_create(control_object_attributes);
	message_object_attributes = message_object_attributes_create(control_object_attributes);
	setting_object_attributes_increment = setting_object_attributes_increment_create(control_object_attributes);
	textfield_object_attributes_string = textfield_object_attributes_string_create(control_object_attributes);
	textfield_object_attributes_increment = textfield_object_attributes_increment_create(control_object_attributes);
	button_object_attributes_apply_to_selected = button_object_attributes_apply_to_selected_create(control_object_attributes);
	button_object_attributes_set_default = button_object_attributes_set_default_create(control_object_attributes);
}

//
// Create object `popup_object_attributes' in the specified instance.

//
Xv_opaque
geolab_popup_object_attributes_objects::popup_object_attributes_create(Xv_opaque owner)
{
	Xv_opaque	obj;
	
	obj = xv_create(owner, FRAME_CMD,
		XV_KEY_DATA, INSTANCE, this,
		XV_WIDTH, 373,
		XV_HEIGHT, 300,
		XV_LABEL, "Geolab: Object Attributes",
		XV_SHOW, FALSE,
		FRAME_SHOW_FOOTER, TRUE,
		FRAME_SHOW_RESIZE_CORNER, TRUE,
		FRAME_CMD_PUSHPIN_IN, FALSE,
		NULL);
	xv_set(xv_get(obj, FRAME_CMD_PANEL), WIN_SHOW, FALSE, NULL);
	return obj;
}

//
// Create object `control_object_attributes' in the specified instance.

//
Xv_opaque
geolab_popup_object_attributes_objects::control_object_attributes_create(Xv_opaque owner)
{
	Xv_opaque	obj;
	
	obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 0,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		WIN_BORDER, FALSE,
		NULL);
	gcm_initialize_colors(obj, NULL, NULL);
	return obj;
}

//
// Create object `setting_object_attributes_deletable' in the specified instance.

//
Xv_opaque
geolab_popup_object_attributes_objects::setting_object_attributes_deletable_create(Xv_opaque owner)
{
	extern void		popup_object_attributes_deletable_handler(Panel_item, int, Event *);
	Xv_opaque	obj;
	
	obj = xv_create(owner, PANEL_TOGGLE, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 16,
		XV_Y, 20,
		XV_WIDTH, 116,
		XV_HEIGHT, 40,
		PANEL_VALUE_X, 16,
		PANEL_VALUE_Y, 37,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 1,			
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Deletable:",
		PANEL_NOTIFY_PROC, popup_object_attributes_deletable_handler,
		PANEL_CHOICE_STRINGS,
			"Yes",
			"No",
			0,
		NULL);
	return obj;
}

//
// Create object `setting_object_attributes_line_width' in the specified instance.

//
Xv_opaque
geolab_popup_object_attributes_objects::setting_object_attributes_line_width_create(Xv_opaque owner)
{
	extern void		popup_object_attributes_line_width_handler(Panel_item, int, Event *);
	Xv_opaque	obj;
	
	obj = xv_create(owner, PANEL_CHOICE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 160,
		XV_Y, 20,
		XV_WIDTH, 77,
		XV_HEIGHT, 63,
		PANEL_VALUE_X, 160,
		PANEL_VALUE_Y, 37,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOICE_NCOLS, 1,
		PANEL_LABEL_STRING, "Line Width:",
		PANEL_NOTIFY_PROC, popup_object_attributes_line_width_handler,
		PANEL_CHOICE_STRINGS,
			"Normal",
			"Bold",
			0,
		NULL);
	return obj;
}

//
// Create object `setting_object_attributes_line_style' in the specified instance.

//
Xv_opaque
geolab_popup_object_attributes_objects::setting_object_attributes_line_style_create(Xv_opaque owner)
{
	extern void		popup_object_attributes_line_style_handler(Panel_item, int, Event *);
	Xv_opaque	obj;
	
	obj = xv_create(owner, PANEL_CHOICE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 248,
		XV_Y, 20,
		XV_WIDTH, 108,
		XV_HEIGHT, 86,
		PANEL_VALUE_X, 248,
		PANEL_VALUE_Y, 37,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOICE_NCOLS, 1,
		PANEL_LABEL_STRING, "Line Style:",
		PANEL_NOTIFY_PROC, popup_object_attributes_line_style_handler,
		PANEL_CHOICE_STRINGS,
			"Solid",
			"Dashed",
			"Double Dashed",
			0,
		NULL);
	return obj;
}

//
// Create object `setting_object_attributes_glue' in the specified instance.

//
Xv_opaque
geolab_popup_object_attributes_objects::setting_object_attributes_glue_create(Xv_opaque owner)
{
	extern void		popup_object_attributes_glue_handler(Panel_item, int, Event *);
	Xv_opaque	obj;
	
	obj = xv_create(owner, PANEL_TOGGLE, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 16,
		XV_Y, 72,
		XV_WIDTH, 116,
		XV_HEIGHT, 40,
		PANEL_VALUE_X, 16,
		PANEL_VALUE_Y, 89,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 1,			
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Glue:",
		PANEL_NOTIFY_PROC, popup_object_attributes_glue_handler,
		PANEL_CHOICE_STRINGS,
			"Yes",
			"No",
			0,
		NULL);
	return obj;
}

//
// Create object `setting_object_attributes_handles' in the specified instance.

//
Xv_opaque
geolab_popup_object_attributes_objects::setting_object_attributes_handles_create(Xv_opaque owner)
{
	extern void		popup_object_attributes_handles_handler(Panel_item, int, Event *);
	Xv_opaque	obj;
	
	obj = xv_create(owner, PANEL_TOGGLE, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 16,
		XV_Y, 124,
		XV_WIDTH, 117,
		XV_HEIGHT, 40,
		PANEL_VALUE_X, 16,
		PANEL_VALUE_Y, 141,
		PANEL_LAYOUT, PANEL_VERTICAL,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_VALUE, 1,			
		PANEL_CHOICE_NROWS, 1,
		PANEL_LABEL_STRING, "Handles:",
		PANEL_NOTIFY_PROC, popup_object_attributes_handles_handler,
		PANEL_CHOICE_STRINGS,
			"On",
			"Off",
			0,
		NULL);
	return obj;
}

//
// Create object `setting_object_attributes_foreground_color' in the specified instance.

//
Xv_opaque
geolab_popup_object_attributes_objects::setting_object_attributes_foreground_color_create(Xv_opaque owner)
{
	extern void		popup_object_attributes_foreground_handler(Panel_item, int, Event *);
	Xv_opaque	obj;
	Xv_opaque		setting_object_attributes_foreground_color_image0;
	static unsigned short	setting_object_attributes_foreground_color_bits0[] = {
#include "ICONS/color.icon"
	};
	
	setting_object_attributes_foreground_color_image0 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_object_attributes_foreground_color_bits0,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 16,
		XV_HEIGHT, 16,
		NULL);
	obj = xv_create(owner, PANEL_CHOICE, PANEL_DISPLAY_LEVEL, PANEL_CURRENT,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 167,
		XV_Y, 120,
		XV_WIDTH, 175,
		XV_HEIGHT, 26,
		PANEL_VALUE_X, 296,
		PANEL_VALUE_Y, 120,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_CHOICE_NCOLS, 4,
		PANEL_LABEL_STRING, "Foreground Color:",
		PANEL_NOTIFY_PROC, popup_object_attributes_foreground_handler,
		PANEL_CHOICE_IMAGES,
			setting_object_attributes_foreground_color_image0,
			setting_object_attributes_foreground_color_image0,
			setting_object_attributes_foreground_color_image0,
			setting_object_attributes_foreground_color_image0,
			setting_object_attributes_foreground_color_image0,
			setting_object_attributes_foreground_color_image0,
			setting_object_attributes_foreground_color_image0,
			setting_object_attributes_foreground_color_image0,
			setting_object_attributes_foreground_color_image0,
			setting_object_attributes_foreground_color_image0,
			setting_object_attributes_foreground_color_image0,
			setting_object_attributes_foreground_color_image0,
			setting_object_attributes_foreground_color_image0,
			setting_object_attributes_foreground_color_image0,
			setting_object_attributes_foreground_color_image0,
			setting_object_attributes_foreground_color_image0,
			0,
		PANEL_CHOICE_COLOR,  0, gcm_color_index("Black"),
		PANEL_CHOICE_COLOR,  1, gcm_color_index("Gray"),
		PANEL_CHOICE_COLOR,  2, gcm_color_index("Cyan"),
		PANEL_CHOICE_COLOR,  3, gcm_color_index("Blue"),
		PANEL_CHOICE_COLOR,  4, gcm_color_index("Sienna"),
		PANEL_CHOICE_COLOR,  5, gcm_color_index("Red"),
		PANEL_CHOICE_COLOR,  6, gcm_color_index("Magenta"),
		PANEL_CHOICE_COLOR,  7, gcm_color_index("Yellow"),
		PANEL_CHOICE_COLOR,  8, gcm_color_index("Orange"),
		PANEL_CHOICE_COLOR,  9, gcm_color_index("Medium Sea Green"),
		PANEL_CHOICE_COLOR,  10, gcm_color_index("Dark Green"),
		PANEL_CHOICE_COLOR,  11, gcm_color_index("Green"),
		PANEL_CHOICE_COLOR,  12, gcm_color_index("Goldenrod"),
		PANEL_CHOICE_COLOR,  13, gcm_color_index("Gold"),
		PANEL_CHOICE_COLOR,  14, gcm_color_index("Khaki"),
		PANEL_CHOICE_COLOR,  15, gcm_color_index("White"),
		// Edited by hand....
		PANEL_VALUE, 0,
		PANEL_INACTIVE, (SCREEN_DEPTH==1),			
		NULL);
	return obj;
}


//
// Create object `setting_object_attributes_background_color' in the specified instance.

//
Xv_opaque
geolab_popup_object_attributes_objects::setting_object_attributes_background_color_create(Xv_opaque owner)
{
	extern void		popup_object_attributes_background_handler(Panel_item, int, Event *);
	Xv_opaque	obj;
	Xv_opaque		setting_object_attributes_background_color_image0;
	static unsigned short	setting_object_attributes_background_color_bits0[] = {
#include "ICONS/color.icon"
	};
	
	setting_object_attributes_background_color_image0 = xv_create(XV_NULL, SERVER_IMAGE,
		SERVER_IMAGE_BITS, setting_object_attributes_background_color_bits0,
		SERVER_IMAGE_DEPTH, 1,
		XV_WIDTH, 16,
		XV_HEIGHT, 16,
		NULL);
	obj = xv_create(owner, PANEL_CHOICE, PANEL_DISPLAY_LEVEL, PANEL_CURRENT,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 164,
		XV_Y, 152,
		XV_WIDTH, 178,
		XV_HEIGHT, 26,
		PANEL_VALUE_X, 296,
		PANEL_VALUE_Y, 152,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_CHOICE_NCOLS, 4,
		PANEL_LABEL_STRING, "Background Color:",
		PANEL_NOTIFY_PROC, popup_object_attributes_background_handler,
		PANEL_CHOICE_IMAGES,
			setting_object_attributes_background_color_image0,
			setting_object_attributes_background_color_image0,
			setting_object_attributes_background_color_image0,
			setting_object_attributes_background_color_image0,
			setting_object_attributes_background_color_image0,
			setting_object_attributes_background_color_image0,
			setting_object_attributes_background_color_image0,
			setting_object_attributes_background_color_image0,
			setting_object_attributes_background_color_image0,
			setting_object_attributes_background_color_image0,
			setting_object_attributes_background_color_image0,
			setting_object_attributes_background_color_image0,
			setting_object_attributes_background_color_image0,
			setting_object_attributes_background_color_image0,
			setting_object_attributes_background_color_image0,
			setting_object_attributes_background_color_image0,
			0,
		PANEL_CHOICE_COLOR,  0, gcm_color_index("Black"),
		PANEL_CHOICE_COLOR,  1, gcm_color_index("Gray"),
		PANEL_CHOICE_COLOR,  2, gcm_color_index("Cyan"),
		PANEL_CHOICE_COLOR,  3, gcm_color_index("Blue"),
		PANEL_CHOICE_COLOR,  4, gcm_color_index("Sienna"),
		PANEL_CHOICE_COLOR,  5, gcm_color_index("Red"),
		PANEL_CHOICE_COLOR,  6, gcm_color_index("Magenta"),
		PANEL_CHOICE_COLOR,  7, gcm_color_index("Yellow"),
		PANEL_CHOICE_COLOR,  8, gcm_color_index("Orange"),
		PANEL_CHOICE_COLOR,  9, gcm_color_index("Medium Sea Green"),
		PANEL_CHOICE_COLOR,  10, gcm_color_index("Dark Green"),
		PANEL_CHOICE_COLOR,  11, gcm_color_index("Green"),
		PANEL_CHOICE_COLOR,  12, gcm_color_index("Goldenrod"),
		PANEL_CHOICE_COLOR,  13, gcm_color_index("Gold"),
		PANEL_CHOICE_COLOR,  14, gcm_color_index("Khaki"),
		PANEL_CHOICE_COLOR,  15, gcm_color_index("White"),				// Edited by hand....
		PANEL_VALUE, 15,
		PANEL_INACTIVE, (SCREEN_DEPTH==1),
		NULL);	
	return obj;
}


//
// Create object `message_object_attributes' in the specified instance.

//
Xv_opaque
geolab_popup_object_attributes_objects::message_object_attributes_create(Xv_opaque owner)
{
	Xv_opaque	obj;
	
	obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 120,
		XV_Y, 196,
		XV_WIDTH, 134,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Selected Attributes:",
		PANEL_LABEL_BOLD, TRUE,
		NULL);
	return obj;
}

//
// Create object `setting_object_attributes_increment' in the specified instance.

//
Xv_opaque
geolab_popup_object_attributes_objects::setting_object_attributes_increment_create(Xv_opaque owner)
{
	extern void		popup_object_attributes_increment_handler(Panel_item, int, Event *);
	Xv_opaque	obj;
	
	obj = xv_create(owner, PANEL_TOGGLE, PANEL_FEEDBACK, PANEL_MARKED,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 180,
		XV_Y, 220,
		XV_WIDTH, 51,
		XV_HEIGHT, 23,
		PANEL_VALUE_X, 189,
		PANEL_VALUE_Y, 220,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_CHOICE_NROWS, 1,
		PANEL_NOTIFY_PROC, popup_object_attributes_increment_handler,
		PANEL_CHOICE_STRING, 0, "  ",
		NULL);
	return obj;
}

//
// Create object `textfield_object_attributes_string' in the specified instance.

//
Xv_opaque
geolab_popup_object_attributes_objects::textfield_object_attributes_string_create(Xv_opaque owner)
{
	extern Panel_setting	popup_object_attributes_string_textfield_handler(Panel_item, Event *);
	Xv_opaque	obj;
	
	obj = xv_create(owner, PANEL_TEXT,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 12,
		XV_Y, 224,
		XV_WIDTH, 157,
		XV_HEIGHT, 15,
		PANEL_LABEL_STRING, "String:",
		PANEL_VALUE_X, 65,
		PANEL_VALUE_Y, 224,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_VALUE_DISPLAY_LENGTH, 13,
		PANEL_VALUE_STORED_LENGTH, 80,
		PANEL_READ_ONLY, FALSE,
		PANEL_NOTIFY_PROC, popup_object_attributes_string_textfield_handler,
		NULL);
	return obj;
}

//
// Create object `textfield_object_attributes_increment' in the specified instance.

//
Xv_opaque
geolab_popup_object_attributes_objects::textfield_object_attributes_increment_create(Xv_opaque owner)
{
	extern Panel_setting	popup_object_attributes_increment_textfield_handler(Panel_item, Event *);
	Xv_opaque	obj;
	
	obj = xv_create(owner, PANEL_NUMERIC_TEXT,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 208,
		XV_Y, 224,
		XV_WIDTH, 152,
		XV_HEIGHT, 15,
		PANEL_LABEL_STRING, "increment",
		PANEL_VALUE_X, 287,
		PANEL_VALUE_Y, 224,
		PANEL_LAYOUT, PANEL_HORIZONTAL,
		PANEL_VALUE_DISPLAY_LENGTH, 2,
		PANEL_VALUE_STORED_LENGTH, 2,
		PANEL_MAX_VALUE, 100,
		PANEL_MIN_VALUE, 0,
		PANEL_READ_ONLY, FALSE,
		PANEL_NOTIFY_PROC, popup_object_attributes_increment_textfield_handler,
		// Edited by hand....
		PANEL_INACTIVE, TRUE,			
		NULL);
	return obj;
}

//
// Create object `button_object_attributes_apply_to_selected' in the specified instance.

//
Xv_opaque
geolab_popup_object_attributes_objects::button_object_attributes_apply_to_selected_create(Xv_opaque owner)
{
	extern void		popup_object_attributes_apply_handler(Panel_item, Event *);
	Xv_opaque	obj;
	
	obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 64,
		XV_Y, 264,
		XV_WIDTH, 124,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "Apply to Selected",
		PANEL_NOTIFY_PROC, popup_object_attributes_apply_handler,
		NULL);
	return obj;
}

//
// Create object `button_object_attributes_set_default' in the specified instance.

//
Xv_opaque
geolab_popup_object_attributes_objects::button_object_attributes_set_default_create(Xv_opaque owner)
{
	extern void		popup_object_attributes_set_default_handler(Panel_item, Event *);
	Xv_opaque	obj;
	
	obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 212,
		XV_Y, 264,
		XV_WIDTH, 86,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "Set Default",
		PANEL_NOTIFY_PROC, popup_object_attributes_set_default_handler,
		NULL);
	return obj;
}

//
// Initialize an instance of object `popup_compose'.
//
void
geolab_popup_compose_objects::objects_initialize(Xv_opaque owner) {
    popup_compose = popup_compose_create(owner);
    controls_compose = controls_compose_create(popup_compose);
    button_compose_start = button_compose_start_create(controls_compose);
    button_compose_stop = button_compose_stop_create(controls_compose);
    button_compose_play = button_compose_play_create(controls_compose);
    const_message_compose_1 = const_message_compose_1_create(controls_compose);
    const_message_compose_2 = const_message_compose_2_create(controls_compose);
    const_message_compose_3 = const_message_compose_3_create(controls_compose);
    const_message_compose_4 = const_message_compose_4_create(controls_compose);
}

//
// Create object `popup_compose' in the specified instance.
//
Xv_opaque
geolab_popup_compose_objects::popup_compose_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, FRAME_CMD,
		XV_KEY_DATA, INSTANCE, this,
		XV_WIDTH, 400,
		XV_HEIGHT, 171,
		XV_LABEL, "GeoLab: Compose",
		XV_SHOW, FALSE,
		FRAME_SHOW_FOOTER, FALSE,
		FRAME_SHOW_RESIZE_CORNER, FALSE,
		FRAME_CMD_PUSHPIN_IN, TRUE,
		NULL);
    xv_set(xv_get(obj, FRAME_CMD_PANEL), WIN_SHOW, FALSE, NULL);
    return obj;
}

//
// Create object `controls_compose' in the specified instance.
//
Xv_opaque
geolab_popup_compose_objects::controls_compose_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 0,
		XV_Y, 0,
		XV_WIDTH, WIN_EXTEND_TO_EDGE,
		XV_HEIGHT, WIN_EXTEND_TO_EDGE,
		WIN_BORDER, FALSE,
		NULL);
    return obj;
}

//
// Create object `button_compose_start' in the specified instance.
//
Xv_opaque
geolab_popup_compose_objects::button_compose_start_create(Xv_opaque owner) {
    extern void popup_compose_start_handler(Panel_item, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 32,
		XV_Y, 24,
		XV_WIDTH, 71,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "   Start   ",
		PANEL_NOTIFY_PROC, popup_compose_start_handler,
		NULL);
    return obj;
}

//
// Create object `button_compose_stop' in the specified instance.
//
Xv_opaque
geolab_popup_compose_objects::button_compose_stop_create(Xv_opaque owner) {
    extern void popup_compose_stop_handler(Panel_item, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 116,
		XV_Y, 24,
		XV_WIDTH, 69,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "   Stop   ",
		PANEL_NOTIFY_PROC, popup_compose_stop_handler,
		NULL);
    return obj;
}

//
// Create object `button_compose_play' in the specified instance.
//
Xv_opaque
geolab_popup_compose_objects::button_compose_play_create(Xv_opaque owner) {
    extern void popup_compose_play_handler(Panel_item, Event *);
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_BUTTON,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 200,
		XV_Y, 24,
		XV_WIDTH, 68,
		XV_HEIGHT, 19,
		PANEL_LABEL_STRING, "   Play   ",
		PANEL_NOTIFY_PROC, popup_compose_play_handler,
		NULL);
    return obj;
}

//
// Create object `const_message_compose_1' in the specified instance.
//
Xv_opaque
geolab_popup_compose_objects::const_message_compose_1_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 35,
		XV_Y, 64,
		XV_WIDTH, 293,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "This is an experimental version of the Compose facility.",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_compose_2' in the specified instance.
//
Xv_opaque
geolab_popup_compose_objects::const_message_compose_2_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 35,
		XV_Y, 84,
		XV_WIDTH, 300,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Press <start> to initiate a composition (by calling",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_compose_3' in the specified instance.
//
Xv_opaque
geolab_popup_compose_objects::const_message_compose_3_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 35,
		XV_Y, 104,
		XV_WIDTH, 345,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "algorithms from the menu). Press <stop> when finished.",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}

//
// Create object `const_message_compose_4' in the specified instance.
//
Xv_opaque
geolab_popup_compose_objects::const_message_compose_4_create(Xv_opaque owner) {
    Xv_opaque obj;
	
    obj = xv_create(owner, PANEL_MESSAGE,
		XV_KEY_DATA, INSTANCE, this,
		XV_X, 35,
		XV_Y, 124,
		XV_WIDTH, 303,
		XV_HEIGHT, 13,
		PANEL_LABEL_STRING, "Use <play> to run the composition over data sets.",
		PANEL_LABEL_BOLD, FALSE,
		NULL);
    return obj;
}




