/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * instantiation.C - modulo para regeneracao de objetos a partir de seus tipos.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    26.09.91  18:25 	Welson R. Jacometti  	Arquivo criado
 *
 */

/*
 * IMPORTANTE:
 *
 *    O disparo de algoritmos externos e' feito unica e exclusivamente em
 *  dois modulos: algorithms.C(.h) e instantiation.C(.h) 
 *    Nestes modulos, antes da referencia ao algoritmo externo, DEVE ser
 *  feita a mudanca da rotina de tratamento de sinais, para que o usuario
 *  seja corretamente informado sobre problemas decorrentes a) do ambiente
 *  ou b) de um determinado algoritmo externo.
 *
 */
 
#include "global.h"
#include "utilities.h"
#include "basic.h"
#include "List.h"
#include "String.h"
#include "World.h"
#include "geolab_ui.h"
#include "Algorithm.h"
#include "algorithms.h"
#include "instantiation.h"
#include "trapsignal.h"
#include "Point2D.h"
#include "Segment2D.h"
#include "Ray2D.h"
#include "Line2D.h"
#include "Rectangle.h"
#include "Square.h"
#include "Circle.h"
#include "Ellipse.h"
#include "Polygon.h"

Obj_ptr geolab_instantiation(ObjType t) {
    switch(t) {
      case Obj_t: 		return new Obj();
      case Point2D_t: 		return new Point2D();
      case Segment2D_t:		return new Segment2D();
      case Ray2D_t:		return new Ray2D();
      case Line2D_t:		return new Line2D();
      case Rectangle_t:		return new Rectangle();
      case Square_t:		return new Square();
      case Circle_t:		return new Circle();
      case Ellipse_t:		return new Ellipse();
      case Polygon_t:		return new Polygon();
    }
    
    warning("instantiation.C", "geolab_instantiation", "don't know an object of the requested type");
    return 0;
}

Obj_ptr external_instantiation(ObjType t, char *function, char *module) {
    void* mhandle;
    String composite_name, strerr;
    address f;    
    Obj_ptr res;

    mhandle = open_dynamic_module(module);    
    composite_name = compose_external_algorithm_name(function);
    res = 0;
        
    if (!(f = dlsym(mhandle, composite_name.value()))) {
      strerr = String("can't find function ") + composite_name;
      warning("instantiation.C", "external_instantiation", strerr.value());
    }
    else {
      Algorithm *a = ((link_function)f)();
      set_external_signal_handler();
      res = a->instantiation(t);
      set_internal_signal_handler();
      delete a;
    }

    dlclose(mhandle);

    return res;
}


