/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * resize.C - modulo de tratamento de eventos no modo de redimensionamento
 *            de objetos.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    12.12.91  08:45 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "resize.h"
#include "Obj.h"
#include "color.h"
#include "objects.h"

#define NO_IF_GLUED true

bool resize_events(World *world, Xv_Window, Event *event) {
    static GraphicObj *current_obj = 0;
    static bool was_selected = false;
    static Int_Mode mode;
    double X = world->virtual_xcoord(event_x(event));
    double Y = world->virtual_ycoord(event_y(event));

    if (event_action(event) == ACTION_SELECT)
      mode = LeftButton;
    if (event_action(event) == ACTION_ADJUST)
      mode = MiddleButton;
      
    switch (event_action(event)) {
      case ACTION_SELECT:
      case ACTION_ADJUST:
        if (event_is_down(event)) {
          if (!current_obj) {
            if (current_obj = objects_object_selected_by_handle_at(X, Y, NO_IF_GLUED)) {
              unsigned long fg, bg;
              current_obj->get_color(fg, bg);
              current_obj->set_color(world, get_pixel_from_value(WHITE), get_pixel_from_value(WHITE));
              current_obj->draw(world, CopyMode);
              objects_repaint_all_but(current_obj, world);
              current_obj->set_color(world, fg, bg);
              if (was_selected = current_obj->selected())
                current_obj->unselect();
              current_obj->resize_start(world, X, Y, mode);
              current_obj->draw(world, XorMode);
            }
          }
        }
        else 
          if (current_obj) {
            current_obj->draw(world, XorMode);
            current_obj->resize_end(world, X, Y, mode);
            if (was_selected) {
              current_obj->select();
              was_selected = false;
            }
            current_obj->draw(world, CopyMode);
            current_obj = 0;
          }
        break;
        
      case LOC_DRAG:
        if (current_obj) {
          current_obj->draw(world, XorMode);
          current_obj->resize_change(world, X, Y, mode);
          current_obj->draw(world, XorMode);
        }
        break;
    }
    
    return false;
}






