/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * select.C - modulo para a selecao de objetos geometricos.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    08.10.91  21:25 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "select.h"
#include "algorithms.h"
#include "Rectangle.h"
#include "objects.h"
#include "color.h"
#include "attributes.h"

extern void show_popup_object_attributes(Attributes& attr);

static GraphicRectangle select_area;

// prototypes

bool double_click (Event *event);
long diftime (struct timeval t1,struct timeval t2);

// cast macro

#define area_values	((Rectangle *)select_area.obj)

bool select_events(World *world, Xv_Window, Event *event) {
    static bool done = false;
    
    double X = world->virtual_xcoord(event_x(event));
    double Y = world->virtual_ycoord(event_y(event));
    
    switch (event_action(event)) {
      case ACTION_SELECT:
        if (event_is_down(event)) {
	    if (double_click(event)) {
		GraphicObj *obj;
		if (obj=objects_object_selected_at(X, Y, false))
		  Obj_Attributes = obj->get_attributes();
		else
		  Obj_Attributes = Default_Obj_Attributes;
		show_popup_object_attributes(Obj_Attributes);
	    }
	    
	    done = objects_select_one_left_button(world, X, Y);
	    if (!done) {
		select_area.set_all(world, get_pixel_from_value(BLACK), 
				    get_pixel_from_value(WHITE),
				    0, 0, false);
		select_area.int_start(world, X, Y);
		select_area.draw(world, XorMode);
	    }
        }
	else {
	    if (!done) {
		select_area.draw(world, XorMode);
		select_area.int_end(world, X, Y);
		objects_select(world, area_values->P1().X(), area_values->P1().Y(), area_values->P2().X(), area_values->P2().Y(), false /* no flip */); 
			   }
		
	    if (objects_selected_empty())
	      enable_all_algorithms();
	    else
	      select_applyable_algorithms();
	}
	break;
        
      case ACTION_ADJUST:
        if (event_is_down(event)) {
          done = objects_select_one_middle_button(world, X, Y);
          if (!done) {
            select_area.set_all(world, get_pixel_from_value(BLACK), 
           			       get_pixel_from_value(WHITE),
          			       0, 0, false);
            select_area.int_start(world, X, Y);
            select_area.draw(world, XorMode);
          }
        }
        else {
          if (!done) {
            select_area.draw(world, XorMode);
            select_area.int_end(world, X, Y);
            objects_select(world, area_values->P1().X(), area_values->P1().Y(),
      			          area_values->P2().X(), area_values->P2().Y(), event_shift_is_down(event) /* flip when shift is pressed */);
          }
          
          if (objects_selected_empty())
            enable_all_algorithms();
          else
            select_applyable_algorithms();
        }
        break;
        
      case LOC_DRAG:
        if (!done) {
          select_area.draw(world, XorMode);
          select_area.int_change(world, X, Y);
          select_area.draw(world, XorMode);
        }
        break;
    }
    
    return false;
}


