/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 *
 * trapsignal.C - este modulo prove ferramentas basicas para a interceptacao
 *                de sinais (normalmente Segmentation Violation, Bus Error, etc)
 *                de forma a tornar mais facil o debug do ambiente e algoritmos,
 *                como tambem torna'-lo mais robusto.
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    26.09.91  18:25 	Welson R. Jacometti  	Arquivo criado
 *
 */

#include "trapsignal.h"
#include "String.h"
#include "geolab_ui.h"
#include "load_save.h"

char *sig_code(int sig) {
    switch(sig) {
      case SIGILL:
        return "illegal instruction";
      case SIGFPE:
        return "arithmetic exception";
      case SIGBUS:
        return "bus error";
      case SIGSEGV:
        return "segmentation violation";
      case SIGSYS:
        return "bad argument to system call";
      default:
        return "unknown exception";
    }
}

void gen_message_and_abort(char *where, int sig) {
    extern geolab_geolab_base_window_objects geolab_geolab_base_window;
    String strerr1, strerr2, strerr3;
    String home = getenv("HOME");
    
    if (!home.value())
      home = CORE_OBJECTS_DIR_TMP;
      
    (void)save(home.value(), CORE_OBJECTS_FILE);

    strerr1 = String("Fatal error ocurred (probably) in ") + String(where) + String(" code.");
    strerr2 = String("The system reported: '") + String(sig_code(sig)) + String("'");  
    strerr3 = String("Geometric objects were saved in ") + home + String("/") + String(CORE_OBJECTS_FILE);

    (void)notice_prompt(geolab_geolab_base_window.geolab_base_window, NULL, 
    			NOTICE_MESSAGE_STRINGS,		strerr1.value(),
    							strerr2.value(),
    							strerr3.value(),
    							"Since GeoLab cannot recover, it will be aborted and",
    							"a core dump genetated for later debugging.",
    							NULL,
			NOTICE_BUTTON_YES,		"Continue, and bye bye!",
			NULL);

    abort();
}

void internal_signal_handler(int sig, int, sigcon *, char *) {
    gen_message_and_abort("INTERNAL", sig);
}

void external_signal_handler(int sig, int, sigcon *, char *) {
    gen_message_and_abort("EXTERNAL", sig);
}

void set_internal_signal_handler() {
    signal(SIGILL , (SIG_PF)internal_signal_handler);   	
    signal(SIGFPE , (SIG_PF)internal_signal_handler);   	
    signal(SIGBUS , (SIG_PF)internal_signal_handler);   	
    signal(SIGSEGV, (SIG_PF)internal_signal_handler);   	
    signal(SIGSYS , (SIG_PF)internal_signal_handler);   	
}

void set_external_signal_handler() {
    signal(SIGILL , (SIG_PF)external_signal_handler);   	
    signal(SIGFPE , (SIG_PF)external_signal_handler);   	
    signal(SIGBUS , (SIG_PF)external_signal_handler);   	
    signal(SIGSEGV, (SIG_PF)external_signal_handler);   	
    signal(SIGSYS , (SIG_PF)external_signal_handler);   	
}




