/*
    Copyright (C) 1992-1994
                    Welson R. Jacometti

    All rights reserved.

    This program is  free software; you can redistribute unmodified
    versions of it as long as you do not  profit from that, neither
    by selling it nor  by   including it  with other products   for
    sale.  This copyright  notice  must  remain in all  distributed
    copies.

    You are  allowed to modify your  personal copy  of this program
    (as long as you do not distribute any changed versions).

    You may,  of course, distribute  along with it, your own source
    code or libraries of algorithms that run under GeoLab.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    If you must contact the authors, you may write to:
                     geolab-request@dcc.unicamp.br
*/

/*
 * voronoi.h - implementacao do algoritmo para calculo da triangulacao de
 *             Delaunay e do diagrama de Voronoi atraves do algoritmo
 *             apresentado por Guibas & Stolfi em "Primitives for the
 *             Manipulation of General Subdivisions and the Computation
 *             of Voronoi Diagrams", ACM TG 85
 *
 *
 *    	Data	Hora  	Nome     		Modificacao/Motivo
 *
 *    20.02.92  13:40 	Welson R. Jacometti  	Arquivo criado
 *
 */

#ifndef _VORONOI_DEFINED
#define _VORONOI_DEFINED

#include <iostream.h>
#include "External.h"
#include "Algorithm.h"
#include "QuadEdge.h"
#include "Point2D.h"
#include "Array.h"

#ifndef _Point2D_ptr_list
#define _Point2D_ptr_list
declare(list,Point2D_ptr)
#endif ~_Point2D_ptr_list;

#ifndef _Point2D_ptr_array
#define _Point2D_ptr_array
declare(array,Point2D_ptr)
#endif ~_Point2D_ptr_array;

class TwoEdges {
  public:
    TwoEdges(QEdge_ptr e1, QEdge_ptr e2) : le(e1), re(e2) {}
    QEdge_ptr le, re;
};

void setup_voronoi_geometry(QuadEdge *q, bool FARTHEST = false);
TwoEdges main_algorithm(array(Point2D_ptr)&, int, int, bool FARTHEST = false, World *w = 0, int animation_depth = 0, int reference_depth = 0);

QuadEdge *calc_triangulation(list(Point2D_ptr)&, bool FARTHEST = false, World *w = 0, int animation_depth = 0, int reference_depth = 0);
QuadEdge *calc_diagram(list(Point2D_ptr)&, bool FARTHEST = false, World *w = 0, int animation_depth = 0, int reference_depth = 0);

QuadEdge *delaunay_triangulation(list(Point2D_ptr)&, World *w = 0, int animation_depth = 0, int reference_depth = 0);
QuadEdge *voronoi_diagram(list(Point2D_ptr)&, World *w = 0, int animation_depth = 0, int reference_depth = 0);
QuadEdge *farthest_neighbor_triangulation(list(Point2D_ptr)&, World *w = 0, int animation_depth = 0, int reference_depth = 0);
QuadEdge *farthest_neighbor_voronoi_diagram(list(Point2D_ptr)&, World *w = 0, int animation_depth = 0, int reference_depth = 0);

// Modelagem dos algoritmos acima em termos do ambiente GeoLab

class GenericVoronoiTools : public Algorithm {
  public:
    bool animation() { return true; }
    bool is_able_to_handle(const SelectedObjects &, const GraphicObjects &);
    Obj* instantiation (ObjType t) {
        if (t == QuadEdge_t)
          return new QuadEdge();
        else
          return 0;
    }
};

class VoronoiDiagram : public GenericVoronoiTools {
  public:
    ResultObjects binder(const SelectedObjects &s, const GraphicObjects &obj, World*, int, int);
};

class DelaunayTriangulation : public GenericVoronoiTools {
  public:
    ResultObjects binder(const SelectedObjects &s, const GraphicObjects &obj, World*, int, int);
};

class FNVoronoiDiagram : public GenericVoronoiTools {
  public:
    ResultObjects binder(const SelectedObjects &s, const GraphicObjects &obj, World*, int, int);
};

class FNTriangulation : public GenericVoronoiTools {
  public:
    ResultObjects binder(const SelectedObjects &s, const GraphicObjects &obj, World*, int, int);
};

Algorithm* VoronoiDiagram_link();
Algorithm* DelaunayTriangulation_link();
Algorithm* FNVoronoiDiagram_link();
Algorithm* FNTriangulation_link();

#endif ~_VORONOI_DEFINED;









